@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #ifndef ZABBIX_DB_H
 #define ZABBIX_DB_H
 
-#include "common.h"
 #include "zbxalgo.h"
 #include "zbxdb.h"
 #include "dbschema.h"
@@ -64,8 +63,6 @@ typedef enum
 }
 zbx_graph_item_type;
 
-struct	_DC_TRIGGER;
-
 #define ZBX_DB_CONNECT_NORMAL	0
 #define ZBX_DB_CONNECT_EXIT	1
 #define ZBX_DB_CONNECT_ONCE	2
@@ -94,8 +91,6 @@ struct	_DC_TRIGGER;
 #define HOST_HOST_LEN			MAX_ZBX_HOSTNAME_LEN
 #define HOST_HOST_LEN_MAX		(HOST_HOST_LEN + 1)
 #define HOST_NAME_LEN			128
-#define HOST_ERROR_LEN			2048
-#define HOST_ERROR_LEN_MAX		(HOST_ERROR_LEN + 1)
 #define HOST_IPMI_USERNAME_LEN		16
 #define HOST_IPMI_USERNAME_LEN_MAX	(HOST_IPMI_USERNAME_LEN + 1)
 #define HOST_IPMI_PASSWORD_LEN		20
@@ -111,6 +106,8 @@ struct	_DC_TRIGGER;
 #define INTERFACE_ADDR_LEN_MAX		(INTERFACE_ADDR_LEN + 1)
 #define INTERFACE_PORT_LEN		64
 #define INTERFACE_PORT_LEN_MAX		(INTERFACE_PORT_LEN + 1)
+#define INTERFACE_ERROR_LEN		2048
+#define INTERFACE_ERROR_LEN_MAX		(INTERFACE_ERROR_LEN + 1)
 
 #define ITEM_NAME_LEN			255
 #define ITEM_KEY_LEN			2048
@@ -179,6 +176,8 @@ struct	_DC_TRIGGER;
 #define ITEM_PARAMETER_NAME_LEN		255
 #define ITEM_PARAMETER_VALUE_LEN	2048
 
+#define ITEM_TAG_FIELD_LEN		255
+
 #define HISTORY_STR_VALUE_LEN		255
 #define HISTORY_TEXT_VALUE_LEN		65535
 #define HISTORY_LOG_VALUE_LEN		65535
@@ -202,12 +201,15 @@ struct	_DC_TRIGGER;
 
 #define PROXY_DHISTORY_VALUE_LEN	255
 
-#define ITEM_PREPROC_PARAMS_LEN		255
+#define ITEM_PREPROC_PARAMS_LEN		65535
 
 #define EVENT_NAME_LEN			2048
 
+#define FUNCTION_NAME_LEN		12
 #define FUNCTION_PARAM_LEN		255
 
+#define REPORT_ERROR_LEN		2048
+
 #define ZBX_SQL_ITEM_FIELDS	"i.itemid,i.key_,h.host,i.type,i.history,i.hostid,i.value_type,i.delta,"	\
 				"i.units,i.multiplier,i.formula,i.state,i.valuemapid,i.trends,i.data_type"
 #define ZBX_SQL_ITEM_TABLES	"hosts h,items i"
@@ -237,7 +239,6 @@ struct	_DC_TRIGGER;
 			'\0' != *str ? "<>'" : "",		\
 			'\0' != *str ? str   : " is not null",	\
 			'\0' != *str ? "'"   : ""
-
 #else
 #	define	DBbegin_multiple_update(sql, sql_alloc, sql_offset)	do {} while (0)
 #	define	DBend_multiple_update(sql, sql_alloc, sql_offset)	do {} while (0)
@@ -306,6 +307,11 @@ typedef struct
 }
 DB_DSERVICE;
 
+#define ZBX_DB_TRIGGER_EVAL_NONE			0x0000
+#define ZBX_DB_TRIGGER_EVAL_EXPRESSION			0x0001
+#define ZBX_DB_TRIGGER_EVAL_EXPRESSION_USERMACRO	0x0002
+#define ZBX_DB_TRIGGER_EVAL_RECOVERY_EXPRESSION		0x0004
+
 typedef struct
 {
 	zbx_uint64_t	triggerid;
@@ -322,9 +328,23 @@ typedef struct
 	unsigned char	type;
 	unsigned char	recovery_mode;
 	unsigned char	correlation_mode;
+
+	/* temporary trigger cache for related data */
+	void		*cache;
 }
 DB_TRIGGER;
 
+typedef struct
+{
+	zbx_uint64_t		serviceid;
+	char			*name;
+	char			*description;
+	zbx_vector_uint64_t	eventids;
+	zbx_vector_ptr_t	events;
+	zbx_vector_tags_t	service_tags;
+}
+DB_SERVICE;
+
 typedef struct
 {
 	zbx_uint64_t		eventid;
@@ -350,11 +370,10 @@ typedef struct
 }
 DB_EVENT;
 
-typedef struct
+typedef struct DB_MEDIATYPE
 {
 	zbx_uint64_t		mediatypeid;
 	zbx_media_type_t	type;
-	char			*description;
 	char			*smtp_server;
 	char			*smtp_helo;
 	char			*smtp_email;
@@ -363,14 +382,25 @@ typedef struct
 	char			*gsm_modem;
 	char			*username;
 	char			*passwd;
+	char			*script;
+	char			*attempt_interval;
+	char			*timeout;
 	unsigned short		smtp_port;
 	unsigned char		smtp_security;
 	unsigned char		smtp_verify_peer;
 	unsigned char		smtp_verify_host;
 	unsigned char		smtp_authentication;
+	unsigned char		content_type;
+	int			maxsessions;
+	int			maxattempts;
 }
 DB_MEDIATYPE;
 
+void 	zbx_db_mediatype_clean(DB_MEDIATYPE *mt);
+void	zbx_serialize_mediatype(unsigned char **data, zbx_uint32_t *data_alloc, zbx_uint32_t *data_offset,
+		const DB_MEDIATYPE *mt);
+zbx_uint32_t	zbx_deserialize_mediatype(const unsigned char *data, DB_MEDIATYPE *mt);
+
 typedef struct
 {
 	zbx_uint64_t	alertid;
@@ -439,6 +469,8 @@ typedef struct
 	zbx_uint64_t		eventid;
 	zbx_uint64_t		r_eventid;
 	zbx_uint64_t		acknowledgeid;
+	zbx_uint64_t		servicealarmid;
+	zbx_uint64_t		serviceid;
 	int			nextcheck;
 	int			esc_step;
 	zbx_escalation_status_t	status;
@@ -454,6 +486,7 @@ typedef struct
 	unsigned char	pause_suppressed;
 	unsigned char	recovery;
 	unsigned char	status;
+	unsigned char	notify_if_canceled;
 }
 DB_ACTION;
 
@@ -469,6 +502,14 @@ typedef struct
 }
 DB_ACKNOWLEDGE;
 
+typedef struct
+{
+	zbx_uint64_t	service_alarmid;
+	int		value;
+	int		clock;
+}
+zbx_service_alarm_t;
+
 int	DBinit(char **error);
 void	DBdeinit(void);
 
@@ -502,7 +543,9 @@ const ZBX_FIELD	*DBget_field(const ZBX_TABLE *table, const char *fieldname);
 #define DBget_maxid(table)	DBget_maxid_num(table, 1)
 zbx_uint64_t	DBget_maxid_num(const char *tablename, int num);
 
-void	DBcheck_capabilities(void);
+void	DBextract_version_info(struct zbx_db_version_info_t *version_info);
+void	DBflush_version_requirements(const char *version);
+int	DBcheck_capabilities(zbx_uint32_t db_version);
 
 #ifdef HAVE_POSTGRESQL
 char	*zbx_db_get_schema_esc(void);
@@ -514,8 +557,6 @@ char	*zbx_db_get_schema_esc(void);
  *                                                                            *
  * Purpose: represent graph item data                                         *
  *                                                                            *
- * Author: Eugene Grigorjev                                                   *
- *                                                                            *
  ******************************************************************************/
 typedef struct
 {
@@ -570,7 +611,6 @@ char	*DBdyn_escape_string(const char *src);
 char	*DBdyn_escape_string_len(const char *src, size_t length);
 char	*DBdyn_escape_like_pattern(const char *src);
 
-zbx_uint64_t	DBadd_host(char *server, int port, int status, int useip, char *ip, int disable_until, int available);
 int	DBadd_templates_to_host(int hostid, int host_templateid);
 
 int	DBadd_template_linkage(int hostid, int templateid, int items, int triggers, int graphs);
@@ -581,19 +621,15 @@ void	DBdelete_sysmaps_hosts_by_hostid(zbx_uint64_t hostid);
 int	DBadd_graph_item_to_linked_hosts(int gitemid, int hostid);
 
 int	DBcopy_template_elements(zbx_uint64_t hostid, zbx_vector_uint64_t *lnk_templateids, char **error);
-int	DBdelete_template_elements(zbx_uint64_t hostid, zbx_vector_uint64_t *del_templateids, char **error);
+int	DBdelete_template_elements(zbx_uint64_t hostid, const char *hostname, zbx_vector_uint64_t *del_templateids,
+		char **error);
 
 void	DBdelete_items(zbx_vector_uint64_t *itemids);
 void	DBdelete_graphs(zbx_vector_uint64_t *graphids);
 void	DBdelete_triggers(zbx_vector_uint64_t *triggerids);
-void	DBdelete_hosts(zbx_vector_uint64_t *hostids);
-void	DBdelete_hosts_with_prototypes(zbx_vector_uint64_t *hostids);
 
-int	DBupdate_itservices(const zbx_vector_ptr_t *trigger_diff);
-int	DBremove_triggers_from_itservices(zbx_uint64_t *triggerids, int triggerids_num);
-
-int	zbx_create_itservices_lock(char **error);
-void	zbx_destroy_itservices_lock(void);
+void	DBdelete_hosts(const zbx_vector_uint64_t *hostids, const zbx_vector_str_t *hostnames);
+void	DBdelete_hosts_with_prototypes(const zbx_vector_uint64_t *hostids, const zbx_vector_str_t *hostnames);
 
 void	DBadd_condition_alloc(char **sql, size_t *sql_alloc, size_t *sql_offset, const char *fieldname,
 		const zbx_uint64_t *values, const int num);
@@ -605,7 +641,7 @@ int	zbx_check_user_permissions(const zbx_uint64_t *userid, const zbx_uint64_t *r
 const char	*zbx_host_string(zbx_uint64_t hostid);
 const char	*zbx_host_key_string(zbx_uint64_t itemid);
 const char	*zbx_user_string(zbx_uint64_t userid);
-int	DBget_user_names(zbx_uint64_t userid, char **alias, char **name, char **surname);
+int	DBget_user_names(zbx_uint64_t userid, char **username, char **name, char **surname);
 
 void	DBregister_host(zbx_uint64_t proxy_hostid, const char *host, const char *ip, const char *dns,
 		unsigned short port, unsigned int connection_type, const char *host_metadata, unsigned short flag,
@@ -632,12 +668,13 @@ typedef enum
 }
 zbx_conn_flags_t;
 
-zbx_uint64_t	DBadd_interface(zbx_uint64_t hostid, unsigned char type, unsigned char useip, const char *ip,
-		const char *dns, unsigned short port, zbx_conn_flags_t flags);
-void	DBadd_interface_snmp(const zbx_uint64_t interfaceid, const unsigned char version, const unsigned char bulk,
-		const char *community, const char *securityname, const unsigned char securitylevel,
-		const char *authpassphrase, const char *privpassphrase, const unsigned char authprotocol,
-		const unsigned char privprotocol, const char *contextname);
+zbx_uint64_t	DBadd_interface(zbx_uint64_t hostid, unsigned char type, unsigned char useip,
+		const char *ip, const char *dns, unsigned short port, zbx_conn_flags_t flags);
+void	DBadd_interface_snmp(const zbx_uint64_t interfaceid, const unsigned char version,
+		const unsigned char bulk, const char *community, const char *securityname,
+		const unsigned char securitylevel, const char *authpassphrase, const char *privpassphrase,
+		const unsigned char authprotocol, const unsigned char privprotocol, const char *contextname,
+		const zbx_uint64_t hostid);
 
 const char	*DBget_inventory_field(unsigned char inventory_link);
 
@@ -651,8 +688,11 @@ int	DBtable_exists(const char *table_name);
 int	DBfield_exists(const char *table_name, const char *field_name);
 #ifndef HAVE_SQLITE3
 int	DBindex_exists(const char *table_name, const char *index_name);
+int	DBpk_exists(const char *table_name);
 #endif
 
+int	DBprepare_multiple_query(const char *query, const char *field_name, zbx_vector_uint64_t *ids, char **sql,
+		size_t	*sql_alloc, size_t *sql_offset);
 int	DBexecute_multiple_query(const char *query, const char *field_name, zbx_vector_uint64_t *ids);
 int	DBlock_record(const char *table, zbx_uint64_t id, const char *add_field, zbx_uint64_t add_id);
 int	DBlock_records(const char *table, const zbx_vector_uint64_t *ids);
@@ -733,15 +773,32 @@ zbx_agent_availability_t;
 
 typedef struct
 {
-	zbx_uint64_t			hostid;
-
-	zbx_agent_availability_t	agents[ZBX_AGENT_MAX];
+	zbx_uint64_t			interfaceid;
+	zbx_agent_availability_t	agent;
+	/* ensure chronological order in case of flapping interface availability */
+	int				id;
 }
-zbx_host_availability_t;
+zbx_interface_availability_t;
 
-int	zbx_sql_add_host_availability(char **sql, size_t *sql_alloc, size_t *sql_offset,
-		const zbx_host_availability_t *ha);
+ZBX_PTR_VECTOR_DECL(availability_ptr, zbx_interface_availability_t *)
+
+typedef struct
+{
+	zbx_uint64_t		eventid;
+	int			clock;
+	int			ns;
+	int			value;
+	int			severity;
+
+	zbx_vector_ptr_t	tags;
+}
+zbx_event_t;
+
+void	zbx_db_update_interface_availabilities(const zbx_vector_availability_ptr_t *interface_availabilities);
 int	DBget_user_by_active_session(const char *sessionid, zbx_user_t *user);
+int	DBget_user_by_auth_token(const char *formatted_auth_token_hash, zbx_user_t *user);
+void	zbx_user_init(zbx_user_t *user);
+void	zbx_user_free(zbx_user_t *user);
 
 typedef struct
 {
@@ -768,7 +825,8 @@ void	zbx_db_get_events_by_eventids(zbx_vector_uint64_t *eventids, zbx_vector_ptr
 void	zbx_db_free_event(DB_EVENT *event);
 void	zbx_db_get_eventid_r_eventid_pairs(zbx_vector_uint64_t *eventids, zbx_vector_uint64_pair_t *event_pairs,
 		zbx_vector_uint64_t *r_eventids);
-
+void	zbx_db_trigger_get_expression(const DB_TRIGGER *trigger, char **expression);
+void	zbx_db_trigger_get_recovery_expression(const DB_TRIGGER *trigger, char **expression);
 void	zbx_db_trigger_clean(DB_TRIGGER *trigger);
 
 typedef struct
@@ -823,8 +881,11 @@ int	zbx_db_check_instanceid(void);
 typedef struct
 {
 	zbx_uint64_t	tagid;
+	char		*tag_orig;
 	char		*tag;
+	char		*value_orig;
 	char		*value;
+#define ZBX_FLAG_DB_TAG_UNSET			__UINT64_C(0x00000000)
 #define ZBX_FLAG_DB_TAG_UPDATE_TAG		__UINT64_C(0x00000001)
 #define ZBX_FLAG_DB_TAG_UPDATE_VALUE		__UINT64_C(0x00000002)
 #define ZBX_FLAG_DB_TAG_REMOVE			__UINT64_C(0x80000000)
@@ -833,10 +894,12 @@ typedef struct
 }
 zbx_db_tag_t;
 
-void	zbx_db_tag_free(zbx_db_tag_t *tag);
-int	zbx_db_tag_compare_func(const void *d1, const void *d2);
+zbx_db_tag_t	*zbx_db_tag_create(const char *tag_tag, const char *tag_value);
+void		zbx_db_tag_free(zbx_db_tag_t *tag);
+int		zbx_db_tag_compare_func(const void *d1, const void *d2);
+int		zbx_db_tag_compare_func_template(const void *d1, const void *d2);
 
-ZBX_PTR_VECTOR_DECL(db_tag_ptr, zbx_db_tag_t *);
+ZBX_PTR_VECTOR_DECL(db_tag_ptr, zbx_db_tag_t *)
 
 typedef enum
 {
@@ -871,6 +934,15 @@ void	zbx_lld_override_operation_free(zbx_lld_override_operation_t *override_oper
 void	zbx_load_lld_override_operations(const zbx_vector_uint64_t *overrideids, char **sql, size_t *sql_alloc,
 		zbx_vector_ptr_t *ops);
 
+#define ZBX_TIMEZONE_DEFAULT_VALUE	"default"
 
+void	zbx_db_trigger_get_all_functionids(const DB_TRIGGER *trigger, zbx_vector_uint64_t *functionids);
+void	zbx_db_trigger_get_functionids(const DB_TRIGGER *trigger, zbx_vector_uint64_t *functionids);
+int	zbx_db_trigger_get_all_hostids(const DB_TRIGGER *trigger, const zbx_vector_uint64_t **hostids);
+int	zbx_db_trigger_get_constant(const DB_TRIGGER *trigger, int index, char **out);
+int	zbx_db_trigger_get_itemid(const DB_TRIGGER *trigger, int index, zbx_uint64_t *itemid);
+void	zbx_db_trigger_get_itemids(const DB_TRIGGER *trigger, zbx_vector_uint64_t *itemids);
+
+int	DBselect_ids_names(const char *sql, zbx_vector_uint64_t *ids, zbx_vector_str_t *names);
 
 #endif
