@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,12 +20,11 @@
 #ifndef ZABBIX_DBCACHE_H
 #define ZABBIX_DBCACHE_H
 
+#include "sysinfo.h" //included for convenience
 #include "db.h"
 #include "comms.h"
-#include "sysinfo.h"
-#include "zbxalgo.h"
-#include "zbxjson.h"
 #include "memalloc.h"
+#include "zbxeval.h"
 
 #define ZBX_SYNC_DONE		0
 #define	ZBX_SYNC_MORE		1
@@ -36,7 +35,9 @@
 #define	ZBX_POLLER_TYPE_IPMI		2
 #define	ZBX_POLLER_TYPE_PINGER		3
 #define	ZBX_POLLER_TYPE_JAVA		4
-#define	ZBX_POLLER_TYPE_COUNT		5	/* number of poller types */
+#define	ZBX_POLLER_TYPE_HISTORY		5
+#define	ZBX_POLLER_TYPE_ODBC		6
+#define	ZBX_POLLER_TYPE_COUNT		7	/* number of poller types */
 
 #define MAX_JAVA_ITEMS		32
 #define MAX_SNMP_ITEMS		128
@@ -50,6 +51,10 @@
 
 #define ZBX_SNMPTRAP_LOGGING_ENABLED	1
 
+#define ZBX_IPC_SERVICE_CONFIG		"config"
+#define ZBX_IPC_CONFIG_RELOAD_REQUEST	1
+#define ZBX_IPC_CONFIG_RELOAD_RESPONSE	2
+
 extern int	CONFIG_TIMEOUT;
 
 extern zbx_uint64_t	CONFIG_CONF_CACHE_SIZE;
@@ -62,12 +67,12 @@ extern int	CONFIG_UNREACHABLE_POLLER_FORKS;
 extern int	CONFIG_IPMIPOLLER_FORKS;
 extern int	CONFIG_JAVAPOLLER_FORKS;
 extern int	CONFIG_PINGER_FORKS;
-extern int	CONFIG_UNAVAILABLE_DELAY;
 extern int	CONFIG_UNREACHABLE_PERIOD;
 extern int	CONFIG_UNREACHABLE_DELAY;
-extern int	CONFIG_HISTSYNCER_FORKS;
 extern int	CONFIG_PROXYCONFIG_FREQUENCY;
 extern int	CONFIG_PROXYDATA_FREQUENCY;
+extern int	CONFIG_HISTORYPOLLER_FORKS;
+extern int	CONFIG_ODBCPOLLER_FORKS;
 
 typedef struct
 {
@@ -80,6 +85,10 @@ typedef struct
 	unsigned char	useip;
 	unsigned char	type;
 	unsigned char	main;
+	unsigned char	available;
+	int		disable_until;
+	char		error[INTERFACE_ERROR_LEN_MAX];
+	int		errors_from;
 }
 DC_INTERFACE;
 
@@ -107,22 +116,10 @@ typedef struct
 	unsigned char	maintenance_status;
 	unsigned char	maintenance_type;
 	int		maintenance_from;
-	int		errors_from;
-	unsigned char	available;
-	int		disable_until;
-	int		snmp_errors_from;
-	unsigned char	snmp_available;
-	int		snmp_disable_until;
-	int		ipmi_errors_from;
-	unsigned char	ipmi_available;
-	int		ipmi_disable_until;
 	signed char	ipmi_authtype;
 	unsigned char	ipmi_privilege;
 	char		ipmi_username[HOST_IPMI_USERNAME_LEN_MAX];
 	char		ipmi_password[HOST_IPMI_PASSWORD_LEN_MAX];
-	int		jmx_errors_from;
-	unsigned char	jmx_available;
-	int		jmx_disable_until;
 	char		inventory_mode;
 	unsigned char	status;
 	unsigned char	tls_connect;
@@ -133,10 +130,6 @@ typedef struct
 	char		tls_psk_identity[HOST_TLS_PSK_IDENTITY_LEN_MAX];
 	char		tls_psk[HOST_TLS_PSK_LEN_MAX];
 #endif
-	char		error[HOST_ERROR_LEN_MAX];
-	char		snmp_error[HOST_ERROR_LEN_MAX];
-	char		ipmi_error[HOST_ERROR_LEN_MAX];
-	char		jmx_error[HOST_ERROR_LEN_MAX];
 }
 DC_HOST;
 
@@ -201,6 +194,7 @@ typedef struct
 	char			ssl_key_password_orig[ITEM_SSL_KEY_PASSWORD_LEN_MAX], *ssl_key_password;
 	char			*script_params;
 	char			*error;
+	unsigned char		*formula_bin;
 }
 DC_ITEM;
 
@@ -211,16 +205,10 @@ typedef struct
 	zbx_uint64_t	itemid;
 	char		*function;
 	char		*parameter;
+	unsigned char	type;
 }
 DC_FUNCTION;
 
-typedef struct
-{
-	char	*tag;
-	char	*value;
-}
-zbx_tag_t;
-
 typedef struct
 {
 	zbx_uint64_t	hostid;
@@ -237,9 +225,6 @@ typedef struct _DC_TRIGGER
 {
 	zbx_uint64_t		triggerid;
 	char			*description;
-	char			*expression_orig;
-	char			*recovery_expression_orig;
-	/* temporary values, allocated during processing and freed right after */
 	char			*expression;
 	char			*recovery_expression;
 
@@ -248,6 +233,8 @@ typedef struct _DC_TRIGGER
 	char			*correlation_tag;
 	char			*opdata;
 	char			*event_name;
+	unsigned char		*expression_bin;
+	unsigned char		*recovery_expression_bin;
 	zbx_timespec_t		timespec;
 	int			lastchange;
 	unsigned char		topoindex;
@@ -263,6 +250,9 @@ typedef struct _DC_TRIGGER
 	unsigned char		flags;
 
 	zbx_vector_ptr_t	tags;
+
+	zbx_eval_context_t	*eval_ctx;
+	zbx_eval_context_t	*eval_ctx_r;
 }
 DC_TRIGGER;
 
@@ -335,6 +325,7 @@ typedef struct
 	int		events_internal;
 	int		events_discovery;
 	int		events_autoreg;
+	int		events_service;
 	int		services;
 	int		audit;
 	int		sessions;
@@ -356,7 +347,6 @@ typedef struct
 {
 	char		*extension;
 	unsigned char	history_compression_status;
-	unsigned char	history_compression_availability;
 	int		history_compress_older;
 }
 zbx_config_db_t;
@@ -373,6 +363,7 @@ typedef struct
 	unsigned char	snmptrap_logging;
 	unsigned char	autoreg_tls_accept;
 	char		*default_timezone;
+	int		auditlog_enabled;
 
 	/* database configuration data for ZBX_CONFIG_DB_EXTENSION_* extensions */
 	zbx_config_db_t	db;
@@ -390,6 +381,7 @@ zbx_config_t;
 #define ZBX_CONFIG_FLAGS_DB_EXTENSION			__UINT64_C(0x0000000000000040)
 #define ZBX_CONFIG_FLAGS_AUTOREG_TLS_ACCEPT		__UINT64_C(0x0000000000000080)
 #define ZBX_CONFIG_FLAGS_DEFAULT_TIMEZONE		__UINT64_C(0x0000000000000100)
+#define ZBX_CONFIG_FLAGS_AUDITLOG_ENABLED		__UINT64_C(0x0000000000000200)
 
 /* possible values for database extensions (if flag ZBX_CONFIG_FLAGS_DB_EXTENSION set) */
 #define ZBX_CONFIG_DB_EXTENSION_TIMESCALE		"timescaledb"
@@ -619,8 +611,12 @@ void	dc_add_history(zbx_uint64_t itemid, unsigned char item_value_type, unsigned
 void	dc_flush_history(void);
 void	zbx_sync_history_cache(int *values_num, int *triggers_num, int *more);
 void	zbx_log_sync_history_cache_progress(void);
+
+#define ZBX_SYNC_NONE	0
+#define ZBX_SYNC_ALL	1
+
 int	init_database_cache(char **error);
-void	free_database_cache(void);
+void	free_database_cache(int);
 
 #define ZBX_STATS_HISTORY_COUNTER	0
 #define ZBX_STATS_HISTORY_FLOAT_COUNTER	1
@@ -675,7 +671,8 @@ zbx_uint64_t	DCget_nextid(const char *table_name, int num);
 
 #define ZBX_ITEM_GET_PROCESS		(ZBX_ITEM_GET_MAINTENANCE|ZBX_ITEM_GET_MISC|ZBX_ITEM_GET_LOGTIMEFMT)
 
-void	DCsync_configuration(unsigned char mode, const struct zbx_json_parse *jp_kvs_paths);
+void	DCsync_configuration(unsigned char mode);
+void	DCsync_kvs_paths(const struct zbx_json_parse *jp_kvs_paths);
 int	init_configuration_cache(char **error);
 void	free_configuration_cache(void);
 
@@ -700,6 +697,7 @@ void	DCconfig_unlock_triggers(const zbx_vector_uint64_t *triggerids);
 void	DCconfig_unlock_all_triggers(void);
 void	DCconfig_get_triggers_by_itemids(zbx_hashset_t *trigger_info, zbx_vector_ptr_t *trigger_order,
 		const zbx_uint64_t *itemids, const zbx_timespec_t *timespecs, int itemids_num);
+int	DCconfig_trigger_exists(zbx_uint64_t triggerid);
 void	DCfree_triggers(zbx_vector_ptr_t *triggers);
 void	DCconfig_update_interface_snmp_stats(zbx_uint64_t interfaceid, int max_snmp_succeed, int min_snmp_fail);
 int	DCconfig_get_suggested_snmp_vars(zbx_uint64_t interfaceid, int *bulk);
@@ -748,7 +746,6 @@ int	DCget_host_inventory_value_by_hostid(zbx_uint64_t hostid, char **replace_to,
 void	*DCconfig_get_stats(int request);
 
 int	DCconfig_get_last_sync_time(void);
-void	DCconfig_wait_sync(void);
 int	DCconfig_get_proxypoller_hosts(DC_PROXY *proxies, int max_hosts);
 int	DCconfig_get_proxypoller_nextcheck(void);
 
@@ -774,14 +771,13 @@ void	DCget_autoregistration_psk(char *psk_identity_buf, size_t psk_identity_buf_
 #define ZBX_MACRO_SECRET_MASK	"******"
 
 void	DCget_user_macro(const zbx_uint64_t *hostids, int hostids_num, const char *macro, char **replace_to);
-char	*DCexpression_expand_user_macros(const char *expression);
 char	*zbx_dc_expand_func_params_user_macros(zbx_uint64_t hostid, const char *params);
 
-int	DChost_activate(zbx_uint64_t hostid, unsigned char agent_type, const zbx_timespec_t *ts,
-		zbx_agent_availability_t *in, zbx_agent_availability_t *out);
+int	DCinterface_activate(zbx_uint64_t interfaceid, const zbx_timespec_t *ts, zbx_agent_availability_t *in,
+		zbx_agent_availability_t *out);
 
-int	DChost_deactivate(zbx_uint64_t hostid, unsigned char agent_type, const zbx_timespec_t *ts,
-		zbx_agent_availability_t *in, zbx_agent_availability_t *out, const char *error_msg);
+int	DCinterface_deactivate(zbx_uint64_t interfaceid, const zbx_timespec_t *ts, zbx_agent_availability_t *in,
+		zbx_agent_availability_t *out, const char *error_msg);
 
 #define ZBX_QUEUE_FROM_DEFAULT	6	/* default lower limit for delay (in seconds) */
 #define ZBX_QUEUE_TO_INFINITY	-1	/* no upper limit for delay */
@@ -819,21 +815,22 @@ unsigned int	DCget_internal_action_count(void);
 #define ZBX_DISCOVERY_GROUPID_UNDEFINED	0
 void	zbx_config_get(zbx_config_t *cfg, zbx_uint64_t flags);
 void	zbx_config_clean(zbx_config_t *cfg);
+void	zbx_config_get_hk_mode(unsigned char *history_mode, unsigned char *trends_mode);
 
-int	DCset_hosts_availability(zbx_vector_ptr_t *availabilities);
+int	DCset_interfaces_availability(zbx_vector_availability_ptr_t *availabilities);
 
-int	DCreset_hosts_availability(zbx_vector_ptr_t *hosts);
-void	DCupdate_hosts_availability(void);
+int	DCreset_interfaces_availability(zbx_vector_availability_ptr_t *interfaces);
+void	DCupdate_interfaces_availability(void);
 
 void	zbx_dc_get_actions_eval(zbx_vector_ptr_t *actions, unsigned char opflags);
 
-int	DCget_hosts_availability(zbx_vector_ptr_t *hosts, int *ts);
-void	DCtouch_hosts_availability(const zbx_vector_uint64_t *hostids);
+int	DCget_interfaces_availability(zbx_vector_ptr_t *interfaces, int *ts);
+void	DCtouch_interfaces_availability(const zbx_vector_uint64_t *interfaceids);
 
-void	zbx_host_availability_init(zbx_host_availability_t *availability, zbx_uint64_t hostid);
-void	zbx_host_availability_clean(zbx_host_availability_t *ha);
-void	zbx_host_availability_free(zbx_host_availability_t *availability);
-int	zbx_host_availability_is_set(const zbx_host_availability_t *ha);
+void	zbx_interface_availability_init(zbx_interface_availability_t *availability, zbx_uint64_t interfaceid);
+void	zbx_interface_availability_clean(zbx_interface_availability_t *ia);
+void	zbx_interface_availability_free(zbx_interface_availability_t *availability);
+int	zbx_interface_availability_is_set(const zbx_interface_availability_t *ia);
 
 void	zbx_set_availability_diff_ts(int ts);
 
@@ -843,7 +840,7 @@ void	zbx_dc_correlation_rules_free(zbx_correlation_rules_t *rules);
 void	zbx_dc_correlation_rules_get(zbx_correlation_rules_t *rules);
 
 void	zbx_dc_get_nested_hostgroupids(zbx_uint64_t *groupids, int groupids_num, zbx_vector_uint64_t *nested_groupids);
-void	zbx_dc_get_nested_hostgroupids_by_names(zbx_vector_str_t *groups, zbx_vector_uint64_t *nested_groupids);
+void	zbx_dc_get_hostids_by_group_name(const char *name, zbx_vector_uint64_t *hostids);
 
 #define ZBX_HC_ITEM_STATUS_NORMAL	0
 #define ZBX_HC_ITEM_STATUS_BUSY		1
@@ -880,7 +877,6 @@ typedef struct
 }
 zbx_hc_item_t;
 
-void	zbx_free_tag(zbx_tag_t *tag);
 void	zbx_free_item_tag(zbx_item_tag_t *item_tag);
 
 int	zbx_dc_get_active_proxy_by_name(const char *name, DC_PROXY *proxy, char **error);
@@ -982,6 +978,8 @@ void	zbx_dc_maintenance_reset_update_flag(int timer);
 int	zbx_dc_maintenance_check_update_flag(int timer);
 int	zbx_dc_maintenance_check_update_flags(void);
 
+int	zbx_dc_maintenance_has_tags(void);
+
 typedef struct
 {
 	char	*lld_macro;
@@ -995,6 +993,7 @@ int	zbx_lld_macro_value_by_name(const struct zbx_json_parse *jp_row, const zbx_v
 		const char *macro, char **value);
 int	zbx_lld_macro_paths_compare(const void *d1, const void *d2);
 
+void	zbx_dc_get_item_tags(zbx_uint64_t itemid, zbx_vector_ptr_t *item_tags);
 void	zbx_dc_get_item_tags_by_functionids(const zbx_uint64_t *functionids, size_t functionids_num,
 		zbx_vector_ptr_t *item_tags);
 
@@ -1003,7 +1002,9 @@ unsigned char	zbx_dc_set_macro_env(unsigned char env);
 const char	*zbx_dc_get_instanceid(void);
 
 char	*zbx_dc_expand_user_macros(const char *text, zbx_uint64_t hostid);
-char	*zbx_dc_expand_user_macros_in_func_params(const char *params, zbx_uint64_t hostid);
+int	zbx_dc_expand_user_macros_len(const char *text, size_t text_len, zbx_uint64_t *hostids, int hostids_num,
+		char **value, char **error);
+
 
 /* diagnostic data */
 void	zbx_hc_get_diag_stats(zbx_uint64_t *items_num, zbx_uint64_t *values_num);
@@ -1014,10 +1015,10 @@ typedef struct
 {
 	zbx_uint64_t		objectid;
 	zbx_uint64_t		triggerid;
-	zbx_function_type_t	type;
+	zbx_uint32_t		type;
 	zbx_time_unit_t		trend_base;
 	unsigned char		lock;		/* 1 if the timer has locked trigger, 0 otherwise */
-	int			revision;	/* function revision */
+	int			revision;	/* revision */
 	zbx_timespec_t		eval_ts;	/* the history time for which trigger must be recalculated */
 	zbx_timespec_t		exec_ts;	/* real time when the timer must be executed */
 	const char		*parameter;	/* function parameters (for trend functions) */
@@ -1034,6 +1035,16 @@ void	zbx_dc_free_timers(zbx_vector_ptr_t *timers);
 int	zbx_db_trigger_queue_locked(void);
 void	zbx_db_trigger_queue_unlock(void);
 
+void	zbx_get_host_interfaces_availability(zbx_uint64_t	hostid, zbx_agent_availability_t *agents);
+
 int	zbx_hc_check_proxy(zbx_uint64_t proxyid);
 
+void	zbx_dc_eval_expand_user_macros(zbx_eval_context_t *ctx);
+
+void	zbx_db_trigger_explain_expression(const DB_TRIGGER *trigger, char **expression,
+		int (*eval_func_cb)(zbx_variant_t *, DC_ITEM *, const char *, const char *, const zbx_timespec_t *,
+		char **), int recovery);
+void	zbx_db_trigger_get_function_value(const DB_TRIGGER *trigger, int index, char **value,
+		int (*eval_func_cb)(zbx_variant_t *, DC_ITEM *, const char *, const char *, const zbx_timespec_t *,
+		char **), int recovery);
 #endif
