@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,6 +20,9 @@
 #ifndef ZABBIX_ZBXHISTORY_H
 #define ZABBIX_ZBXHISTORY_H
 
+#include "zbxvariant.h"
+#include "zbxjson.h"
+
 /* the item history value */
 typedef struct
 {
@@ -30,6 +33,8 @@ zbx_history_record_t;
 
 ZBX_VECTOR_DECL(history_record, zbx_history_record_t)
 
+int     history_record_float_compare(const zbx_history_record_t *d1, const zbx_history_record_t *d2);
+
 void	zbx_history_record_vector_clean(zbx_vector_history_record_t *vector, int value_type);
 void	zbx_history_record_vector_destroy(zbx_vector_history_record_t *vector, int value_type);
 void	zbx_history_record_clear(zbx_history_record_t *value, int value_type);
@@ -40,6 +45,7 @@ int	zbx_history_record_compare_desc_func(const zbx_history_record_t *d1, const z
 void	zbx_history_value2str(char *buffer, size_t size, const history_value_t *value, int value_type);
 char	*zbx_history_value2str_dyn(const history_value_t *value, int value_type);
 void	zbx_history_value_print(char *buffer, size_t size, const history_value_t *value, int value_type);
+void	zbx_history_value2variant(const history_value_t *value, unsigned char value_type, zbx_variant_t *var);
 
 /* In most cases zbx_history_record_vector_destroy() function should be used to free the  */
 /* value vector filled by zbx_vc_get_value* functions. This define simply better          */
@@ -50,11 +56,15 @@ void	zbx_history_value_print(char *buffer, size_t size, const history_value_t *v
 int	zbx_history_init(char **error);
 void	zbx_history_destroy(void);
 
-int	zbx_history_add_values(const zbx_vector_ptr_t *history);
+int	zbx_history_add_values(const zbx_vector_ptr_t *history, int *ret_flush);
 int	zbx_history_get_values(zbx_uint64_t itemid, int value_type, int start, int count, int end,
 		zbx_vector_history_record_t *values);
 
 int	zbx_history_requires_trends(int value_type);
+void	zbx_history_check_version(struct zbx_json *json);
 
+#define FLUSH_SUCCEED		0
+#define FLUSH_FAIL		-1
+#define FLUSH_DUPL_REJECTED	-2
 
 #endif
