@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -40,13 +40,24 @@ func checkMetrics(s scheduler.Scheduler) {
 		`vfs.file.regmatch[c:\windows\win.ini,fonts]`,
 		`vfs.file.md5sum[c:\windows\win.ini]`,
 		`vfs.file.cksum[c:\windows\win.ini]`,
+		`vfs.file.owner[c:\windows\win.ini]`,
+		`vfs.file.permissions[c:\windows\win.ini]`,
+		`vfs.file.get[c:\windows\win.ini]`,
 		`vfs.dir.size[c:\windows]`,
 		`vfs.dir.count[c:\windows]`,
+		`vfs.dir.get[c:\windows]`,
 		`net.dns[,zabbix.com]`,
 		`net.dns.record[,zabbix.com]`,
 		`net.tcp.dns[,zabbix.com]`,
 		`net.tcp.dns.query[,zabbix.com]`,
 		`net.tcp.port[,80]`,
+		`net.tcp.listen[80]`,
+		`net.tcp.service[ssh,127.0.0.1,22]`,
+		`net.tcp.service.perf[ssh,127.0.0.1,22]`,
+		`net.tcp.socket.count[,80]`,
+		`net.udp.service[ntp,127.0.0.1,123]`,
+		`net.udp.service.perf[ntp,127.0.0.1,123]`,
+		`net.udp.socket.count[,53]`,
 		`system.users.num`,
 		`log[logfile]`,
 		`log.count[logfile]`,
@@ -57,7 +68,6 @@ func checkMetrics(s scheduler.Scheduler) {
 		`vfs.fs.size[c:,free]`,
 		`vfs.fs.discovery`,
 		`vfs.fs.get`,
-		`net.tcp.listen[80]`,
 		`net.if.in[MS TCP Loopback interface,bytes]`,
 		`net.if.out[MS TCP Loopback interface,bytes]`,
 		`net.if.total[MS TCP Loopback interface,bytes]`,
@@ -83,13 +93,10 @@ func checkMetrics(s scheduler.Scheduler) {
 		`proc_info[svchost.exe]`,
 		`wmi.get[root\cimv2,select Caption from Win32_OperatingSystem]`,
 		`wmi.getall[root\cimv2,select * from Win32_OperatingSystem]`,
-		`net.tcp.service[ssh,127.0.0.1,22]`,
-		`net.tcp.service.perf[ssh,127.0.0.1,22]`,
-		`net.udp.service[ntp,127.0.0.1,123]`,
-		`net.udp.service.perf[ntp,127.0.0.1,123]`,
-		`system.hostname`,
 		`perf_instance.discovery[Processor]`,
 		`perf_instance_en.discovery[Processor]`,
+		`modbus.get[tcp://localhost]`,
+		`system.hostname`,
 	}
 
 	for _, metric := range metrics {
