@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -36,10 +36,10 @@ type mockWriter struct {
 	t       *testing.T
 }
 
-func (w *mockWriter) Write(data []byte, timeout time.Duration) (err error) {
+func (w *mockWriter) Write(data []byte, timeout time.Duration) (err []error) {
 	log.Debugf("%s", string(data))
 	if w.counter&1 != 0 {
-		err = errors.New("mock error")
+		err = []error{errors.New("mock error")}
 	} else {
 		var request AgentDataRequest
 		_ = json.Unmarshal(data, &request)
@@ -64,6 +64,10 @@ func (w *mockWriter) CanRetry() bool {
 	return false
 }
 
+func (w *mockWriter) Hostname() string {
+	return ""
+}
+
 func TestResultCache(t *testing.T) {
 	agent.Options.BufferSize = 10
 	agent.Options.EnablePersistentBuffer = 0
