@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,37 +23,32 @@ package version
 import (
 	"fmt"
 	"strings"
-
-	"zabbix.com/pkg/tls"
 )
 
 const (
-	APPLICATION_NAME        = "Zabbix Agent"
-	ZABBIX_REVDATE          = "28 June 2021"
-	ZABBIX_VERSION_MAJOR    = 5
-	ZABBIX_VERSION_MINOR    = 2
-	ZABBIX_VERSION_PATCH    = 7
+	ZABBIX_REVDATE          = "4 April 2022"
+	ZABBIX_VERSION_MAJOR    = 6
+	ZABBIX_VERSION_MINOR    = 0
+	ZABBIX_VERSION_PATCH    = 3
 	ZABBIX_VERSION_RC       = ""
 	ZABBIX_VERSION_RC_NUM   = "2400"
-	ZABBIX_VERSION_REVISION = "91e8333180"
-	copyrightMessage        = "Copyright (C) 2021 Zabbix SIA\n" +
+	ZABBIX_VERSION_REVISION = "506e2b51e2"
+	copyrightMessage        = "Copyright (C) 2022 Zabbix SIA\n" +
 		"License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>.\n" +
 		"This is free software: you are free to change and redistribute it according to\n" +
 		"the license. There is NO WARRANTY, to the extent permitted by law."
 )
 
 var (
-	titleMessage string = "{undefined}"
-	compileDate  string = "{undefined}"
-	compileTime  string = "{undefined}"
-	compileOs    string = "{undefined}"
-	compileArch  string = "{undefined}"
-	compileMode  string
+	titleMessage  string = "{undefined}"
+	compileDate   string = "{undefined}"
+	compileTime   string = "{undefined}"
+	compileOs     string = "{undefined}"
+	compileArch   string = "{undefined}"
+	compileMode   string
+	extraLicenses []string
 )
 
-func ApplicationName() string {
-	return APPLICATION_NAME
-}
 func RevDate() string {
 	return ZABBIX_REVDATE
 }
@@ -98,28 +93,14 @@ func Revision() string {
 	return ZABBIX_VERSION_REVISION
 }
 
-func copyrightMessageMQTT() string {
-	return "\nWe use the library Eclipse Paho (eclipse/paho.mqtt.golang), which is\n" +
-		"distributed under the terms of the Eclipse Distribution License 1.0 (The 3-Clause BSD License)\n" +
-		"available at https://www.eclipse.org/org/documents/edl-v10.php\n"
-}
-
-func copyrightMessageModbus() string {
-	return "\nWe use the library go-modbus (goburrow/modbus), which is\n" +
-		"distributed under the terms of the 3-Clause BSD License\n" +
-		"available at https://github.com/goburrow/modbus/blob/master/LICENSE\n"
-}
-
 func CopyrightMessage() string {
 	msg := copyrightMessage
-	tlsMsg := tls.CopyrightMessage()
-	if tlsMsg == "" {
-		msg += "\n"
-	} else {
-		msg += tlsMsg
+
+	for _, license := range extraLicenses {
+		msg += license
 	}
 
-	return msg + copyrightMessageModbus() + copyrightMessageMQTT()
+	return msg
 }
 
 func CompileDate() string {
@@ -164,3 +145,12 @@ func Display() {
 	fmt.Printf("Revision %s %s, compilation time: %s %s\n\n", Revision(), RevDate(), CompileDate(), CompileTime())
 	fmt.Println(CopyrightMessage())
 }
+
+func Init(title string, extra ...string) {
+	titleMessage = title
+	extraLicenses = append(extraLicenses, extra...)
+}
+
+func init() {
+	extraLicenses = make([]string, 0)
+}
