@@ -1,8 +1,9 @@
 //go:build !windows
+// +build !windows
 
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@ package zbxcmd
 
 import (
 	"bytes"
+	"errors"
 	"fmt"
 	"os/exec"
 	"strings"
@@ -32,8 +34,9 @@ import (
 	"zabbix.com/pkg/log"
 )
 
-func execute(s string, timeout time.Duration, strict bool) (string, error) {
+func execute(s string, timeout time.Duration, path string, strict bool) (string, error) {
 	cmd := exec.Command("sh", "-c", s)
+	cmd.Dir = path
 
 	var b bytes.Buffer
 	cmd.Stdout = &b
@@ -61,14 +64,12 @@ func execute(s string, timeout time.Duration, strict bool) (string, error) {
 	}
 
 	// we need to check error after t.Stop so we can inform the user if timeout was reached and Zabbix agent2 terminated the command
-	if strict && werr != nil {
-		log.Debugf("Command [%s] execution failed: %s\n%s", s, werr, b.String())
-
-		return "", fmt.Errorf("Command execution failed: %w", werr)
+	if strict && werr != nil && !errors.Is(werr, syscall.ECHILD) {
+		return "", fmt.Errorf("Command execution failed: %s", werr)
 	}
 
-	if maxExecuteOutputLenB <= len(b.String()) {
-		return "", fmt.Errorf("Command output exceeded limit of %d KB", maxExecuteOutputLenB/1024)
+	if MaxExecuteOutputLenB <= len(b.String()) {
+		return "", fmt.Errorf("Command output exceeded limit of %d KB", MaxExecuteOutputLenB/1024)
 	}
 
 	return strings.TrimRight(b.String(), " \t\r\n"), nil
