@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@ import (
 	"fmt"
 	"time"
 
-	"zabbix.com/pkg/metric"
 	"zabbix.com/pkg/uri"
 	"zabbix.com/pkg/zbxerr"
 
@@ -46,12 +45,7 @@ var impl Plugin
 
 // Export implements the Exporter interface.
 func (p *Plugin) Export(key string, rawParams []string, _ plugin.ContextProvider) (result interface{}, err error) {
-	params, _, hc, err := metrics[key].EvalParams(rawParams, p.options.Sessions)
-	if err != nil {
-		return nil, err
-	}
-
-	err = metric.SetDefaults(params, hc, p.options.Default)
+	params, _, err := metrics[key].EvalParams(rawParams, p.options.Sessions)
 	if err != nil {
 		return nil, err
 	}
