@@ -3,7 +3,7 @@
 
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -36,5 +36,6 @@ func init() {
 	plugin.RegisterMetrics(&impl, "TCP",
 		"net.tcp.port", "Checks if it is possible to make TCP connection to specified port.",
 		"net.tcp.service", "Checks if service is running and accepting TCP connections.",
-		"net.tcp.service.perf", "Checks performance of TCP service.")
+		"net.tcp.service.perf", "Checks performance of TCP service.",
+		"net.tcp.socket.count", "Returns number of TCP sockets that match parameters.")
 }
