@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -42,8 +42,7 @@ const (
 	ssdType     = "ssd"
 	hddType     = "hdd"
 
-	spinUpAttrName  = "Spin_Up_Time"
-	unknownAttrName = "Unknown_Attribute"
+	spinUpAttrName = "Spin_Up_Time"
 
 	ataSmartAttrFieldName      = "ata_smart_attributes"
 	ataSmartAttrTableFieldName = "table"
@@ -54,11 +53,6 @@ const (
 	typeFieldName   = "type"
 )
 
-const (
-	parseError = 1 << iota
-	openError
-)
-
 var (
 	cpuCount     int
 	lastVerCheck time.Time
@@ -74,79 +68,12 @@ type device struct {
 	DeviceType   string `json:"{#DISKTYPE}"`
 	Model        string `json:"{#MODEL}"`
 	SerialNumber string `json:"{#SN}"`
-	Path         string `json:"{#PATH}"`
-	RaidType     string `json:"{#RAIDTYPE}"`
-	Attributes   string `json:"{#ATTRIBUTES}"`
 }
 type jsonDevice struct {
 	serialNumber string
 	jsonData     string
 }
 
-type singleDevice struct {
-	DiskType        string              `json:"disk_type"`
-	Firmware        string              `json:"firmware_version"`
-	ModelName       string              `json:"model_name"`
-	SerialNumber    string              `json:"serial_number"`
-	Smartctl        smartctlField       `json:"smartctl"`
-	HealthLog       healthLog           `json:"nvme_smart_health_information_log"`
-	SmartAttributes singelRequestTables `json:"ata_smart_attributes"`
-	Data            ataData             `json:"ata_smart_data"`
-	Temperature     temperature         `json:"temperature"`
-	PowerOnTime     power               `json:"power_on_time"`
-	Err             string              `json:"-"`
-	SelfTest        bool                `json:"-"`
-}
-
-type healthLog struct {
-	Temperature     int `json:"temperature"`
-	PowerOnTime     int `json:"power_on_hours"`
-	CriticalWarning int `json:"critical_warning"`
-	MediaErrors     int `json:"media_errors"`
-	Percentage_used int `json:"percentage_used"`
-}
-
-type temperature struct {
-	Current int `json:"current"`
-}
-
-type power struct {
-	Hours int `json:"hours"`
-}
-
-type singelRequestTables struct {
-	Table []singelRequestRaw `json:"table"`
-}
-
-type singelRequestRaw struct {
-	Name string   `json:"name"`
-	Raw  rawField `json:"raw"`
-}
-
-type singleRequestAttribute struct {
-	Value int    `json:"value"`
-	Raw   string `json:"raw"`
-}
-
-type rawField struct {
-	Value int    `json:"value"`
-	Str   string `json:"string"`
-}
-
-type ataData struct {
-	SelfTest     selfTest     `json:"self_test"`
-	Capabilities capabilities `json:"capabilities"`
-}
-type capabilities struct {
-	SelfTestsSupported bool `json:"self_tests_supported"`
-}
-type selfTest struct {
-	Status status `json:"status"`
-}
-type status struct {
-	Passed bool `json:"passed"`
-}
-
 type attribute struct {
 	Name       string `json:"{#NAME}"`
 	DeviceType string `json:"{#DISKTYPE}"`
@@ -169,8 +96,6 @@ type deviceInfo struct {
 	Name     string `json:"name"`
 	InfoName string `json:"info_name"`
 	DevType  string `json:"type"`
-	name     string `json:"-"`
-	raidType string `json:"-"`
 }
 
 type smartctl struct {
@@ -259,17 +184,7 @@ func (p *Plugin) execute(jsonRunner bool) (*runner, error) {
 	return r, err
 }
 
-// executeSingle returns device data for single device from smartctl based on provided path.
-func (p *Plugin) executeSingle(path string) (device []byte, err error) {
-	device, err = p.executeSmartctl(fmt.Sprintf("-a %s -j", path), false)
-	if err != nil {
-		return nil, fmt.Errorf("Failed to execute smartctl: %w.", err)
-	}
-
-	return
-}
-
-// executeBase executed runners for basic devices retrieved from smartctl.
+//executeBase executed runners for basic devices retrived from smartctl
 func (r *runner) executeBase(basicDev []deviceInfo, jsonRunner bool) error {
 	r.startBasicRunners(jsonRunner)
 
@@ -282,7 +197,7 @@ func (r *runner) executeBase(basicDev []deviceInfo, jsonRunner bool) error {
 	return r.waitForExecution()
 }
 
-// executeRaids executes runners for raid devices (except megaraid) retrieved from smartctl
+//executeRaids executes runners for raid devices (except megaraid) retrived from smartctl
 func (r *runner) executeRaids(raids []deviceInfo, jsonRunner bool) {
 	raidTypes := []string{"3ware", "areca", "cciss", "sat"}
 
@@ -301,7 +216,7 @@ func (r *runner) executeRaids(raids []deviceInfo, jsonRunner bool) {
 	r.waitForRaidExecution(r.raidDone)
 }
 
-// executeMegaRaids executes runners for megaraid devices retrieved from smartctl
+//executeMegaRaids executes runners for megaraid devices retrived from smartctl
 func (r *runner) executeMegaRaids(megaraids []deviceInfo, jsonRunner bool) {
 	r.megaraids = make(chan raidParameters, len(megaraids))
 
@@ -448,13 +363,6 @@ func (r *runner) getBasicDevices(jsonRunner bool) {
 	defer r.wg.Done()
 
 	for name := range r.names {
-		err := clearString(name)
-		if err != nil {
-			r.err <- zbxerr.ErrorCannotFetchData.Wrap(err)
-
-			return
-		}
-
 		devices, err := r.plugin.executeSmartctl(fmt.Sprintf("-a %s -j", name), false)
 		if err != nil {
 			r.err <- fmt.Errorf("Failed to execute smartctl: %s.", err.Error())
@@ -473,8 +381,6 @@ func (r *runner) getBasicDevices(jsonRunner bool) {
 			return
 		}
 
-		dp.Info.name = name
-
 		r.mux.Lock()
 
 		if jsonRunner {
@@ -517,16 +423,6 @@ runner:
 				name = fmt.Sprintf("%s -d %s,%d", raid.name, raid.rType, i)
 			}
 
-			err := clearString(name)
-			if err != nil {
-				r.plugin.Tracef(
-					"stopped looking for RAID devices of %s type, err: %s",
-					raid.rType, fmt.Errorf("failed to parse RAID disk data from smartctl: %s", err.Error()),
-				)
-
-				continue runner
-			}
-
 			device, err := r.plugin.executeSmartctl(fmt.Sprintf("-a %s -j ", name), false)
 			if err != nil {
 				r.plugin.Tracef(
@@ -558,14 +454,10 @@ runner:
 			}
 
 			if dp.SmartStatus != nil {
-				dp.Info.name = raid.name
-
 				if raid.rType == satType {
 					dp.Info.Name = fmt.Sprintf("%s %s", raid.name, raid.rType)
-					dp.Info.raidType = raid.rType
 				} else {
 					dp.Info.Name = fmt.Sprintf("%s %s,%d", raid.name, raid.rType, i)
-					dp.Info.raidType = fmt.Sprintf("%s,%d", raid.rType, i)
 				}
 
 				if r.setRaidDevices(dp, device, raid.rType, jsonRunner) {
@@ -598,16 +490,6 @@ func (r *runner) getMegaRaidDevices(jsonRunner bool) {
 
 		name := fmt.Sprintf("%s -d %s", raid.name, raid.rType)
 
-		err := clearString(name)
-		if err != nil {
-			r.plugin.Tracef(
-				"stopped looking for RAID devices of %s type, err: %s",
-				raid.rType, fmt.Errorf("failed to parse RAID disk data from smartctl: %s", err.Error()),
-			)
-
-			continue
-		}
-
 		device, err := r.plugin.executeSmartctl(fmt.Sprintf("-a %s -j ", name), false)
 		if err != nil {
 			r.plugin.Tracef(
@@ -641,8 +523,6 @@ func (r *runner) getMegaRaidDevices(jsonRunner bool) {
 		}
 
 		dp.Info.Name = fmt.Sprintf("%s %s", raid.name, raid.rType)
-		dp.Info.name = raid.name
-		dp.Info.raidType = raid.rType
 
 		r.setRaidDevices(dp, device, raid.rType, jsonRunner)
 	}
@@ -712,7 +592,7 @@ func (r *runner) parseOutput(jsonRunner bool) {
 }
 
 func (dp *deviceParser) checkErr() (err error) {
-	if (parseError|openError)&dp.Smartctl.ExitStatus == 0 {
+	if dp.Smartctl.ExitStatus != 2 {
 		return
 	}
 
@@ -739,41 +619,32 @@ func (dp *deviceParser) checkErr() (err error) {
 func (p *Plugin) getDevices() (basic, raid, megaraid []deviceInfo, err error) {
 	basicTmp, err := p.scanDevices("--scan -j")
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("Failed to scan for devices: %w.", err)
+		return nil, nil, nil, fmt.Errorf("Failed to scan for devices: %s.", err)
 	}
 
 	raidTmp, err := p.scanDevices("--scan -d sat -j")
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("Failed to scan for sat devices: %w.", err)
+		return nil, nil, nil, fmt.Errorf("Failed to scan for sat devices: %s.", err)
 	}
 
-	basic, raid, megaraid = formatDeviceOutput(basicTmp, raidTmp)
-
-	return
-}
-
-// formatDeviceOutput removes raid devices from basic device list and separates megaraid devices from the rest of raid
-// devices.
-func formatDeviceOutput(basic, raid []deviceInfo) (basicDev, raidDev, megaraidDev []deviceInfo) {
-loop:
-	for _, tmp := range basic {
-		for _, r := range raid {
+raid:
+	for _, tmp := range basicTmp {
+		for _, r := range raidTmp {
 			if tmp.Name == r.Name {
-				continue loop
+				continue raid
 			}
 		}
 
-		basicDev = append(basicDev, tmp)
+		basic = append(basic, tmp)
 	}
 
-	for _, r := range raid {
+	for _, r := range raidTmp {
 		if strings.Contains(r.DevType, "megaraid") {
-			megaraidDev = append(megaraidDev, r)
-
+			megaraid = append(megaraid, r)
 			continue
 		}
 
-		raidDev = append(raidDev, r)
+		raid = append(raid, r)
 	}
 
 	return
