@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,8 +26,9 @@ import (
 )
 
 type Options struct {
-	Timeout  int `conf:"optional,range=1:30"`
-	Capacity int `conf:"optional,range=1:100"`
+	plugin.SystemOptions `conf:"optional,name=System"`
+	Timeout              int `conf:"optional,range=1:30"`
+	Capacity             int `conf:"optional,range=1:100"`
 }
 
 // Plugin -
@@ -57,6 +58,12 @@ func (p *Plugin) Export(key string, params []string, ctx plugin.ContextProvider)
 		return p.exportRegmatch(params)
 	case "vfs.file.md5sum":
 		return p.exportMd5sum(params)
+	case "vfs.file.owner":
+		return p.exportOwner(params)
+	case "vfs.file.permissions":
+		return p.exportPermissions(params)
+	case "vfs.file.get":
+		return p.exportGet(params)
 	default:
 		return nil, plugin.UnsupportedMetricError
 	}
@@ -88,5 +95,8 @@ func init() {
 		"vfs.file.size", "Returns file size.",
 		"vfs.file.regexp", "Find string in a file.",
 		"vfs.file.regmatch", "Find string in a file.",
-		"vfs.file.md5sum", "Returns MD5 checksum of file.")
+		"vfs.file.md5sum", "Returns MD5 checksum of file.",
+		"vfs.file.owner", "Returns the ownership of a file.",
+		"vfs.file.permissions", "Returns 4-digit string containing octal number with Unix permissions.",
+		"vfs.file.get", "Return json object with information about a file.")
 }
