@@ -3,7 +3,7 @@
 
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -55,16 +55,21 @@ func (p *Plugin) getFsInfoStats() (data []*FsInfoNew, err error) {
 			continue
 		}
 
-		fsmap[*info.FsName+*info.FsType] = &FsInfoNew{info.FsName, info.FsType, nil, bytes, inodes}
+		if bytes.Total > 0 && inodes.Total > 0 {
+			fsmap[*info.FsName] = &FsInfoNew{info.FsName, info.FsType, nil, nil, bytes, inodes}
+		}
 	}
 
+	fsStatCaller.close()
+	fsInodeCaller.close()
+
 	allData, err = p.getFsInfo()
 	if err != nil {
 		return nil, err
 	}
 
 	for _, info := range allData {
-		if fsInfo, ok := fsmap[*info.FsName+*info.FsType]; ok {
+		if fsInfo, ok := fsmap[*info.FsName]; ok {
 			data = append(data, fsInfo)
 		}
 	}
@@ -107,8 +112,6 @@ func (p *Plugin) getFsInfo() (data []*FsInfo, err error) {
 }
 
 func getFsStats(path string) (stats *FsStats, err error) {
-	var pused float64
-
 	fs := unix.Statfs_t{}
 	err = unix.Statfs(path, &fs)
 	if err != nil {
@@ -123,30 +126,19 @@ func getFsStats(path string) (stats *FsStats, err error) {
 	total := fs.Blocks * uint64(fs.Bsize)
 	free := available * uint64(fs.Bsize)
 	used := (fs.Blocks - fs.Bfree) * uint64(fs.Bsize)
-	pfree := float64(fs.Blocks - fs.Bfree + fs.Bavail)
-
-	if pfree > 0 {
-		pfree = 100.00 * float64(available) / pfree
-		pused = 100 - pfree
-	} else {
-		pfree = 0
-		pused = 0
-	}
-
+	pfree := 100.00 * float64(available) / float64(fs.Blocks-fs.Bfree+fs.Bavail)
 	stats = &FsStats{
 		Total: total,
 		Free:  free,
 		Used:  used,
 		PFree: pfree,
-		PUsed: pused,
+		PUsed: 100 - pfree,
 	}
 
 	return
 }
 
 func getFsInode(path string) (stats *FsStats, err error) {
-	var pfree, pused float64
-
 	fs := unix.Statfs_t{}
 	err = unix.Statfs(path, &fs)
 	if err != nil {
@@ -156,21 +148,12 @@ func getFsInode(path string) (stats *FsStats, err error) {
 	total := fs.Files
 	free := fs.Ffree
 	used := fs.Files - fs.Ffree
-
-	if 0 < total {
-		pfree = 100 * float64(free) / float64(total)
-		pused = 100 * float64(total-free) / float64(total)
-	} else {
-		pfree = 100.0
-		pused = 0.0
-	}
-
 	stats = &FsStats{
 		Total: total,
 		Free:  free,
 		Used:  used,
-		PFree: pfree,
-		PUsed: pused,
+		PFree: 100 * float64(free) / float64(total),
+		PUsed: 100 * float64(total-free) / float64(total),
 	}
 
 	return
