@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ func getMountPaths() (paths []string, err error) {
 	if h, err = windows.FindFirstVolume(&volume[0], uint32(len(volume))); err != nil {
 		return
 	}
-	defer windows.FindVolumeClose(h)
+	defer windows.FindClose(h)
 
 	var result []string
 	var size uint32
@@ -71,7 +71,7 @@ func getMountPaths() (paths []string, err error) {
 	return result, nil
 }
 
-func getFsInfo(path string) (fsname, fstype, drivetype string, err error) {
+func getFsInfo(path string) (fsname, fstype, drivetype string, drivelabel string, err error) {
 	fsname = path
 	if len(fsname) > 0 && fsname[len(fsname)-1] == '\\' {
 		fsname = fsname[:len(fsname)-1]
@@ -82,11 +82,14 @@ func getFsInfo(path string) (fsname, fstype, drivetype string, err error) {
 	}
 
 	wpath := windows.StringToUTF16Ptr(path)
-	buf := make([]uint16, windows.MAX_PATH+1)
-	if err = windows.GetVolumeInformation(wpath, nil, 0, nil, nil, nil, &buf[0], uint32(len(buf))); err != nil {
+	bufType := make([]uint16, windows.MAX_PATH+1)
+	bufLabel := make([]uint16, windows.MAX_PATH+1)
+	if err = windows.GetVolumeInformation(wpath, &bufLabel[0], uint32(len(bufLabel)),
+		nil, nil, nil, &bufType[0], uint32(len(bufType))); err != nil {
 		fstype = "UNKNOWN"
 	} else {
-		fstype = windows.UTF16ToString(buf)
+		fstype = windows.UTF16ToString(bufType)
+		drivelabel = windows.UTF16ToString(bufLabel)
 	}
 
 	dt := windows.GetDriveType(wpath)
@@ -133,11 +136,12 @@ func (p *Plugin) getFsInfo() (data []*FsInfo, err error) {
 		return
 	}
 	for _, path := range paths {
-		if fsname, fstype, drivetype, fserr := getFsInfo(path); fserr == nil {
+		if fsname, fstype, drivetype, drivelabel, fserr := getFsInfo(path); fserr == nil {
 			data = append(data, &FsInfo{
-				FsName:    &fsname,
-				FsType:    &fstype,
-				DriveType: &drivetype,
+				FsName:     &fsname,
+				FsType:     &fstype,
+				DriveType:  &drivetype,
+				DriveLabel: &drivelabel,
 			})
 		} else {
 			p.Debugf(`cannot obtain file system information for "%s": %s`, path, fserr)
@@ -154,10 +158,11 @@ func (p *Plugin) getFsInfoStats() (data []*FsInfoNew, err error) {
 	fsmap := make(map[string]*FsInfoNew)
 	for _, path := range paths {
 		var info FsInfoNew
-		if fsname, fstype, drivetype, fserr := getFsInfo(path); fserr == nil {
+		if fsname, fstype, drivetype, drivelabel, fserr := getFsInfo(path); fserr == nil {
 			info.FsName = &fsname
 			info.FsType = &fstype
 			info.DriveType = &drivetype
+			info.DriveLabel = &drivelabel
 		} else {
 			p.Debugf(`cannot obtain file system information for "%s": %s`, path, fserr)
 			continue
