@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -730,10 +730,10 @@ typedef struct
 }
 dbu_interface_t;
 
-ZBX_PTR_VECTOR_DECL(dbu_interface, dbu_interface_t);
-ZBX_PTR_VECTOR_IMPL(dbu_interface, dbu_interface_t);
-ZBX_PTR_VECTOR_DECL(dbu_snmp_if, dbu_snmp_if_t);
-ZBX_PTR_VECTOR_IMPL(dbu_snmp_if, dbu_snmp_if_t);
+ZBX_PTR_VECTOR_DECL(dbu_interface, dbu_interface_t)
+ZBX_PTR_VECTOR_IMPL(dbu_interface, dbu_interface_t)
+ZBX_PTR_VECTOR_DECL(dbu_snmp_if, dbu_snmp_if_t)
+ZBX_PTR_VECTOR_IMPL(dbu_snmp_if, dbu_snmp_if_t)
 
 static void	db_interface_free(dbu_interface_t interface)
 {
@@ -812,8 +812,6 @@ static int	db_snmp_new_if_find(const dbu_snmp_if_t *snmp, const zbx_vector_dbu_s
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: DBpatch_load_data                                                *
  *                                                                            *
  * Purpose: loading a set of unique combination of snmp data within a single  *
  *          interface and associated interface data                           *
@@ -1148,8 +1146,6 @@ static int	DBpatch_items_type_update(void)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: DBpatch_4050046                                                  *
  *                                                                            *
  * Purpose: migration snmp data from 'items' table to 'interface_snmp' new    *
  *          table linked with 'interface' table, except interface links for   *
@@ -1259,8 +1255,6 @@ static void	db_if_link(zbx_uint64_t if_slave, zbx_uint64_t if_master, zbx_vector
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: DBpatch_if_load_data                                             *
  *                                                                            *
  * Purpose: loading all unlinked interfaces, snmp data and hostid of host     *
  *          prototype for discovered hosts                                    *
@@ -1391,8 +1385,6 @@ static int	DBpatch_interface_discovery_save(zbx_vector_uint64_pair_t *if_links)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: DBpatch_4050047                                                  *
  *                                                                            *
  * Purpose: recovery links between the interfaceid of discovered host and     *
  *          parent interfaceid from parent host                               *
