@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,14 +17,14 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+#include "httprequest.h"
+
 #include "common.h"
-#include "log.h"
 #include "zbxjson.h"
 #include "zbxhttp.h"
-#include "zbxembed.h"
-#include "httprequest.h"
 #include "embed.h"
 #include "duktape.h"
+#include "zbxalgo.h"
 
 #ifdef HAVE_LIBCURL
 
@@ -87,8 +87,6 @@ static size_t	curl_header_cb(void *ptr, size_t size, size_t nmemb, void *userdat
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: es_httprequest                                                   *
  *                                                                            *
  * Purpose: return backing C structure embedded in CurlHttpRequest object     *
  *                                                                            *
@@ -106,8 +104,6 @@ static zbx_es_httprequest_t *es_httprequest(duk_context *ctx)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: es_httprequest_dtor                                              *
  *                                                                            *
  * Purpose: CurlHttpRequest destructor                                        *
  *                                                                            *
@@ -136,8 +132,6 @@ static duk_ret_t	es_httprequest_dtor(duk_context *ctx)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: es_httprequest_ctor                                              *
  *                                                                            *
  * Purpose: CurlHttpRequest constructor                                       *
  *                                                                            *
@@ -146,20 +140,11 @@ static duk_ret_t	es_httprequest_ctor(duk_context *ctx)
 {
 	zbx_es_httprequest_t	*request;
 	CURLcode		err;
-	zbx_es_env_t		*env;
 	int			err_index = -1;
 
 	if (!duk_is_constructor_call(ctx))
 		return DUK_RET_TYPE_ERROR;
 
-	duk_push_global_stash(ctx);
-
-	if (1 != duk_get_prop_string(ctx, -1, "\xff""\xff""zbx_env"))
-		return duk_error(ctx, DUK_RET_TYPE_ERROR, "cannot access internal environment");
-
-	env = (zbx_es_env_t *)duk_to_pointer(ctx, -1);
-	duk_pop(ctx);
-
 	duk_push_this(ctx);
 
 	request = (zbx_es_httprequest_t *)zbx_malloc(NULL, sizeof(zbx_es_httprequest_t));
@@ -177,7 +162,6 @@ static duk_ret_t	es_httprequest_ctor(duk_context *ctx)
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_WRITEDATA, request, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_PRIVATE, request, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_SSL_VERIFYPEER, 0L, err);
-	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_TIMEOUT, (long)env->timeout, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_SSL_VERIFYHOST, 0L, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_HEADERFUNCTION, curl_header_cb, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_HEADERDATA, request, err);
@@ -202,8 +186,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: es_httprequest_add_header                                        *
  *                                                                            *
  * Purpose: CurlHttpRequest.SetHeader method                                  *
  *                                                                            *
@@ -237,8 +219,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: es_httprequest_clear_header                                      *
  *                                                                            *
  * Purpose: CurlHttpRequest.ClearHeader method                                *
  *                                                                            *
@@ -258,8 +238,6 @@ static duk_ret_t	es_httprequest_clear_header(duk_context *ctx)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: es_httprequest_query                                             *
  *                                                                            *
  * Purpose: CurlHttpRequest HTTP request implementation                       *
  *                                                                            *
@@ -273,6 +251,20 @@ static duk_ret_t	es_httprequest_query(duk_context *ctx, const char *http_request
 	char			*url = NULL, *contents = NULL;
 	CURLcode		err;
 	int			err_index = -1;
+	zbx_es_env_t		*env;
+	zbx_uint64_t		timeout_ms, elapsed_ms;
+
+	if (NULL == (env = zbx_es_get_env(ctx)))
+		return duk_error(ctx, DUK_RET_TYPE_ERROR, "cannot access internal environment");
+
+	elapsed_ms = zbx_get_duration_ms(&env->start_time);
+	timeout_ms = (zbx_uint64_t)env->timeout * 1000;
+
+	if (elapsed_ms >= timeout_ms)
+	{
+		err_index = duk_push_error_object(ctx, DUK_RET_EVAL_ERROR, "script execution timeout occurred");
+		goto out;
+	}
 
 	if (SUCCEED != zbx_cesu8_to_utf8(duk_to_string(ctx, 0), &url))
 	{
@@ -319,7 +311,9 @@ static duk_ret_t	es_httprequest_query(duk_context *ctx, const char *http_request
 
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_HTTPHEADER, request->headers, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_CUSTOMREQUEST, http_request, err);
+	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_TIMEOUT_MS, timeout_ms - elapsed_ms, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_POSTFIELDS, ZBX_NULL2EMPTY_STR(contents), err);
+	ZBX_CURL_SETOPT(ctx, request->handle, ZBX_CURLOPT_ACCEPT_ENCODING, "", err);
 
 	request->data_offset = 0;
 	request->headers_in_offset = 0;
@@ -344,9 +338,7 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Function: es_httprequest_get                                               *
- *                                                                            *
- * Purpose: CurlHttpRequest.Get method                                        *
+ * Purpose: HttpRequest.Get / CurlHttpRequest.Get method                      *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_get(duk_context *ctx)
@@ -356,9 +348,7 @@ static duk_ret_t	es_httprequest_get(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Function: es_httprequest_put                                               *
- *                                                                            *
- * Purpose: CurlHttpRequest.Put method                                        *
+ * Purpose: HttpRequest.Put / CurlHttpRequest.Put method                      *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_put(duk_context *ctx)
@@ -368,9 +358,7 @@ static duk_ret_t	es_httprequest_put(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Function: es_httprequest_post                                              *
- *                                                                            *
- * Purpose: CurlHttpRequest.Post method                                       *
+ * Purpose: HttpRequest.Post / CurlHttpRequest.Post method                    *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_post(duk_context *ctx)
@@ -380,9 +368,7 @@ static duk_ret_t	es_httprequest_post(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Function: es_httprequest_delete                                            *
- *                                                                            *
- * Purpose: CurlHttpRequest.Delete method                                     *
+ * Purpose: HttpRequest.Delete / CurlHttpRequest.Delete method                *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_delete(duk_context *ctx)
@@ -392,7 +378,73 @@ static duk_ret_t	es_httprequest_delete(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Function: es_httprequest_set_proxy                                         *
+ * Purpose: HttpRequest.head method                                           *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	es_httprequest_head(duk_context *ctx)
+{
+	return es_httprequest_query(ctx, "HEAD");
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: HttpRequest.patch method                                          *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	es_httprequest_patch(duk_context *ctx)
+{
+	return es_httprequest_query(ctx, "PATCH");
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: HttpRequest.options method                                        *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	es_httprequest_options(duk_context *ctx)
+{
+	return es_httprequest_query(ctx, "OPTIONS");
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: HttpRequest.trace method                                          *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	es_httprequest_trace(duk_context *ctx)
+{
+	return es_httprequest_query(ctx, "TRACE");
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: HttpRequest.connect method                                        *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	es_httprequest_connect(duk_context *ctx)
+{
+	return es_httprequest_query(ctx, "CONNECT");
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: HttpRequest.customRequest method                                  *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	es_httprequest_customrequest(duk_context *ctx)
+{
+	const char	*method;
+
+	if (0 != duk_is_null_or_undefined(ctx, 0))
+		return duk_error(ctx, DUK_RET_EVAL_ERROR, "HTTP method cannot be undefined or null");
+
+	method = duk_to_string(ctx, 0);
+	duk_remove(ctx, 0);
+
+	return es_httprequest_query(ctx, method);
+}
+
+/******************************************************************************
  *                                                                            *
  * Purpose: CurlHttpRequest.SetProxy method                                   *
  *                                                                            *
@@ -415,8 +467,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: es_httprequest_status                                            *
  *                                                                            *
  * Purpose: CurlHttpRequest.Status method                                     *
  *                                                                            *
@@ -440,7 +490,29 @@ static duk_ret_t	es_httprequest_status(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Function: es_obj_put_http_header                                           *
+ * Purpose: retrieves value of a header                                       *
+ *                                                                            *
+ * Parameters: header    - [IN] the http header to extract value from         *
+ *             value_out - [OUT] the value                                    *
+ *                                                                            *
+ ******************************************************************************/
+static int	parse_header(char *header, char **value_out)
+{
+	char *value;
+
+	if (NULL == (value = strchr(header, ':')))
+		return FAIL;
+
+	*value++ = '\0';
+	while (' ' == *value || '\t' == *value)
+		value++;
+
+	*value_out = value;
+
+	return SUCCEED;
+}
+
+/******************************************************************************
  *                                                                            *
  * Purpose: puts http header <field>: <value> as object property/value        *
  *                                                                            *
@@ -453,13 +525,9 @@ static void	es_put_header(duk_context *ctx, int idx, char *header)
 {
 	char	*value;
 
-	if (NULL == (value = strchr(header, ':')))
+	if (FAIL == parse_header(header, &value))
 		return;
 
-	*value++ = '\0';
-	while (' ' == *value || '\t' == *value)
-		value++;
-
 	duk_push_string(ctx, value);
 
 	/* duk_put_prop_string() throws error on failure, no need to check return code */
@@ -468,19 +536,16 @@ static void	es_put_header(duk_context *ctx, int idx, char *header)
 
 /******************************************************************************
  *                                                                            *
- * Function: es_httprequest_get_headers                                       *
+ * Purpose: retrieve headers from request in form of strings                  *
  *                                                                            *
- * Purpose: CurlHttpRequest.GetHeaders method                                 *
+ * Parameters: ctx     - [IN] the duktape context                             *
+ *             request - [IN] the request to retrieve headers from            *
  *                                                                            *
  ******************************************************************************/
-static duk_ret_t	es_httprequest_get_headers(duk_context *ctx)
+static duk_ret_t	get_headers_as_strings(duk_context *ctx, zbx_es_httprequest_t *request)
 {
-	zbx_es_httprequest_t	*request;
-	duk_idx_t		idx;
 	char			*ptr, *header;
-
-	if (NULL == (request = es_httprequest(ctx)))
-		return duk_error(ctx, DUK_RET_EVAL_ERROR, "internal scripting error: null object");
+	duk_idx_t		idx;
 
 	idx = duk_push_object(ctx);
 
@@ -496,9 +561,128 @@ static duk_ret_t	es_httprequest_get_headers(duk_context *ctx)
 	return 1;
 }
 
+typedef struct
+{
+	char			*name;
+	zbx_vector_str_t	values;
+}
+zbx_cached_header_t;
+
+static void	cached_headers_free(zbx_cached_header_t *header)
+{
+	zbx_vector_str_clear_ext(&header->values, zbx_str_free);
+	zbx_vector_str_destroy(&header->values);
+	zbx_free(header->name);
+	zbx_free(header);
+}
+
 /******************************************************************************
  *                                                                            *
- * Function: es_httprequest_set_httpauth                                      *
+ * Purpose: retrieve headers from request in form of arrays                   *
+ *                                                                            *
+ * Parameters: ctx     - [IN] the duktape context                             *
+ *             request - [IN] the request to retrieve headers from            *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	get_headers_as_arrays(duk_context *ctx, zbx_es_httprequest_t *request)
+{
+	char			*ptr, *header;
+	zbx_vector_ptr_t	headers;
+	duk_idx_t		idx;
+	int			i, j;
+
+	zbx_vector_ptr_create(&headers);
+
+	idx = duk_push_object(ctx);
+
+	if (0 == request->headers_in_offset)
+		goto out;
+
+	for (ptr = request->headers_in; NULL != (header = zbx_http_parse_header(&ptr)); )
+	{
+		char			*value;
+		zbx_cached_header_t	*existing_header = NULL;
+
+		if (FAIL == parse_header(header, &value))
+		{
+			zbx_free(header);
+			continue;
+		}
+
+		for (j = 0; j < headers.values_num; j++)
+		{
+			zbx_cached_header_t *h = (zbx_cached_header_t*)headers.values[j];
+
+			if (0 == strcmp(header, h->name))
+			{
+				existing_header = h;
+				zbx_vector_str_append(&existing_header->values, zbx_strdup(NULL, value));
+				zbx_free(header);
+
+				break;
+			}
+		}
+
+		if (NULL == existing_header)
+		{
+			zbx_cached_header_t	*cached_header;
+
+			cached_header = zbx_malloc(NULL, sizeof(zbx_cached_header_t));
+
+			cached_header->name = header;
+			zbx_vector_str_create(&cached_header->values);
+			zbx_vector_str_append(&cached_header->values, zbx_strdup(NULL, value));
+			zbx_vector_ptr_append(&headers, cached_header);
+		}
+	}
+
+	for (i = 0; i < headers.values_num; i++) {
+		zbx_cached_header_t	*h = (zbx_cached_header_t*)headers.values[i];
+		duk_idx_t		arr_idx;
+
+		arr_idx = duk_push_array(ctx);
+
+		for (j = 0; j < h->values.values_num; j++)
+		{
+			duk_push_string(ctx, h->values.values[j]);
+			duk_put_prop_index(ctx, arr_idx, (duk_uarridx_t)j);
+		}
+
+		(void)duk_put_prop_string(ctx, idx, h->name);
+	}
+
+out:
+	zbx_vector_ptr_clear_ext(&headers, (zbx_mem_free_func_t)cached_headers_free);
+	zbx_vector_ptr_destroy(&headers);
+	return 1;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: HttpRequest.getHeaders / CurlHttpRequest.GetHeaders method        *
+ *                                                                            *
+ ******************************************************************************/
+static duk_ret_t	es_httprequest_get_headers(duk_context *ctx)
+{
+	zbx_es_httprequest_t	*request;
+
+	if (NULL == (request = es_httprequest(ctx)))
+		return duk_error(ctx, DUK_RET_EVAL_ERROR, "internal scripting error: null object");
+
+	if (0 == duk_is_null_or_undefined(ctx, 0))
+	{
+		duk_bool_t	as_array;
+
+		as_array = duk_to_boolean(ctx, 0);
+
+		if (0 != as_array)
+			return get_headers_as_arrays(ctx, request);
+	}
+
+	return get_headers_as_strings(ctx, request);
+}
+
+/******************************************************************************
  *                                                                            *
  * Purpose: CurlHttpRequest.SetHttpAuth method                                *
  *                                                                            *
@@ -543,6 +727,7 @@ static duk_ret_t	es_httprequest_set_httpauth(duk_context *ctx)
 
 	if (NULL != password)
 		ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_PASSWORD, password, err);
+
 out:
 	zbx_free(password);
 	zbx_free(username);
@@ -553,7 +738,7 @@ out:
 	return 0;
 }
 
-static const duk_function_list_entry	httprequest_methods[] = {
+static const duk_function_list_entry	curlhttprequest_methods[] = {
 	{"AddHeader", es_httprequest_add_header, 1},
 	{"ClearHeader", es_httprequest_clear_header, 0},
 	{"Get", es_httprequest_get, 2},
@@ -562,11 +747,31 @@ static const duk_function_list_entry	httprequest_methods[] = {
 	{"Delete", es_httprequest_delete, 2},
 	{"Status", es_httprequest_status, 0},
 	{"SetProxy", es_httprequest_set_proxy, 1},
-	{"GetHeaders", es_httprequest_get_headers, 0},
+	{"GetHeaders", es_httprequest_get_headers, 1},
 	{"SetHttpAuth", es_httprequest_set_httpauth, 3},
 	{NULL, NULL, 0}
 };
 
+static const duk_function_list_entry	httprequest_methods[] = {
+	{"addHeader", es_httprequest_add_header, 1},
+	{"clearHeader", es_httprequest_clear_header, 0},
+	{"get", es_httprequest_get, 2},
+	{"put", es_httprequest_put, 2},
+	{"post", es_httprequest_post, 2},
+	{"delete", es_httprequest_delete, 2},
+	{"head", es_httprequest_head, 2},
+	{"patch", es_httprequest_patch, 2},
+	{"options", es_httprequest_options, 2},
+	{"trace", es_httprequest_trace, 2},
+	{"connect", es_httprequest_connect, 2},
+	{"getStatus", es_httprequest_status, 0},
+	{"setProxy", es_httprequest_set_proxy, 1},
+	{"getHeaders", es_httprequest_get_headers, 1},
+	{"setHttpAuth", es_httprequest_set_httpauth, 3},
+	{"customRequest", es_httprequest_customrequest, 3},
+	{NULL, NULL, 0}
+};
+
 #else
 
 static duk_ret_t	es_httprequest_ctor(duk_context *ctx)
@@ -580,19 +785,22 @@ static duk_ret_t	es_httprequest_ctor(duk_context *ctx)
 static const duk_function_list_entry	httprequest_methods[] = {
 	{NULL, NULL, 0}
 };
+
+static const duk_function_list_entry	*curlhttprequest_methods = httprequest_methods;
 #endif
 
-static int	es_httprequest_create_prototype(duk_context *ctx)
+static int	es_httprequest_create_prototype(duk_context *ctx, const char *obj_name,
+		const duk_function_list_entry *methods)
 {
 	duk_push_c_function(ctx, es_httprequest_ctor, 0);
 	duk_push_object(ctx);
 
-	duk_put_function_list(ctx, -1, httprequest_methods);
+	duk_put_function_list(ctx, -1, methods);
 
 	if (1 != duk_put_prop_string(ctx, -2, "prototype"))
 		return FAIL;
 
-	if (1 != duk_put_global_string(ctx, "CurlHttpRequest"))
+	if (1 != duk_put_global_string(ctx, obj_name))
 		return FAIL;
 
 	return SUCCEED;
@@ -606,7 +814,8 @@ int	zbx_es_init_httprequest(zbx_es_t *es, char **error)
 		return FAIL;
 	}
 
-	if (FAIL == es_httprequest_create_prototype(es->env->ctx))
+	if (FAIL == es_httprequest_create_prototype(es->env->ctx, "CurlHttpRequest", curlhttprequest_methods) ||
+			FAIL == es_httprequest_create_prototype(es->env->ctx, "HttpRequest", httprequest_methods))
 	{
 		*error = zbx_strdup(*error, duk_safe_to_string(es->env->ctx, -1));
 		duk_pop(es->env->ctx);
