@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,6 +20,10 @@
 #ifndef ZABBIX_HISTORY_H
 #define ZABBIX_HISTORY_H
 
+#include "zbxjson.h"
+#include "zbxalgo.h"
+#include "zbxhistory.h"
+
 #define ZBX_HISTORY_IFACE_SQL		0
 #define ZBX_HISTORY_IFACE_ELASTIC	1
 
@@ -31,12 +35,17 @@ typedef int (*zbx_history_get_values_func_t)(struct zbx_history_iface *hist, zbx
 		int count, int end, zbx_vector_history_record_t *values);
 typedef int (*zbx_history_flush_func_t)(struct zbx_history_iface *hist);
 
+typedef void (*zbx_history_func_t)(const zbx_vector_ptr_t *);
+
 struct zbx_history_iface
 {
 	unsigned char			value_type;
 	unsigned char			requires_trends;
-	void				*data;
-
+	union
+	{
+		void				*elastic_data;
+		zbx_history_func_t		sql_history_func;
+	} data;
 	zbx_history_destroy_func_t	destroy;
 	zbx_history_add_values_func_t	add_values;
 	zbx_history_get_values_func_t	get_values;
@@ -48,5 +57,7 @@ int	zbx_history_sql_init(zbx_history_iface_t *hist, unsigned char value_type, ch
 
 /* elastic hist */
 int	zbx_history_elastic_init(zbx_history_iface_t *hist, unsigned char value_type, char **error);
+void	zbx_elastic_version_extract(struct zbx_json *json);
+zbx_uint32_t	zbx_elastic_version_get(void);
 
 #endif
