@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 **/
 
 #include "zbxicmpping.h"
+
 #include "threads.h"
 #include "comms.h"
 #include "zbxexec.h"
@@ -59,8 +60,6 @@ static void	get_source_ip_option(const char *fping, const char **option, unsigne
 
 	zbx_snprintf(tmp, sizeof(tmp), "%s -h 2>&1", fping);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "executing %s", tmp);
-
 	if (NULL == (f = popen(tmp, "r")))
 		return;
 
@@ -87,116 +86,6 @@ static void	get_source_ip_option(const char *fping, const char **option, unsigne
 	*checked = 1;
 }
 
-/******************************************************************************
- *                                                                            *
- * Purpose: execute external program and return stdout and stderr values      *
- *                                                                            *
- * Parameters: fping         - [IN] location of fping program                 *
- *             out           - [OUT] stdout and stderr values                 *
- *             error         - [OUT] error string if function fails           *
- *             max_error_len - [IN] length of error buffer                    *
- *                                                                            *
- * Return value: SUCCEED if processed successfully or FAIL otherwise          *
- *                                                                            *
- ******************************************************************************/
-static int	get_fping_out(const char *fping, const char *address, char **out, char *error, size_t max_error_len)
-{
-	FILE		*f;
-	size_t		buf_size = 0, offset = 0, len;
-	ssize_t		n;
-	char		tmp[MAX_STRING_LEN], *buffer = NULL;
-	int		ret = FAIL, fd;
-	sigset_t	mask, orig_mask;
-	char		filename[MAX_STRING_LEN];
-	mode_t		mode;
-
-	if (FAIL == zbx_validate_hostname(address) && FAIL == is_supported_ip(address))
-	{
-		zbx_strlcpy(error, "Invalid host name or IP address", max_error_len);
-		return FAIL;
-	}
-
-	zbx_snprintf(filename, sizeof(filename), "%s/%s_XXXXXX", CONFIG_TMPDIR, progname);
-
-	mode = umask(077);
-	fd = mkstemp(filename);
-	umask(mode);
-
-	if (-1 == fd)
-	{
-		zbx_snprintf(error, max_error_len, "Cannot create temporary file \"%s\": %s", filename,
-				zbx_strerror(errno));
-
-		return FAIL;
-	}
-
-	sigemptyset(&mask);
-	sigaddset(&mask, SIGINT);
-	sigaddset(&mask, SIGQUIT);
-
-	len = strlen(address);
-	if (-1 == (n = write(fd, address, len)))
-	{
-		zbx_snprintf(error, max_error_len, "Cannot write address into temporary file: %s", zbx_strerror(errno));
-		(void)close(fd);
-		goto out;
-	}
-
-	if (n != (ssize_t)len)
-	{
-		zbx_strlcpy(error, "Cannot write full address into temporary file", max_error_len);
-		(void)close(fd);
-		goto out;
-	}
-
-	if (-1 == close(fd))
-	{
-		zbx_snprintf(error, max_error_len, "Cannot close temporary file: %s", zbx_strerror(errno));
-		goto out;
-	}
-
-	zbx_snprintf(tmp, sizeof(tmp), "%s 2>&1 < %s", fping, filename);
-
-	if (0 > sigprocmask(SIG_BLOCK, &mask, &orig_mask))
-		zbx_error("cannot set sigprocmask to block the user signal");
-
-	zabbix_log(LOG_LEVEL_DEBUG, "executing %s", tmp);
-
-	if (NULL == (f = popen(tmp, "r")))
-	{
-		zbx_strlcpy(error, zbx_strerror(errno), max_error_len);
-		goto out;
-	}
-
-	while (NULL != zbx_fgets(tmp, sizeof(tmp), f))
-	{
-		len = strlen(tmp);
-
-		if (MAX_EXECUTE_OUTPUT_LEN < offset + len)
-			break;
-
-		zbx_strncpy_alloc(&buffer, &buf_size, &offset, tmp, len);
-	}
-
-	pclose(f);
-
-	if (NULL == buffer)
-	{
-		zbx_strlcpy(error, "Cannot obtain the program output", max_error_len);
-		goto out;
-	}
-
-	*out = buffer;
-	ret = SUCCEED;
-out:
-	unlink(filename);
-
-	if (0 > sigprocmask(SIG_SETMASK, &orig_mask, NULL))
-		zbx_error("cannot restore sigprocmask");
-
-	return ret;
-}
-
 /******************************************************************************
  *                                                                            *
  * Purpose: detect minimal possible fping packet interval                     *
@@ -241,17 +130,25 @@ static int	get_interval_option(const char *fping, ZBX_FPING_HOST *hosts, int hos
 
 		for (j = 0; j < ARRSIZE(intervals); j++)
 		{
+			int		ret_exec;
 			char		tmp[MAX_STRING_LEN], err[255];
 			const char	*p;
 
 			zabbix_log(LOG_LEVEL_DEBUG, "testing fping interval %u ms", intervals[j]);
 
-			zbx_snprintf(tmp, sizeof(tmp), "%s -c1 -t50 -i%u", fping, intervals[j]);
+			zbx_snprintf(tmp, sizeof(tmp), "%s -c1 -t50 -i%u %s", fping, intervals[j], dst);
 
 			zbx_free(out);
 
 			/* call fping, ignore its exit code but mind execution failures */
-			if (SUCCEED != get_fping_out(tmp, dst, &out, err, sizeof(err)))
+			if (TIMEOUT_ERROR == (ret_exec = zbx_execute(tmp, &out, err, sizeof(err), 1,
+					ZBX_EXIT_CODE_CHECKS_DISABLED, NULL)))
+			{
+				zbx_snprintf(error, max_error_len, "Timeout while executing \"%s\"", tmp);
+				goto out;
+			}
+
+			if (FAIL == ret_exec)
 			{
 				zbx_snprintf(error, max_error_len, "Cannot execute \"%s\": %s", tmp, err);
 				goto out;
@@ -336,8 +233,6 @@ out:
 
 #ifdef HAVE_IPV6
 /******************************************************************************
- *                                                                            *
- * Function: get_ipv6_support                                                 *
  *                                                                            *
  * Purpose: check fping supports IPv6                                         *
  *                                                                            *
@@ -353,10 +248,10 @@ static int	get_ipv6_support(const char * fping, const char *dst)
 	int	ret;
 	char	tmp[MAX_STRING_LEN], error[255], *out = NULL;
 
-	zbx_snprintf(tmp, sizeof(tmp), "%s -6 -c1 -t50", fping);
+	zbx_snprintf(tmp, sizeof(tmp), "%s -6 -c1 -t50 %s", fping, dst);
 
-	if (SUCCEED == (ret = get_fping_out(tmp, dst, &out, error, sizeof(error)) &&
-				ZBX_KIBIBYTE > strlen(out) && NULL != strstr(out, dst)))
+	if ((SUCCEED == (ret = zbx_execute(tmp, &out, error, sizeof(error), 1, ZBX_EXIT_CODE_CHECKS_DISABLED, NULL)) &&
+				ZBX_KIBIBYTE > strlen(out) && NULL != strstr(out, dst)) || TIMEOUT_ERROR == ret)
 	{
 		ret = SUCCEED;
 	}
@@ -640,7 +535,7 @@ static int	process_ping(ZBX_FPING_HOST *hosts, int hosts_count, int count, int i
 
 	fclose(f);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "executing %s", tmp);
+	zabbix_log(LOG_LEVEL_DEBUG, "%s", tmp);
 
 	sigemptyset(&mask);
 	sigaddset(&mask, SIGINT);
@@ -807,8 +702,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_ping                                                         *
  *                                                                            *
  * Purpose: ping hosts listed in the host files                               *
  *                                                                            *
@@ -829,8 +722,6 @@ out:
  * Return value: SUCCEED - successfully processed hosts                       *
  *               NOTSUPPORTED - otherwise                                     *
  *                                                                            *
- * Author: Alexei Vladishev                                                   *
- *                                                                            *
  * Comments: use external binary 'fping' to avoid superuser privileges        *
  *                                                                            *
  ******************************************************************************/
