@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,21 +17,18 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
-#include "mutexs.h"
-#include "log.h"
-
 #include "memalloc.h"
 
+#include "common.h"
+#include "log.h"
+
 /******************************************************************************
  *                                                                            *
  *                     Some information on memory layout                      *
  *                  ---------------------------------------                   *
  *                                                                            *
- *                                                                            *
  * (*) chunk: a contiguous piece of memory that is either free or used        *
  *                                                                            *
- *                                                                            *
  *                    +-------- size of + --------------+                     *
  *                    |       (8 bytes) |               |                     *
  *                    |                 v               |                     *
@@ -47,7 +44,6 @@
  *                                                                            *
  *                     8-aligned               8-aligned                      *
  *                                                                            *
- *                                                                            *
  *     when a chunk is used, `size' fields have MEM_FLG_USED bit set          *
  *                                                                            *
  *     when a chunk is free, the first 2 * ZBX_PTR_SIZE bytes of allocatable  *
@@ -61,12 +57,10 @@
  *           (when freeing a chunk, we can quickly see if the previous        *
  *           and next chunks are free, those will not have MEM_FLG_USED)      *
  *                                                                            *
- *                                                                            *
  * (*) free chunks are stored in doubly-linked lists according to their sizes *
  *                                                                            *
  *     a typical situation is thus as follows (1 used chunk, 2 free chunks)   *
  *                                                                            *
- *                                                                            *
  *  +--------------------------- shared memory ----------------------------+  *
  *  |                         (can be misaligned)                          |  *
  *  |                                                                      |  *
@@ -580,6 +574,7 @@ int	zbx_mem_create(zbx_mem_info_t **info, zbx_uint64_t size, const char *descr,
 	/* allocate zbx_mem_info_t structure, its buckets, and description inside shared memory */
 
 	*info = (zbx_mem_info_t *)ALIGN8(base);
+	(*info)->base = base;
 	(*info)->shm_id = shm_id;
 	(*info)->orig_size = size;
 	size -= (char *)(*info + 1) - (char *)base;
@@ -629,6 +624,11 @@ out:
 	return ret;
 }
 
+void	zbx_mem_destroy(zbx_mem_info_t *info)
+{
+	(void)shmdt(info->base);
+}
+
 void	*__zbx_mem_malloc(const char *file, int line, zbx_mem_info_t *info, const void *old, size_t size)
 {
 	void	*chunk;
