@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,9 +17,9 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
 #include "daemon.h"
 
+#include "common.h"
 #include "pid.h"
 #include "cfg.h"
 #include "log.h"
@@ -46,8 +46,6 @@ static void	(*zbx_sigusr_handler)(int flags);
 
 #ifdef HAVE_SIGQUEUE
 /******************************************************************************
- *                                                                            *
- * Function: common_sigusr_handler                                            *
  *                                                                            *
  * Purpose: common SIGUSR1 handler for Zabbix processes                       *
  *                                                                            *
@@ -87,12 +85,14 @@ static void	common_sigusr_handler(int flags)
 	}
 }
 
-static void	zbx_signal_process_by_type(int proc_type, int proc_num, int flags)
+void	zbx_signal_process_by_type(int proc_type, int proc_num, int flags, char **out)
 {
-	int		process_num, found = 0, i;
+	int		process_num, found = 0, i, failed_num = 0;
 	union sigval	s;
 	unsigned char	process_type;
+	size_t		out_alloc = 0, out_offset = 0;
 
+	s.sival_ptr = NULL;
 	s.ZBX_SIVAL_INT = flags;
 
 	for (i = 0; i < threads_num; i++)
@@ -120,53 +120,70 @@ static void	zbx_signal_process_by_type(int proc_type, int proc_num, int flags)
 					" pid:%d", get_process_type_string(process_type), threads[i]);
 		}
 		else
+		{
 			zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal: %s", zbx_strerror(errno));
+			failed_num++;
+		}
 	}
 
 	if (0 == found)
 	{
 		if (0 == proc_num)
 		{
-			zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal:"
+			zbx_strlog_alloc(LOG_LEVEL_ERR, out, &out_alloc, &out_offset, "cannot redirect signal:"
 					" \"%s\" process does not exist",
 					get_process_type_string(proc_type));
 		}
 		else
 		{
-			zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal:"
+			zbx_strlog_alloc(LOG_LEVEL_ERR, out, &out_alloc, &out_offset, "cannot redirect signal:"
 					" \"%s #%d\" process does not exist",
 					get_process_type_string(proc_type), proc_num);
 		}
 	}
+	else
+	{
+		if (0 != failed_num && NULL != out)
+			*out = zbx_strdup(*out, "failed to redirect remote control signal(s)");
+	}
 }
 
-static void	zbx_signal_process_by_pid(int pid, int flags)
+void	zbx_signal_process_by_pid(int pid, int flags, char **out)
 {
 	union sigval	s;
-	int		i, found = 0;
+	int		i, found = 0, failed_num = 0;
+	size_t		out_alloc = 0, out_offset = 0;
 
+	s.sival_ptr = NULL;
 	s.ZBX_SIVAL_INT = flags;
 
 	for (i = 0; i < threads_num; i++)
 	{
-		if (0 != pid && threads[i] != ZBX_RTC_GET_DATA(flags))
+		if ((0 != pid && threads[i] != pid) || 0 == threads[i])
 			continue;
 
 		found = 1;
 
 		if (-1 != sigqueue(threads[i], SIGUSR1, s))
 		{
-			zabbix_log(LOG_LEVEL_DEBUG, "the signal was redirected to process pid:%d",
-					threads[i]);
+			zabbix_log(LOG_LEVEL_DEBUG, "the signal was redirected to process pid:%d",	threads[i]);
 		}
 		else
+		{
 			zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal: %s", zbx_strerror(errno));
+			failed_num++;
+		}
 	}
 
-	if (0 != ZBX_RTC_GET_DATA(flags) && 0 == found)
+	if (0 != pid && 0 == found)
 	{
-		zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal: process pid:%d is not a Zabbix child"
-				" process", ZBX_RTC_GET_DATA(flags));
+		zbx_strlog_alloc(LOG_LEVEL_DEBUG, out, &out_alloc, &out_offset,
+				"cannot redirect signal: process pid:%d is not a Zabbix child process", pid);
+	}
+	else
+	{
+		if (0 != failed_num && NULL != out)
+			*out = zbx_strdup(*out, "failed to redirect remote control signal(s)");
 	}
 }
 
@@ -178,8 +195,6 @@ void	zbx_set_sigusr_handler(void (*handler)(int flags))
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: user1_signal_handler                                             *
  *                                                                            *
  * Purpose: handle user signal SIGUSR1                                        *
  *                                                                            *
@@ -188,6 +203,7 @@ static void	user1_signal_handler(int sig, siginfo_t *siginfo, void *context)
 {
 #ifdef HAVE_SIGQUEUE
 	int	flags;
+	int	scope;
 #endif
 	SIG_CHECK_PARAMS(sig, siginfo, context);
 
@@ -208,39 +224,44 @@ static void	user1_signal_handler(int sig, siginfo_t *siginfo, void *context)
 
 	if (NULL == threads)
 	{
-		zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal: shutdown in progress");
+		zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal: server is either shutting down"
+				" or is running in standby mode");
+		return;
+	}
+
+	if (0 == (program_type & ZBX_PROGRAM_TYPE_AGENTD))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "cannot redirect signal: runtime control signals are supported only by agent");
 		return;
 	}
 
 	switch (ZBX_RTC_GET_MSG(flags))
 	{
-		case ZBX_RTC_CONFIG_CACHE_RELOAD:
-			if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
-			{
-				zabbix_log(LOG_LEVEL_WARNING, "forced reloading of the configuration cache"
-						" cannot be performed for a passive proxy");
-				return;
-			}
-			ZBX_FALLTHROUGH;
-		case ZBX_RTC_SECRETS_RELOAD:
-			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_CONFSYNCER, 1, flags);
-			break;
-		case ZBX_RTC_HOUSEKEEPER_EXECUTE:
-			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_HOUSEKEEPER, 1, flags);
-			break;
 		case ZBX_RTC_LOG_LEVEL_INCREASE:
 		case ZBX_RTC_LOG_LEVEL_DECREASE:
-			if ((ZBX_RTC_LOG_SCOPE_FLAG | ZBX_RTC_LOG_SCOPE_PID) == ZBX_RTC_GET_SCOPE(flags))
-				zbx_signal_process_by_pid(ZBX_RTC_GET_DATA(flags), flags);
+			scope = ZBX_RTC_GET_SCOPE(flags);
+
+			if ((ZBX_RTC_LOG_SCOPE_FLAG | ZBX_RTC_LOG_SCOPE_PID) == scope)
+			{
+				zbx_signal_process_by_pid(ZBX_RTC_GET_DATA(flags), flags, NULL);
+			}
 			else
-				zbx_signal_process_by_type(ZBX_RTC_GET_SCOPE(flags), ZBX_RTC_GET_DATA(flags), flags);
+			{
+				if (scope < ZBX_PROCESS_TYPE_EXT_FIRST)
+				{
+					zbx_signal_process_by_type(ZBX_RTC_GET_SCOPE(flags), ZBX_RTC_GET_DATA(flags),
+							flags, NULL);
+				}
+			}
+
+			/* call custom sigusr handler to handle log level changes for non worker processes */
+			if (NULL != zbx_sigusr_handler)
+				zbx_sigusr_handler(flags);
+
 			break;
-		case ZBX_RTC_SNMP_CACHE_RELOAD:
-			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_UNREACHABLE, ZBX_RTC_GET_DATA(flags), flags);
-			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_POLLER, ZBX_RTC_GET_DATA(flags), flags);
-			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_TRAPPER, ZBX_RTC_GET_DATA(flags), flags);
-			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_DISCOVERER, ZBX_RTC_GET_DATA(flags), flags);
-			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_TASKMANAGER, ZBX_RTC_GET_DATA(flags), flags);
+		case ZBX_RTC_USER_PARAMETERS_RELOAD:
+			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_ACTIVE_CHECKS, ZBX_RTC_GET_DATA(flags), flags, NULL);
+			zbx_signal_process_by_type(ZBX_PROCESS_TYPE_LISTENER, ZBX_RTC_GET_DATA(flags), flags, NULL);
 			break;
 		default:
 			if (NULL != zbx_sigusr_handler)
@@ -250,8 +271,6 @@ static void	user1_signal_handler(int sig, siginfo_t *siginfo, void *context)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: pipe_signal_handler                                              *
  *                                                                            *
  * Purpose: handle pipe signal SIGPIPE                                        *
  *                                                                            *
@@ -266,8 +285,6 @@ static void	pipe_signal_handler(int sig, siginfo_t *siginfo, void *context)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: set_daemon_signal_handlers                                       *
  *                                                                            *
  * Purpose: set the signal handlers used by daemons                           *
  *                                                                            *
@@ -289,8 +306,6 @@ static void	set_daemon_signal_handlers(void)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: daemon_start                                                     *
  *                                                                            *
  * Purpose: init process as daemon                                            *
  *                                                                            *
@@ -299,14 +314,11 @@ static void	set_daemon_signal_handlers(void)
  *                          privileges                                        *
  *             flags      - daemon startup flags                              *
  *                                                                            *
- * Author: Alexei Vladishev                                                   *
- *                                                                            *
  * Comments: it doesn't allow running under 'root' if allow_root is zero      *
  *                                                                            *
  ******************************************************************************/
 int	daemon_start(int allow_root, const char *user, unsigned int flags)
 {
-	pid_t		pid;
 	struct passwd	*pwd;
 
 	if (0 == allow_root && 0 == getuid())	/* running as root? */
@@ -363,14 +375,14 @@ int	daemon_start(int allow_root, const char *user, unsigned int flags)
 
 	if (0 == (flags & ZBX_TASK_FLAG_FOREGROUND))
 	{
-		if (0 != (pid = zbx_fork()))
+		if (0 != zbx_fork())
 			exit(EXIT_SUCCESS);
 
 		setsid();
 
 		signal(SIGHUP, SIG_IGN);
 
-		if (0 != (pid = zbx_fork()))
+		if (0 != zbx_fork())
 			exit(EXIT_SUCCESS);
 
 		if (-1 == chdir("/"))	/* this is to eliminate warning: ignoring return value of chdir */
@@ -420,6 +432,7 @@ int	zbx_sigusr_send(int flags)
 	{
 		union sigval	s;
 
+		s.sival_ptr = NULL;
 		s.ZBX_SIVAL_INT = flags;
 
 		if (-1 != sigqueue(pid, SIGUSR1, s))
@@ -434,6 +447,7 @@ int	zbx_sigusr_send(int flags)
 		}
 	}
 #else
+	ZBX_UNUSED(flags);
 	zbx_snprintf(error, sizeof(error), "operation is not supported on the given operating system");
 #endif
 	if (SUCCEED != ret)
