@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -112,46 +112,7 @@ extern "C" void	zbx_co_uninitialize()
 		CoUninitialize();
 }
 
-extern "C" static void	get_error_code_text(HRESULT hres, char **error)
-{
-	IWbemStatusCodeText	*pStatus = NULL;
-	SCODE			sc;
-
-	sc = CoCreateInstance(CLSID_WbemStatusCodeText, 0, CLSCTX_INPROC_SERVER, IID_IWbemStatusCodeText,
-			(LPVOID *) &pStatus);
-
-	if(S_OK == sc)
-	{
-		BSTR	bstr = 0;
-
-		sc = pStatus->GetErrorCodeText(hres, 0, 0, &bstr);
-		if (S_OK == sc)
-		{
-			*error = zbx_unicode_to_utf8((wchar_t *)bstr);
-			zbx_rtrim(*error, "\n\r");
-			SysFreeString(bstr);
-		}
-		else
-		{
-			*error = zbx_dsprintf(*error, "error code:" ZBX_FS_I64, hres);
-			zabbix_log(LOG_LEVEL_DEBUG, "GetErrorCodeText() failed with code:" ZBX_FS_I64 " when retrieving error"
-					" code for " ZBX_FS_I64, sc, hres);
-		}
-	}
-	else
-	{
-		*error = zbx_dsprintf(*error, "error code:" ZBX_FS_I64, hres);
-		zabbix_log(LOG_LEVEL_DEBUG, "CoCreateInstance() failed with code:" ZBX_FS_I64 " when retrieving error code"
-				" for:" ZBX_FS_I64, sc, hres);
-	}
-
-	if (NULL != pStatus)
-		pStatus->Release();
-}
-
 /******************************************************************************
- *                                                                            *
- * Function: parse_first_first                                                *
  *                                                                            *
  * Purpose: extract only one value from the search result                     *
  *                                                                            *
@@ -161,7 +122,7 @@ extern "C" static void	get_error_code_text(HRESULT hres, char **error)
  *             error       - [OUT] the error description                      *
  *                                                                            *
  * Return value: SYSINFO_RET_OK   - wmi_values contains the retrieved value   *
- *               SYSINFO_RET_FAIL - retreiving WMI value failed               *
+ *               SYSINFO_RET_FAIL - retrieving WMI value failed               *
  *                                                                            *
  * Comments: one value is the value from the first property of the first      *
  *           instance from search result                                      *
@@ -186,13 +147,7 @@ extern "C" static int	parse_first_first(IEnumWbemClassObject *pEnumerator, doubl
 		goto out2;
 	}
 
-	if (FAILED(hres))
-	{
-		get_error_code_text(hres, error);
-		goto out2;
-	}
-
-	if (0 == uReturn)
+	if (FAILED(hres) || 0 == uReturn)
 		goto out2;
 
 	hres = pclsObj->BeginEnumeration(WBEM_FLAG_NONSYSTEM_ONLY);
@@ -237,8 +192,6 @@ out2:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: parse_all                                                        *
  *                                                                            *
  * Purpose: extract all values from the search result                         *
  *                                                                            *
@@ -248,7 +201,7 @@ out2:
  *             error       - [OUT] the error description                      *
  *                                                                            *
  * Return value: SYSINFO_RET_OK   - wmi_values contains the retrieved values  *
- *               SYSINFO_RET_FAIL - retreiving WMI value failed               *
+ *               SYSINFO_RET_FAIL - retrieving WMI value failed               *
  *                                                                            *
  ******************************************************************************/
 extern "C" static int	parse_all(IEnumWbemClassObject *pEnumerator, double timeout,
@@ -276,13 +229,7 @@ extern "C" static int	parse_all(IEnumWbemClassObject *pEnumerator, double timeou
 		if (WBEM_S_FALSE == hres && 0 == uReturn)
 			return SYSINFO_RET_OK;
 
-		if (FAILED(hres))
-		{
-			get_error_code_text(hres, error);
-			return ret;
-		}
-
-		if (0 == uReturn)
+		if (FAILED(hres) || 0 == uReturn)
 			return ret;
 
 		hres = pclsObj->BeginEnumeration(WBEM_FLAG_NONSYSTEM_ONLY);
@@ -330,8 +277,6 @@ extern "C" static int	parse_all(IEnumWbemClassObject *pEnumerator, double timeou
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_wmi_get_variant                                              *
  *                                                                            *
  * Purpose: retrieves WMI value and stores it in the provided memory location *
  *                                                                            *
@@ -421,8 +366,6 @@ exit:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_wmi_get                                                      *
  *                                                                            *
  * Purpose: wrapper function for zbx_wmi_get_variant(), stores the retrieved  *
  *          WMI value as UTF-8 encoded string                                 *
@@ -478,8 +421,6 @@ out:
 
 
 /******************************************************************************
- *                                                                            *
- * Function: WMI_GET                                                          *
  *                                                                            *
  * Purpose: wrapper function for wmi.get metric                               *
  *                                                                            *
@@ -487,7 +428,7 @@ out:
  *             result  - [OUT] one value of property from WMI Class           *
  *                                                                            *
  * Return value: SYSINFO_RET_OK   - result contains the retrieved WMI value   *
- *               SYSINFO_RET_FAIL - retreiving WMI value failed               *
+ *               SYSINFO_RET_FAIL - retrieving WMI value failed               *
  *                                                                            *
  ******************************************************************************/
 extern "C" int	WMI_GET(AGENT_REQUEST *request, AGENT_RESULT *result)
@@ -598,8 +539,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: proc_arr_element                                                 *
  *                                                                            *
  * Purpose: take one element from array and put value to JSON document        *
  *                                                                            *
@@ -734,8 +673,6 @@ extern "C" int	proc_arr_element(SAFEARRAY *sa, LONG *index, const char *prop_err
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: convert_wmiarray_json                                            *
  *                                                                            *
  * Purpose: transformation of variant array from WMI search result to JSON    *
  *                                                                            *
@@ -805,8 +742,6 @@ extern "C" int	convert_wmiarray_json(VARIANT *vtProp, const char *prop_name, ULO
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: put_variant_json                                                 *
  *                                                                            *
  * Purpose: copy value of VARIANT type to JSON document                       *
  *                                                                            *
@@ -898,8 +833,6 @@ extern "C" int	put_variant_json(const char *prop_json, const char *prop_err, VAR
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: convert_wmi_json                                                 *
  *                                                                            *
  * Purpose: transformation of WMI search result to JSON                       *
  *                                                                            *
@@ -950,8 +883,6 @@ extern "C" int	convert_wmi_json(zbx_vector_wmi_instance_t *wmi_values, char **js
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: WMI_GETALL                                                       *
  *                                                                            *
  * Purpose: wrapper function for wmi.getall metric                            *
  *                                                                            *
