@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,15 +17,14 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
-#include "stats.h"
 #include "perfstat.h"
+
+#include "stats.h"
 #include "alias.h"
 #include "log.h"
 #include "mutexs.h"
 #include "sysinfo.h"
 
-#define UNSUPPORTED_REFRESH_PERIOD	600
 #define OBJECT_CACHE_REFRESH_INTERVAL	60
 #define NAMES_UPDATE_INTERVAL		60
 
@@ -39,7 +38,6 @@ typedef struct
 {
 	zbx_perf_counter_data_t	*pPerfCounterList;
 	PDH_HQUERY		pdh_query;
-	time_t			nextcheck;		/* refresh time of not supported counters */
 	time_t			lastrefresh_objects;	/* last refresh time of object cache */
 	time_t			lastupdate_names;	/* last update time of object names */
 }
@@ -77,7 +75,7 @@ static void	deactivate_perf_counter(zbx_perf_counter_data_t *counter)
 
 /******************************************************************************
  *                                                                            *
- * Comments: if the specified counter exists or a new is successfully         *
+ * Comments: if the specified counter exists or the new one is successfully   *
  *           added, a pointer to that counter is returned, NULL otherwise     *
  *                                                                            *
  ******************************************************************************/
@@ -165,8 +163,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: extend_perf_counter_interval                                     *
  *                                                                            *
  * Purpose: extends the performance counter buffer to store the new data      *
  *          interval                                                          *
@@ -213,8 +209,6 @@ static void	free_object_names(void)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: set_object_names                                                 *
  *                                                                            *
  * Purpose: obtains PDH object localized names and associates them with       *
  *          English names, to be used by perf_instance_en.discovery           *
@@ -451,7 +445,6 @@ int	init_perf_collector(zbx_threadedness_t threadedness, char **error)
 		goto out;
 	}
 
-	ppsd.nextcheck = time(NULL) + UNSUPPORTED_REFRESH_PERIOD;
 	ppsd.lastrefresh_objects = 0;
 	ppsd.lastupdate_names = 0;
 
@@ -523,18 +516,13 @@ void	collect_perfstat(void)
 	now = time(NULL);
 
 	/* refresh unsupported counters */
-	if (ppsd.nextcheck <= now)
+	for (cptr = ppsd.pPerfCounterList; NULL != cptr; cptr = cptr->next)
 	{
-		for (cptr = ppsd.pPerfCounterList; NULL != cptr; cptr = cptr->next)
- 		{
-			if (PERF_COUNTER_NOTSUPPORTED != cptr->status)
-				continue;
+		if (PERF_COUNTER_NOTSUPPORTED != cptr->status)
+			continue;
 
-			zbx_PdhAddCounter(__func__, cptr, ppsd.pdh_query, cptr->counterpath,
-					cptr->lang, &cptr->handle);
-		}
-
-		ppsd.nextcheck = now + UNSUPPORTED_REFRESH_PERIOD;
+		zbx_PdhAddCounter(__func__, cptr, ppsd.pdh_query, cptr->counterpath,
+				cptr->lang, &cptr->handle);
 	}
 
 	/* query for new data */
@@ -634,8 +622,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: get_perf_counter_value_by_name                                   *
  *                                                                            *
  * Purpose: gets average named performance counter value                      *
  *                                                                            *
@@ -713,8 +699,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: get_perf_counter_value_by_path                                   *
  *                                                                            *
  * Purpose: gets average performance counter value                            *
  *                                                                            *
@@ -785,8 +769,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: get_perf_counter_value                                           *
  *                                                                            *
  * Purpose: gets average value of the specified performance counter interval  *
  *                                                                            *
@@ -886,8 +868,6 @@ static wchar_t	*get_object_name(char *eng_name)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: get_object_name_local                                            *
  *                                                                            *
  * Purpose: get localized name of the object                                  *
  *                                                                            *
