@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,25 +17,21 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
 #include "zbxconf.h"
 
-#include "cfg.h"
 #include "log.h"
 #include "alias.h"
 #include "sysinfo.h"
+
 #ifdef _WINDOWS
 #	include "perfstat.h"
 #endif
-#include "comms.h"
 
 /******************************************************************************
- *                                                                            *
- * Function: load_aliases                                                     *
  *                                                                            *
  * Purpose: load aliases from configuration                                   *
  *                                                                            *
- * Parameters: lines - aliase entries from configuration file                 *
+ * Parameters: lines - aliases from configuration file                        *
  *                                                                            *
  * Comments: calls add_alias() for each entry                                 *
  *                                                                            *
@@ -74,19 +70,19 @@ void	load_aliases(char **lines)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: load_user_parameters                                             *
  *                                                                            *
  * Purpose: load user parameters from configuration                           *
  *                                                                            *
  * Parameters: lines - user parameter entries from configuration file         *
+ *             error - error message                                          *
  *                                                                            *
- * Author: Vladimir Levijev                                                   *
+ * Return value: SUCCEED - successfully loaded user parameters                *
+ *               FAIL    - failed to load user parameters                     *
  *                                                                            *
  * Comments: calls add_user_parameter() for each entry                        *
  *                                                                            *
  ******************************************************************************/
-void	load_user_parameters(char **lines)
+int	load_user_parameters(char **lines, char **err)
 {
 	char	*p, **pline, error[MAX_STRING_LEN];
 
@@ -94,24 +90,24 @@ void	load_user_parameters(char **lines)
 	{
 		if (NULL == (p = strchr(*pline, ',')))
 		{
-			zabbix_log(LOG_LEVEL_CRIT, "cannot add user parameter \"%s\": not comma-separated", *pline);
-			exit(EXIT_FAILURE);
+			*err = zbx_dsprintf(*err, "user parameter \"%s\": not comma-separated", *pline);
+			return FAIL;
 		}
 		*p = '\0';
 
 		if (FAIL == add_user_parameter(*pline, p + 1, error, sizeof(error)))
 		{
 			*p = ',';
-			zabbix_log(LOG_LEVEL_CRIT, "cannot add user parameter \"%s\": %s", *pline, error);
-			exit(EXIT_FAILURE);
+			*err = zbx_dsprintf(*err, "user parameter \"%s\": %s", *pline, error);
+			return FAIL;
 		}
 		*p = ',';
 	}
+
+	return SUCCEED;
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: load_key_access_rule                                             *
  *                                                                            *
  * Purpose: Adds key access rule from configuration                           *
  *                                                                            *
@@ -121,8 +117,6 @@ void	load_user_parameters(char **lines)
  * Return value: SUCCEED - successful execution                               *
  *               FAIL    - failed to add rule                                 *
  *                                                                            *
- * Author: Andrejs Tumilovics                                                 *
- *                                                                            *
  ******************************************************************************/
 int	load_key_access_rule(const char *value, const struct cfg_line *cfg)
 {
@@ -140,20 +134,12 @@ int	load_key_access_rule(const char *value, const struct cfg_line *cfg)
 
 #ifdef _WINDOWS
 /******************************************************************************
- *                                                                            *
- * Function: load_perf_counters                                               *
  *                                                                            *
  * Purpose: load performance counters from configuration                      *
  *                                                                            *
  * Parameters: def_lines - array of PerfCounter configuration entries         *
  *             eng_lines - array of PerfCounterEn configuration entries       *
  *                                                                            *
- * Return value:                                                              *
- *                                                                            *
- * Author: Vladimir Levijev                                                   *
- *                                                                            *
- * Comments:                                                                  *
- *                                                                            *
  ******************************************************************************/
 void	load_perf_counters(const char **def_lines, const char **eng_lines)
 {
@@ -230,6 +216,66 @@ void	load_perf_counters(const char **def_lines, const char **eng_lines)
 			break;
 	}
 }
+#else
+/******************************************************************************
+ *                                                                            *
+ * Purpose: load user parameters from configuration file                      *
+ *                                                                            *
+ ******************************************************************************/
+static int	load_config_user_params(void)
+{
+	struct cfg_line	cfg[] =
+	{
+		/* PARAMETER,			VAR,					TYPE,
+			MANDATORY,	MIN,			MAX */
+		{"UserParameter",		&CONFIG_USER_PARAMETERS,		TYPE_MULTISTRING,
+			PARM_OPT,	0,			0},
+		{NULL}
+	};
+
+	return parse_cfg_file(CONFIG_FILE, cfg, ZBX_CFG_FILE_REQUIRED, ZBX_CFG_NOT_STRICT, ZBX_CFG_NO_EXIT_FAILURE);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: reload user parameters                                            *
+ *                                                                            *
+ * Parameters: process_type - process type                                    *
+ *             process_num - process number                                   *
+ *                                                                            *
+ ******************************************************************************/
+void	reload_user_parameters(unsigned char process_type, int process_num)
+{
+	char		*error = NULL;
+	ZBX_METRIC	*metrics_fallback = NULL;
+
+	zbx_strarr_init(&CONFIG_USER_PARAMETERS);
+
+	if (FAIL == load_config_user_params())
+	{
+		zabbix_log(LOG_LEVEL_ERR, "cannot reload user parameters [%s #%d]: error processing configuration file",
+				get_process_type_string(process_type), process_num);
+		goto out;
+	}
+
+	get_metrics_copy(&metrics_fallback);
+	remove_user_parameters();
+
+	if (FAIL == load_user_parameters(CONFIG_USER_PARAMETERS, &error))
+	{
+		set_metrics(metrics_fallback);
+		zabbix_log(LOG_LEVEL_ERR, "cannot reload user parameters [%s #%d], %s",
+				get_process_type_string(process_type), process_num, error);
+		zbx_free(error);
+		goto out;
+	}
+
+	free_metrics_ext(&metrics_fallback);
+	zabbix_log(LOG_LEVEL_INFORMATION, "user parameters reloaded [%s #%d]", get_process_type_string(process_type),
+			process_num);
+out:
+	zbx_strarr_free(&CONFIG_USER_PARAMETERS);
+}
 #endif	/* _WINDOWS */
 
 #ifdef _AIX
