@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
-
 #include "threads.h"
 #include "comms.h"
 #include "cfg.h"
@@ -38,16 +36,16 @@ const char	title_message[] = "zabbix_sender";
 const char	syslog_app_name[] = "zabbix_sender";
 
 const char	*usage_message[] = {
-	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "-s host", "-k key", "-o value", NULL,
-	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-s host]", "[-T]", "[-N]", "[-r]", "-i input-file",
-	NULL,
-	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-s host]", "-k key", "-o value",
-	NULL,
-	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-s host]", "[-T]", "[-N]", "[-r]",
+	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-t timeout]", "-s host", "-k key", "-o value", NULL,
+	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]", "[-T]", "[-N]", "[-r]",
 	"-i input-file", NULL,
+	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]", "-k key",
+	"-o value", NULL,
+	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]", "[-T]",
+	"[-N]", "[-r]", "-i input-file", NULL,
 #if defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
-	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "-s host", "--tls-connect cert", "--tls-ca-file CA-file",
-	"[--tls-crl-file CRL-file]", "[--tls-server-cert-issuer cert-issuer]",
+	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-t timeout]", "-s host", "--tls-connect cert",
+	"--tls-ca-file CA-file", "[--tls-crl-file CRL-file]", "[--tls-server-cert-issuer cert-issuer]",
 	"[--tls-server-cert-subject cert-subject]", "--tls-cert-file cert-file", "--tls-key-file key-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13 cipher-string]",
@@ -56,8 +54,8 @@ const char	*usage_message[] = {
 	"[--tls-cipher cipher-string]",
 #endif
 	"-k key", "-o value", NULL,
-	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-s host]", "--tls-connect cert", "--tls-ca-file CA-file",
-	"[--tls-crl-file CRL-file]", "[--tls-server-cert-issuer cert-issuer]",
+	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]", "--tls-connect cert",
+	"--tls-ca-file CA-file", "[--tls-crl-file CRL-file]", "[--tls-server-cert-issuer cert-issuer]",
 	"[--tls-server-cert-subject cert-subject]", "--tls-cert-file cert-file", "--tls-key-file key-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13] cipher-string",
@@ -66,9 +64,10 @@ const char	*usage_message[] = {
 	"[--tls-cipher cipher-string]",
 #endif
 	"[-T]", "[-N]", "[-r]", "-i input-file", NULL,
-	"[-v]", "-c config-file [-z server]", "[-p port]", "[-I IP-address]", "[-s host]", "--tls-connect cert",
-	"--tls-ca-file CA-file", "[--tls-crl-file CRL-file]", "[--tls-server-cert-issuer cert-issuer]",
-	"[--tls-server-cert-subject cert-subject]", "--tls-cert-file cert-file", "--tls-key-file key-file",
+	"[-v]", "-c config-file [-z server]", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]",
+	"--tls-connect cert", "--tls-ca-file CA-file", "[--tls-crl-file CRL-file]",
+	"[--tls-server-cert-issuer cert-issuer]", "[--tls-server-cert-subject cert-subject]",
+	"--tls-cert-file cert-file", "--tls-key-file key-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13 cipher-string]",
 #endif
@@ -76,9 +75,10 @@ const char	*usage_message[] = {
 	"[--tls-cipher cipher-string]",
 #endif
 	"-k key", "-o value", NULL,
-	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-s host]", "--tls-connect cert",
-	"--tls-ca-file CA-file", "[--tls-crl-file CRL-file]", "[--tls-server-cert-issuer cert-issuer]",
-	"[--tls-server-cert-subject cert-subject]", "--tls-cert-file cert-file", "--tls-key-file key-file",
+	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]",
+	"--tls-connect cert", "--tls-ca-file CA-file", "[--tls-crl-file CRL-file]",
+	"[--tls-server-cert-issuer cert-issuer]", "[--tls-server-cert-subject cert-subject]",
+	"--tls-cert-file cert-file", "--tls-key-file key-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13 cipher-string]",
 #endif
@@ -86,7 +86,7 @@ const char	*usage_message[] = {
 	"[--tls-cipher cipher-string]",
 #endif
 	"[-T]", "[-N]", "[-r]", "-i input-file", NULL,
-	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "-s host", "--tls-connect psk",
+	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-t timeout]", "-s host", "--tls-connect psk",
 	"--tls-psk-identity PSK-identity", "--tls-psk-file PSK-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13 cipher-string]",
@@ -95,7 +95,7 @@ const char	*usage_message[] = {
 	"[--tls-cipher cipher-string]",
 #endif
 	"-k key", "-o value", NULL,
-	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-s host]", "--tls-connect psk",
+	"[-v]", "-z server", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]", "--tls-connect psk",
 	"--tls-psk-identity PSK-identity", "--tls-psk-file PSK-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13 cipher-string]",
@@ -104,8 +104,8 @@ const char	*usage_message[] = {
 	"[--tls-cipher cipher-string]",
 #endif
 	"[-T]", "[-N]", "[-r]", "-i input-file", NULL,
-	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-s host]", "--tls-connect psk",
-	"--tls-psk-identity PSK-identity", "--tls-psk-file PSK-file",
+	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]",
+	"--tls-connect psk", "--tls-psk-identity PSK-identity", "--tls-psk-file PSK-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13 cipher-string]",
 #endif
@@ -113,8 +113,8 @@ const char	*usage_message[] = {
 	"[--tls-cipher cipher-string]",
 #endif
 	"-k key", "-o value", NULL,
-	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-s host]", "--tls-connect psk",
-	"--tls-psk-identity PSK-identity", "--tls-psk-file PSK-file",
+	"[-v]", "-c config-file", "[-z server]", "[-p port]", "[-I IP-address]", "[-t timeout]", "[-s host]",
+	"--tls-connect psk", "--tls-psk-identity PSK-identity", "--tls-psk-file PSK-file",
 #if defined(HAVE_OPENSSL)
 	"[--tls-cipher13 cipher-string]",
 #endif
@@ -130,6 +130,13 @@ const char	*usage_message[] = {
 
 unsigned char	program_type	= ZBX_PROGRAM_TYPE_SENDER;
 
+static int	CONFIG_SENDER_TIMEOUT = GET_SENDER_TIMEOUT;
+
+#define CONFIG_SENDER_TIMEOUT_MIN	1
+#define CONFIG_SENDER_TIMEOUT_MAX	300
+#define CONFIG_SENDER_TIMEOUT_MIN_STR	ZBX_STR(CONFIG_SENDER_TIMEOUT_MIN)
+#define CONFIG_SENDER_TIMEOUT_MAX_STR	ZBX_STR(CONFIG_SENDER_TIMEOUT_MAX)
+
 const char	*help_message[] = {
 	"Utility for sending monitoring data to Zabbix server or proxy.",
 	"",
@@ -151,6 +158,10 @@ const char	*help_message[] = {
 	"                             together with --config, overrides \"SourceIP\"",
 	"                             parameter specified in agentd configuration file",
 	"",
+	"  -t --timeout seconds       Specify timeout. Valid range: " CONFIG_SENDER_TIMEOUT_MIN_STR "-"
+			CONFIG_SENDER_TIMEOUT_MAX_STR " seconds",
+	"                             (default: " ZBX_STR(GET_SENDER_TIMEOUT) " seconds)",
+	"",
 	"  -s --host host             Specify host name the item belongs to (as",
 	"                             registered in Zabbix frontend). Host IP address",
 	"                             and DNS name will not work. When used together",
@@ -280,6 +291,8 @@ char	*CONFIG_TLS_CIPHER_CMD		= NULL;	/* parameter '--tls-cipher' from sender com
 int	CONFIG_PASSIVE_FORKS		= 0;	/* not used in zabbix_sender, just for linking with tls.c */
 int	CONFIG_ACTIVE_FORKS		= 0;	/* not used in zabbix_sender, just for linking with tls.c */
 
+int	CONFIG_TCP_MAX_BACKLOG_SIZE	= SOMAXCONN;
+
 /* COMMAND LINE OPTIONS */
 
 /* long options */
@@ -290,6 +303,7 @@ static struct zbx_option	longopts[] =
 	{"port",			1,	NULL,	'p'},
 	{"host",			1,	NULL,	's'},
 	{"source-address",		1,	NULL,	'I'},
+	{"timeout",			1,	NULL,	't'},
 	{"key",				1,	NULL,	'k'},
 	{"value",			1,	NULL,	'o'},
 	{"input-file",			1,	NULL,	'i'},
@@ -314,7 +328,7 @@ static struct zbx_option	longopts[] =
 };
 
 /* short options */
-static char	shortopts[] = "c:I:z:p:s:k:o:TNi:rvhV";
+static char	shortopts[] = "c:I:t:z:p:s:k:o:TNi:rvhV";
 
 /* end of COMMAND LINE OPTIONS */
 
@@ -325,7 +339,7 @@ static int	WITH_TIMESTAMPS = 0;
 static int	WITH_NS = 0;
 static int	REAL_TIME = 0;
 
-static char	*CONFIG_SOURCE_IP = NULL;
+char		*CONFIG_SOURCE_IP = NULL;
 static char	*ZABBIX_SERVER = NULL;
 static char	*ZABBIX_SERVER_PORT = NULL;
 static char	*ZABBIX_HOSTNAME = NULL;
@@ -334,8 +348,7 @@ static char	*ZABBIX_KEY_VALUE = NULL;
 
 typedef struct
 {
-	char			*host;
-	unsigned short		port;
+	zbx_vector_ptr_t	addrs;
 	ZBX_THREAD_HANDLE	*thread;
 }
 zbx_send_destinations_t;
@@ -355,7 +368,6 @@ static void	sender_signal_handler(int sig)
 
 	switch (sig)
 	{
-		CASE_LOG_WARNING(SIGALRM);
 		CASE_LOG_WARNING(SIGINT);
 		CASE_LOG_WARNING(SIGQUIT);
 		CASE_LOG_WARNING(SIGTERM);
@@ -392,21 +404,51 @@ static void	main_signal_handler(int sig)
 
 typedef struct
 {
-	char		*server;
-	unsigned short	port;
-	struct zbx_json	json;
+	zbx_vector_ptr_t		*addrs;
+	struct zbx_json			json;
 #if defined(_WINDOWS) && (defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL))
 	ZBX_THREAD_SENDVAL_TLS_ARGS	tls_vars;
 #endif
-	int		sync_timestamp;
+	int				sync_timestamp;
+#ifndef _WINDOWS
+	int				fds[2];
+#endif
 }
 ZBX_THREAD_SENDVAL_ARGS;
 
 #define SUCCEED_PARTIAL	2
 
+#if !defined(_WINDOWS)
+static void	zbx_thread_handle_pipe_response(ZBX_THREAD_SENDVAL_ARGS *sendval_args)
+{
+	int	offset;
+	char	buffer[sizeof(int)], *ptr = buffer;
+
+	while (0 < (offset = (int)read(sendval_args->fds[0], ptr, (size_t)(buffer + sizeof(buffer) - ptr))))
+		ptr += offset;
+
+	if (-1 == offset)
+		zabbix_log(LOG_LEVEL_WARNING, "cannot read data from pipe: %s", zbx_strerror(errno));
+
+	if (ptr - buffer != sizeof(int))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Incorrect response from child thread");
+		return;
+	}
+
+	memcpy(&offset, buffer, sizeof(int));
+
+	while (0 < offset--)
+	{
+		zbx_addr_t	*addr = sendval_args->addrs->values[0];
+
+		zbx_vector_ptr_remove(sendval_args->addrs, 0);
+		zbx_vector_ptr_append(sendval_args->addrs, addr);
+	}
+}
+#endif
+
 /******************************************************************************
- *                                                                            *
- * Function: sender_threads_wait                                              *
  *                                                                            *
  * Purpose: waits until the "threads" are in the signalled state and manages  *
  *          exit status updates                                               *
@@ -426,7 +468,8 @@ ZBX_THREAD_SENDVAL_ARGS;
  *           SUCCEED statuses that come after should not overwrite it         *
  *                                                                            *
  ******************************************************************************/
-static int	sender_threads_wait(ZBX_THREAD_HANDLE *threads, int threads_num, const int old_status)
+static int	sender_threads_wait(ZBX_THREAD_HANDLE *threads, zbx_thread_args_t *threads_args, int threads_num,
+		const int old_status)
 {
 	int		i, sp_count = 0, fail_count = 0;
 #if defined(_WINDOWS)
@@ -437,6 +480,12 @@ static int	sender_threads_wait(ZBX_THREAD_HANDLE *threads, int threads_num, cons
 	{
 		int	new_status;
 
+		if (ZBX_THREAD_ERROR == threads[i])
+		{
+			threads[i] = ZBX_THREAD_HANDLE_NULL;
+			continue;
+		}
+
 		if (SUCCEED_PARTIAL == (new_status = zbx_thread_wait(threads[i])))
 				sp_count++;
 
@@ -448,12 +497,21 @@ static int	sender_threads_wait(ZBX_THREAD_HANDLE *threads, int threads_num, cons
 			{
 				if (destinations[j].thread == &threads[i])
 				{
-					zbx_free(destinations[j].host);
+					zbx_vector_ptr_clear_ext(&destinations[j].addrs,
+							(zbx_clean_func_t)zbx_addr_free);
+					zbx_vector_ptr_destroy(&destinations[j].addrs);
 					destinations[j] = destinations[--destinations_count];
 					break;
 				}
 			}
 		}
+#if !defined(_WINDOWS)
+		else
+			zbx_thread_handle_pipe_response((ZBX_THREAD_SENDVAL_ARGS *)threads_args[i].args);
+
+		close(((ZBX_THREAD_SENDVAL_ARGS *)threads_args[i].args)->fds[0]);
+		close(((ZBX_THREAD_SENDVAL_ARGS *)threads_args[i].args)->fds[1]);
+#endif
 
 		threads[i] = ZBX_THREAD_HANDLE_NULL;
 	}
@@ -467,8 +525,6 @@ static int	sender_threads_wait(ZBX_THREAD_HANDLE *threads, int threads_num, cons
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: get_string                                                       *
  *                                                                            *
  * Purpose: get current string from the quoted or unquoted string list,       *
  *          delimited by blanks                                               *
@@ -567,8 +623,6 @@ static const char	*get_string(const char *p, char *buf, size_t bufsize)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: check_response                                                   *
  *                                                                            *
  * Purpose: Check whether JSON response is SUCCEED                            *
  *                                                                            *
@@ -579,8 +633,6 @@ static const char	*get_string(const char *p, char *buf, size_t bufsize)
  *                SUCCEED_PARTIAL - the sending operation was completed       *
  *                successfully, but processing of at least one value failed   *
  *                                                                            *
- * Author: Alexei Vladishev                                                   *
- *                                                                            *
  * Comments: active agent has almost the same function!                       *
  *                                                                            *
  ******************************************************************************/
@@ -612,21 +664,46 @@ static int	check_response(char *response, const char *server, unsigned short por
 
 	return ret;
 }
-
-static	ZBX_THREAD_ENTRY(send_value, args)
+#if !defined(_WINDOWS) && !defined(__MINGW32)
+static void	alarm_signal_handler(int sig, siginfo_t *siginfo, void *context)
 {
-	ZBX_THREAD_SENDVAL_ARGS	*sendval_args = (ZBX_THREAD_SENDVAL_ARGS *)((zbx_thread_args_t *)args)->args;
-	int			tcp_ret, ret = FAIL;
-	char			*tls_arg1, *tls_arg2;
-	zbx_socket_t		sock;
+	ZBX_UNUSED(sig);
+	ZBX_UNUSED(siginfo);
+	ZBX_UNUSED(context);
+
+	zbx_alarm_flag_set();	/* set alarm flag */
+}
+
+static void	zbx_set_sender_signal_handlers(void)
+{
+	struct sigaction	phan;
+
+	sigemptyset(&phan.sa_mask);
+	phan.sa_flags = SA_SIGINFO;
+
+	phan.sa_sigaction = alarm_signal_handler;
+	sigaction(SIGALRM, &phan, NULL);
 
-#if !defined(_WINDOWS)
 	signal(SIGINT, sender_signal_handler);
 	signal(SIGQUIT, sender_signal_handler);
 	signal(SIGTERM, sender_signal_handler);
 	signal(SIGHUP, sender_signal_handler);
-	signal(SIGALRM, sender_signal_handler);
 	signal(SIGPIPE, sender_signal_handler);
+}
+#endif
+
+static	ZBX_THREAD_ENTRY(send_value, args)
+{
+	ZBX_THREAD_SENDVAL_ARGS	*sendval_args = (ZBX_THREAD_SENDVAL_ARGS *)((zbx_thread_args_t *)args)->args;
+	int			ret = FAIL;
+	zbx_socket_t		sock;
+#if !defined(_WINDOWS)
+	int			i;
+	zbx_addr_t		*last_addr;
+
+	last_addr = (zbx_addr_t *)sendval_args->addrs->values[0];
+
+	zbx_set_sender_signal_handlers();
 #endif
 
 #if defined(_WINDOWS) && (defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL))
@@ -636,29 +713,8 @@ static	ZBX_THREAD_ENTRY(send_value, args)
 		zbx_tls_take_vars(&sendval_args->tls_vars);
 	}
 #endif
-	switch (configured_tls_connect_mode)
-	{
-		case ZBX_TCP_SEC_UNENCRYPTED:
-			tls_arg1 = NULL;
-			tls_arg2 = NULL;
-			break;
-#if defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
-		case ZBX_TCP_SEC_TLS_CERT:
-			tls_arg1 = CONFIG_TLS_SERVER_CERT_ISSUER;
-			tls_arg2 = CONFIG_TLS_SERVER_CERT_SUBJECT;
-			break;
-		case ZBX_TCP_SEC_TLS_PSK:
-			tls_arg1 = CONFIG_TLS_PSK_IDENTITY;
-			tls_arg2 = NULL;	/* zbx_tls_connect() will find PSK */
-			break;
-#endif
-		default:
-			THIS_SHOULD_NEVER_HAPPEN;
-			goto out;
-	}
-
-	if (SUCCEED == (tcp_ret = zbx_tcp_connect(&sock, CONFIG_SOURCE_IP, sendval_args->server, sendval_args->port,
-			GET_SENDER_TIMEOUT, configured_tls_connect_mode, tls_arg1, tls_arg2)))
+	if (SUCCEED == connect_to_server(&sock, CONFIG_SOURCE_IP, sendval_args->addrs,
+			CONFIG_SENDER_TIMEOUT, CONFIG_TIMEOUT, configured_tls_connect_mode, 0, LOG_LEVEL_DEBUG))
 	{
 		if (1 == sendval_args->sync_timestamp)
 		{
@@ -670,33 +726,64 @@ static	ZBX_THREAD_ENTRY(send_value, args)
 			zbx_json_adduint64(&sendval_args->json, ZBX_PROTO_TAG_NS, ts.ns);
 		}
 
-		if (SUCCEED == (tcp_ret = zbx_tcp_send(&sock, sendval_args->json.buffer)))
+		if (SUCCEED == zbx_tcp_send(&sock, sendval_args->json.buffer))
 		{
-			if (SUCCEED == (tcp_ret = zbx_tcp_recv(&sock)))
+			if (SUCCEED == zbx_tcp_recv(&sock))
 			{
 				zabbix_log(LOG_LEVEL_DEBUG, "answer [%s]", sock.buffer);
 
-				if (FAIL == (ret = check_response(sock.buffer, sendval_args->server,
-						sendval_args->port)))
+				if (FAIL == (ret = check_response(sock.buffer,
+						((zbx_addr_t *)sendval_args->addrs->values[0])->ip,
+						((zbx_addr_t *)sendval_args->addrs->values[0])->port)))
 				{
 					zabbix_log(LOG_LEVEL_WARNING, "incorrect answer from \"%s:%hu\": [%s]",
-							sendval_args->server, sendval_args->port, sock.buffer);
+							((zbx_addr_t *)sendval_args->addrs->values[0])->ip,
+							((zbx_addr_t *)sendval_args->addrs->values[0])->port,
+							sock.buffer);
 				}
 			}
+			else
+			{
+				zabbix_log(LOG_LEVEL_DEBUG, "Unable to receive from [%s]:%d [%s]",
+						((zbx_addr_t *)sendval_args->addrs->values[0])->ip,
+						((zbx_addr_t *)sendval_args->addrs->values[0])->port,
+						zbx_socket_strerror());
+			}
+		}
+		else
+		{
+			zabbix_log(LOG_LEVEL_DEBUG, "Unable to send to [%s]:%d [%s]",
+					((zbx_addr_t *)sendval_args->addrs->values[0])->ip,
+					((zbx_addr_t *)sendval_args->addrs->values[0])->port,
+					zbx_socket_strerror());
 		}
 
 		zbx_tcp_close(&sock);
 	}
+#if !defined(_WINDOWS)
+	for (i = sendval_args->addrs->values_num - 1; i >= 0; i--)
+	{
+		if (last_addr == sendval_args->addrs->values[i])
+		{
+			int	offset = sendval_args->addrs->values_num - i;
 
-	if (FAIL == tcp_ret)
-		zabbix_log(LOG_LEVEL_DEBUG, "send value error: %s", zbx_socket_strerror());
-out:
+			if (0 == i)
+				offset = 0;
+
+			if (FAIL == zbx_write_all(sendval_args->fds[1], (char *)&offset, sizeof(offset)))
+				zabbix_log(LOG_LEVEL_WARNING, "cannot write data to pipe: %s", zbx_strerror(errno));
+
+			close(sendval_args->fds[0]);
+			close(sendval_args->fds[1]);
+			break;
+		}
+	}
+
+#endif
 	zbx_thread_exit(ret);
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: perform_data_sending                                             *
  *                                                                            *
  * Purpose: Send data to all destinations each in a separate thread and wait  *
  *          till threads have completed their task                            *
@@ -716,19 +803,18 @@ static int	perform_data_sending(ZBX_THREAD_SENDVAL_ARGS *sendval_args, int old_s
 {
 	int			i, ret;
 	ZBX_THREAD_HANDLE	*threads = NULL;
+	zbx_thread_args_t	*threads_args;
 
-	threads = (ZBX_THREAD_HANDLE *)zbx_calloc(threads, destinations_count, sizeof(ZBX_THREAD_HANDLE));
+	threads = (ZBX_THREAD_HANDLE *)zbx_calloc(threads, (size_t)destinations_count, sizeof(ZBX_THREAD_HANDLE));
+	threads_args = (zbx_thread_args_t *)zbx_calloc(NULL, (size_t)destinations_count, sizeof(zbx_thread_args_t));
 
 	for (i = 0; i < destinations_count; i++)
 	{
-		zbx_thread_args_t	*thread_args;
-
-		thread_args = (zbx_thread_args_t *)zbx_malloc(NULL, sizeof(zbx_thread_args_t));
+		zbx_thread_args_t	*thread_args = threads_args + i;
 
 		thread_args->args = &sendval_args[i];
 
-		sendval_args[i].server = destinations[i].host;
-		sendval_args[i].port = destinations[i].port;
+		sendval_args[i].addrs = &destinations[i].addrs;
 
 		if (0 != i)
 		{
@@ -740,23 +826,27 @@ static int	perform_data_sending(ZBX_THREAD_SENDVAL_ARGS *sendval_args, int old_s
 		}
 
 		destinations[i].thread = &threads[i];
-
-		zbx_thread_start(send_value, thread_args, &threads[i]);
 #ifndef _WINDOWS
-		zbx_free(thread_args);
+		if (-1 == pipe(sendval_args[i].fds))
+		{
+			zabbix_log(LOG_LEVEL_ERR, "Cannot create data pipe: %s",
+					strerror_from_system((unsigned long)errno));
+			threads[i] = (ZBX_THREAD_HANDLE)ZBX_THREAD_ERROR;
+			continue;
+		}
 #endif
+		zbx_thread_start(send_value, thread_args, &threads[i]);
 	}
 
-	ret = sender_threads_wait(threads, destinations_count, old_status);
+	ret = sender_threads_wait(threads, threads_args, destinations_count, old_status);
 
+	zbx_free(threads_args);
 	zbx_free(threads);
 
 	return ret;
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: sender_add_serveractive_host_cb                                  *
  *                                                                            *
  * Purpose: add server or proxy to the list of destinations                   *
  *                                                                            *
@@ -768,17 +858,10 @@ static int	perform_data_sending(ZBX_THREAD_SENDVAL_ARGS *sendval_args, int old_s
  *                FAIL - destination has been already added                   *
  *                                                                            *
  ******************************************************************************/
-static int	sender_add_serveractive_host_cb(const char *host, unsigned short port, zbx_vector_str_t *hostnames)
+static int	sender_add_serveractive_host_cb(const zbx_vector_ptr_t *addrs, zbx_vector_str_t *hostnames, void *data)
 {
-	int	i;
-
 	ZBX_UNUSED(hostnames);
-
-	for (i = 0; i < destinations_count; i++)
-	{
-		if (0 == strcmp(destinations[i].host, host) && destinations[i].port == port)
-			return FAIL;
-	}
+	ZBX_UNUSED(data);
 
 	destinations_count++;
 #if defined(_WINDOWS)
@@ -792,8 +875,9 @@ static int	sender_add_serveractive_host_cb(const char *host, unsigned short port
 	destinations = (zbx_send_destinations_t *)zbx_realloc(destinations,
 			sizeof(zbx_send_destinations_t) * destinations_count);
 
-	destinations[destinations_count - 1].host = zbx_strdup(NULL, host);
-	destinations[destinations_count - 1].port = port;
+	zbx_vector_ptr_create(&destinations[destinations_count - 1].addrs);
+
+	zbx_addr_copy(&destinations[destinations_count - 1].addrs, addrs);
 
 	return SUCCEED;
 }
@@ -856,11 +940,13 @@ static void	zbx_load_config(const char *config_file)
 			PARM_OPT,	0,			0},
 		{"TLSCipherPSK",		&cfg_tls_cipher_psk,			TYPE_STRING,
 			PARM_OPT,	0,			0},
+		{"ListenBacklog",		&CONFIG_TCP_MAX_BACKLOG_SIZE,		TYPE_INT,
+			PARM_OPT,	0,			INT_MAX},
 		{NULL}
 	};
 
 	/* do not complain about unknown parameters in agent configuration file */
-	parse_cfg_file(config_file, cfg, ZBX_CFG_FILE_REQUIRED, ZBX_CFG_NOT_STRICT);
+	parse_cfg_file(config_file, cfg, ZBX_CFG_FILE_REQUIRED, ZBX_CFG_NOT_STRICT, ZBX_CFG_EXIT_FAILURE);
 
 	/* get first hostname only */
 	if (NULL != cfg_hostname)
@@ -882,7 +968,17 @@ static void	zbx_load_config(const char *config_file)
 	if (NULL == ZABBIX_SERVER)
 	{
 		if (NULL != cfg_active_hosts && '\0' != *cfg_active_hosts)
-			zbx_set_data_destination_hosts(cfg_active_hosts, sender_add_serveractive_host_cb, NULL);
+		{
+			char	*error;
+
+			if (FAIL == zbx_set_data_destination_hosts(cfg_active_hosts,
+					(unsigned short)ZBX_DEFAULT_SERVER_PORT, "ServerActive",
+					sender_add_serveractive_host_cb, NULL, NULL, &error))
+			{
+				zbx_error("%s", error);
+				exit(EXIT_FAILURE);
+			}
+		}
 	}
 	zbx_free(cfg_active_hosts);
 
@@ -970,6 +1066,16 @@ static void	parse_commandline(int argc, char **argv)
 			case 'r':
 				REAL_TIME = 1;
 				break;
+			case 't':
+				if (FAIL == is_uint_n_range(zbx_optarg, ZBX_MAX_UINT64_LEN, &CONFIG_SENDER_TIMEOUT,
+						sizeof(CONFIG_SENDER_TIMEOUT), CONFIG_SENDER_TIMEOUT_MIN,
+						CONFIG_SENDER_TIMEOUT_MAX))
+				{
+					zbx_error("Invalid timeout, valid range %d:%d seconds",
+							CONFIG_SENDER_TIMEOUT_MIN, CONFIG_SENDER_TIMEOUT_MAX);
+					exit(EXIT_FAILURE);
+				}
+				break;
 			case 'v':
 				if (LOG_LEVEL_WARNING > CONFIG_LOG_LEVEL)
 					CONFIG_LOG_LEVEL = LOG_LEVEL_WARNING;
@@ -1043,6 +1149,7 @@ static void	parse_commandline(int argc, char **argv)
 	if (NULL != ZABBIX_SERVER)
 	{
 		unsigned short	port;
+		char		*error;
 
 		if (NULL != ZABBIX_SERVER_PORT)
 		{
@@ -1057,7 +1164,12 @@ static void	parse_commandline(int argc, char **argv)
 		else
 			port = (unsigned short)ZBX_DEFAULT_SERVER_PORT;
 
-		sender_add_serveractive_host_cb(ZABBIX_SERVER, port, NULL);
+		if (FAIL == zbx_set_data_destination_hosts(ZABBIX_SERVER, port, "-z",
+				sender_add_serveractive_host_cb, NULL, NULL, &error))
+		{
+			zbx_error("%s", error);
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	/* every option may be specified only once */
@@ -1335,8 +1447,6 @@ static void	parse_commandline(int argc, char **argv)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_fgets_alloc                                                  *
  *                                                                            *
  * Purpose: reads a line from file                                            *
  *                                                                            *
@@ -1393,7 +1503,6 @@ int	main(int argc, char **argv)
 
 	if (NULL != CONFIG_FILE)
 		zbx_load_config(CONFIG_FILE);
-
 #ifndef _WINDOWS
 	if (SUCCEED != zbx_locks_create(&error))
 	{
@@ -1757,6 +1866,7 @@ exit:
 #if !defined(_WINDOWS) && defined(HAVE_PTHREAD_PROCESS_SHARED)
 	zbx_locks_disable();
 #endif
+
 	if (FAIL == ret)
 		ret = EXIT_FAILURE;
 
