@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,60 +17,23 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
+#include "dbsyncer.h"
 
-#include "db.h"
 #include "log.h"
 #include "daemon.h"
 #include "zbxself.h"
+#include "sighandler.h"
 
 #include "dbcache.h"
-#include "dbsyncer.h"
 #include "export.h"
 
-extern int		CONFIG_HISTSYNCER_FREQUENCY;
-extern unsigned char	process_type, program_type;
-extern int		server_num, process_num;
-static sigset_t		orig_mask;
+extern int				CONFIG_HISTSYNCER_FREQUENCY;
+extern ZBX_THREAD_LOCAL unsigned char	process_type;
+extern unsigned char			program_type;
+extern ZBX_THREAD_LOCAL int		server_num, process_num;
+static sigset_t				orig_mask;
 
 /******************************************************************************
- *                                                                            *
- * Function: block_signals                                                    *
- *                                                                            *
- * Purpose: block signals to avoid interruption                               *
- *                                                                            *
- ******************************************************************************/
-static	void	block_signals(void)
-{
-	sigset_t	mask;
-
-	sigemptyset(&mask);
-	sigaddset(&mask, SIGUSR1);
-	sigaddset(&mask, SIGUSR2);
-	sigaddset(&mask, SIGTERM);
-	sigaddset(&mask, SIGINT);
-	sigaddset(&mask, SIGQUIT);
-
-	if (0 > sigprocmask(SIG_BLOCK, &mask, &orig_mask))
-		zabbix_log(LOG_LEVEL_WARNING, "cannot set sigprocmask to block the signal");
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: unblock_signals                                                  *
- *                                                                            *
- * Purpose: unblock signals after blocking                                    *
- *                                                                            *
- ******************************************************************************/
-static	void	unblock_signals(void)
-{
-	if (0 > sigprocmask(SIG_SETMASK, &orig_mask, NULL))
-		zabbix_log(LOG_LEVEL_WARNING,"cannot restore sigprocmask");
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: zbx_db_flush_timer_queue                                         *
  *                                                                            *
  * Purpose: flush timer queue to the database                                 *
  *                                                                            *
@@ -86,18 +49,19 @@ static void	zbx_db_flush_timer_queue(void)
 
 	if (0 != persistent_timers.values_num)
 	{
-		zbx_db_insert_prepare(&db_insert, "trigger_queue", "objectid", "type", "clock", "ns", NULL);
+		zbx_db_insert_prepare(&db_insert, "trigger_queue", "trigger_queueid", "objectid", "type", "clock", "ns", NULL);
 
 		for (i = 0; i < persistent_timers.values_num; i++)
 		{
 			zbx_trigger_timer_t	*timer = (zbx_trigger_timer_t *)persistent_timers.values[i];
 
-			zbx_db_insert_add_values(&db_insert, timer->objectid, timer->type, timer->eval_ts.sec,
-					timer->eval_ts.ns);
+			zbx_db_insert_add_values(&db_insert, __UINT64_C(0), timer->objectid, timer->type,
+					timer->eval_ts.sec, timer->eval_ts.ns);
 		}
 
 		zbx_dc_free_timers(&persistent_timers);
 
+		zbx_db_insert_autoincrement(&db_insert, "trigger_queueid");
 		zbx_db_insert_execute(&db_insert);
 		zbx_db_insert_clean(&db_insert);
 	}
@@ -112,13 +76,9 @@ static void	db_trigger_queue_cleanup(void)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: main_dbsyncer_loop                                               *
  *                                                                            *
  * Purpose: periodically synchronises data in memory cache with database      *
  *                                                                            *
- * Author: Alexei Vladishev                                                   *
- *                                                                            *
  * Comments: never returns                                                    *
  *                                                                            *
  ******************************************************************************/
@@ -149,13 +109,13 @@ ZBX_THREAD_ENTRY(dbsyncer_thread, args)
 	zbx_strcpy_alloc(&stats, &stats_alloc, &stats_offset, "started");
 
 	/* database APIs might not handle signals correctly and hang, block signals to avoid hanging */
-	block_signals();
+	zbx_block_signals(&orig_mask);
 	DBconnect(ZBX_DB_CONNECT_NORMAL);
 
 	if (1 == process_num)
 		db_trigger_queue_cleanup();
 
-	unblock_signals();
+	zbx_unblock_signals(&orig_mask);
 
 	if (SUCCEED == zbx_is_export_enabled(ZBX_FLAG_EXPTYPE_HISTORY))
 		zbx_history_export_init("history-syncer", process_num);
@@ -179,13 +139,13 @@ ZBX_THREAD_ENTRY(dbsyncer_thread, args)
 			zbx_log_sync_history_cache_progress();
 
 		/* database APIs might not handle signals correctly and hang, block signals to avoid hanging */
-		block_signals();
+		zbx_block_signals(&orig_mask);
 		zbx_sync_history_cache(&values_num, &triggers_num, &more);
 
 		if (!ZBX_IS_RUNNING() && SUCCEED != zbx_db_trigger_queue_locked())
 			zbx_db_flush_timer_queue();
 
-		unblock_signals();
+		zbx_unblock_signals(&orig_mask);
 
 		total_values_num += values_num;
 		total_triggers_num += triggers_num;
@@ -227,12 +187,12 @@ ZBX_THREAD_ENTRY(dbsyncer_thread, args)
 	}
 
 	/* database APIs might not handle signals correctly and hang, block signals to avoid hanging */
-	block_signals();
+	zbx_block_signals(&orig_mask);
 	if (SUCCEED != zbx_db_trigger_queue_locked())
 		zbx_db_flush_timer_queue();
 
 	DBclose();
-	unblock_signals();
+	zbx_unblock_signals(&orig_mask);
 
 	zbx_log_sync_history_cache_progress();
 
