@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,15 +17,14 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
 #include "lld.h"
-#include "db.h"
+#include "proxy.h"
+
 #include "log.h"
-#include "../events.h"
-#include "zbxalgo.h"
 #include "zbxserver.h"
 #include "zbxregexp.h"
-#include "proxy.h"
+
+#include "../../libs/zbxaudit/audit.h"
 
 #define OVERRIDE_STOP_TRUE	1
 
@@ -61,8 +60,6 @@ typedef struct
 lld_override_t;
 
 /******************************************************************************
- *                                                                            *
- * Function: lld_condition_free                                               *
  *                                                                            *
  * Purpose: release resources allocated by filter condition                   *
  *                                                                            *
@@ -80,8 +77,6 @@ static void	lld_condition_free(lld_condition_t *condition)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: lld_conditions_free                                              *
  *                                                                            *
  * Purpose: release resources allocated by filter conditions                  *
  *                                                                            *
@@ -95,8 +90,6 @@ static void	lld_conditions_free(zbx_vector_ptr_t *conditions)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: lld_condition_compare_by_macro                                   *
  *                                                                            *
  * Purpose: compare two filter conditions by their macros                     *
  *                                                                            *
@@ -113,8 +106,6 @@ static int	lld_condition_compare_by_macro(const void *item1, const void *item2)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: lld_filter_init                                                  *
  *                                                                            *
  * Purpose: initializes lld filter                                            *
  *                                                                            *
@@ -129,8 +120,6 @@ static void	lld_filter_init(lld_filter_t *filter)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: lld_filter_clean                                                 *
  *                                                                            *
  * Purpose: releases resources allocated by lld filter                        *
  *                                                                            *
@@ -171,7 +160,7 @@ static int	lld_filter_condition_add(zbx_vector_ptr_t *conditions, const char *id
 	}
 	else
 	{
-		substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, NULL, item, NULL, NULL, NULL,
+		substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, NULL, item, NULL, NULL, NULL, NULL, NULL,
 				&condition->regexp, MACRO_TYPE_LLD_FILTER, NULL, 0);
 	}
 
@@ -179,8 +168,6 @@ static int	lld_filter_condition_add(zbx_vector_ptr_t *conditions, const char *id
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: lld_filter_load                                                  *
  *                                                                            *
  * Purpose: loads lld filter data                                             *
  *                                                                            *
@@ -217,38 +204,54 @@ static int	lld_filter_load(lld_filter_t *filter, zbx_uint64_t lld_ruleid, const
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: filter_condition_match                                           *
  *                                                                            *
  * Purpose: check if the lld data passes filter evaluation                    *
  *                                                                            *
- * Parameters: filter          - [IN] the lld filter                          *
- *             jp_row          - [IN] the lld data row                        *
+ * Parameters: jp_row          - [IN] the lld data row                        *
  *             lld_macro_paths - [IN] use json path to extract from jp_row    *
+ *             condition       - [IN] the lld filter condition                *
+ *             info            - [OUT] the warning description                *
  *                                                                            *
  * Return value: SUCCEED - the lld data passed filter evaluation              *
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
 static int	filter_condition_match(const struct zbx_json_parse *jp_row, const zbx_vector_ptr_t *lld_macro_paths,
-		const lld_condition_t *condition)
+		const lld_condition_t *condition, char **info)
 {
 	char	*value = NULL;
 	int	ret;
 
 	if (SUCCEED == (ret = zbx_lld_macro_value_by_name(jp_row, lld_macro_paths, condition->macro, &value)))
 	{
-		switch (regexp_match_ex(&condition->regexps, value, condition->regexp, ZBX_CASE_SENSITIVE))
+		if (CONDITION_OPERATOR_NOT_EXIST == condition->op)
 		{
-			case ZBX_REGEXP_MATCH:
-				ret = (CONDITION_OPERATOR_REGEXP == condition->op ? SUCCEED : FAIL);
-				break;
-			case ZBX_REGEXP_NO_MATCH:
-				ret = (CONDITION_OPERATOR_NOT_REGEXP == condition->op ? SUCCEED : FAIL);
-				break;
-			default:
-				ret = FAIL;
+			ret = FAIL;
 		}
+		else if (CONDITION_OPERATOR_EXIST != condition->op)
+		{
+			switch (regexp_match_ex(&condition->regexps, value, condition->regexp, ZBX_CASE_SENSITIVE))
+			{
+				case ZBX_REGEXP_MATCH:
+					ret = (CONDITION_OPERATOR_REGEXP == condition->op ? SUCCEED : FAIL);
+					break;
+				case ZBX_REGEXP_NO_MATCH:
+					ret = (CONDITION_OPERATOR_NOT_REGEXP == condition->op ? SUCCEED : FAIL);
+					break;
+				default:
+					ret = FAIL;
+			}
+		}
+	}
+	else if (CONDITION_OPERATOR_NOT_EXIST == condition->op)
+	{
+		ret = SUCCEED;
+	}
+	else if (CONDITION_OPERATOR_EXIST != condition->op)
+	{
+		*info = zbx_strdcatf(*info,
+				"Cannot accurately apply filter: no value received for macro \"%s\".\n",
+				condition->macro);
 	}
 
 	zbx_free(value);
@@ -257,32 +260,32 @@ static int	filter_condition_match(const struct zbx_json_parse *jp_row, const zbx
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: filter_evaluate_and_or                                           *
  *                                                                            *
  * Purpose: check if the lld data passes filter evaluation by and/or rule     *
  *                                                                            *
  * Parameters: filter          - [IN] the lld filter                          *
  *             jp_row          - [IN] the lld data row                        *
  *             lld_macro_paths - [IN] use json path to extract from jp_row    *
+ *             info            - [OUT] the warning description                *
  *                                                                            *
  * Return value: SUCCEED - the lld data passed filter evaluation              *
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
 static int	filter_evaluate_and_or(const lld_filter_t *filter, const struct zbx_json_parse *jp_row,
-		const zbx_vector_ptr_t *lld_macro_paths)
+		const zbx_vector_ptr_t *lld_macro_paths, char **info)
 {
-	int	i, ret = SUCCEED, rc = SUCCEED;
+	int	i, ret = SUCCEED;
 	char	*lastmacro = NULL;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	for (i = 0; i < filter->conditions.values_num; i++)
 	{
+		int			rc;
 		const lld_condition_t	*condition = (lld_condition_t *)filter->conditions.values[i];
 
-		rc = filter_condition_match(jp_row, lld_macro_paths, condition);
+		rc = filter_condition_match(jp_row, lld_macro_paths, condition, info);
 		/* check if a new condition group has started */
 		if (NULL == lastmacro || 0 != strcmp(lastmacro, condition->macro))
 		{
@@ -307,21 +310,20 @@ static int	filter_evaluate_and_or(const lld_filter_t *filter, const struct zbx_j
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: filter_evaluate_and                                              *
  *                                                                            *
  * Purpose: check if the lld data passes filter evaluation by and rule        *
  *                                                                            *
  * Parameters: filter          - [IN] the lld filter                          *
  *             jp_row          - [IN] the lld data row                        *
  *             lld_macro_paths - [IN] use json path to extract from jp_row    *
+ *             info            - [OUT] the warning description                *
  *                                                                            *
  * Return value: SUCCEED - the lld data passed filter evaluation              *
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
 static int	filter_evaluate_and(const lld_filter_t *filter, const struct zbx_json_parse *jp_row,
-		const zbx_vector_ptr_t *lld_macro_paths)
+		const zbx_vector_ptr_t *lld_macro_paths, char **info)
 {
 	int	i, ret = SUCCEED;
 
@@ -331,7 +333,7 @@ static int	filter_evaluate_and(const lld_filter_t *filter, const struct zbx_json
 	{
 		/* if any of conditions are false the evaluation returns false */
 		if (SUCCEED != (ret = filter_condition_match(jp_row, lld_macro_paths,
-				(lld_condition_t *)filter->conditions.values[i])))
+				(lld_condition_t *)filter->conditions.values[i], info)))
 		{
 			break;
 		}
@@ -343,21 +345,20 @@ static int	filter_evaluate_and(const lld_filter_t *filter, const struct zbx_json
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: filter_evaluate_or                                               *
  *                                                                            *
  * Purpose: check if the lld data passes filter evaluation by or rule         *
  *                                                                            *
  * Parameters: filter          - [IN] the lld filter                          *
  *             jp_row          - [IN] the lld data row                        *
  *             lld_macro_paths - [IN] use json path to extract from jp_row    *
+ *             info            - [OUT] the warning description                *
  *                                                                            *
  * Return value: SUCCEED - the lld data passed filter evaluation              *
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
 static int	filter_evaluate_or(const lld_filter_t *filter, const struct zbx_json_parse *jp_row,
-		const zbx_vector_ptr_t *lld_macro_paths)
+		const zbx_vector_ptr_t *lld_macro_paths, char **info)
 {
 	int	i, ret = SUCCEED;
 
@@ -367,7 +368,7 @@ static int	filter_evaluate_or(const lld_filter_t *filter, const struct zbx_json_
 	{
 		/* if any of conditions are true the evaluation returns true */
 		if (SUCCEED == (ret = filter_condition_match(jp_row, lld_macro_paths,
-				(lld_condition_t *)filter->conditions.values[i])))
+				(lld_condition_t *)filter->conditions.values[i], info)))
 		{
 			break;
 		}
@@ -379,8 +380,6 @@ static int	filter_evaluate_or(const lld_filter_t *filter, const struct zbx_json_
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: filter_evaluate_expression                                       *
  *                                                                            *
  * Purpose: check if the lld data passes filter evaluation by custom          *
  *          expression                                                        *
@@ -388,6 +387,7 @@ static int	filter_evaluate_or(const lld_filter_t *filter, const struct zbx_json_
  * Parameters: filter          - [IN] the lld filter                          *
  *             jp_row          - [IN] the lld data row                        *
  *             lld_macro_paths - [IN] use json path to extract from jp_row    *
+ *             info            - [OUT] the warning description                *
  *                                                                            *
  * Return value: SUCCEED - the lld data passed filter evaluation              *
  *               FAIL    - otherwise                                          *
@@ -398,7 +398,7 @@ static int	filter_evaluate_or(const lld_filter_t *filter, const struct zbx_json_
  *                                                                            *
  ******************************************************************************/
 static int	filter_evaluate_expression(const lld_filter_t *filter, const struct zbx_json_parse *jp_row,
-		const zbx_vector_ptr_t *lld_macro_paths)
+		const zbx_vector_ptr_t *lld_macro_paths, char **info)
 {
 	int	i, ret = FAIL, id_len;
 	char	*expression, id[ZBX_MAX_UINT64_LEN + 2], *p, error[256];
@@ -412,7 +412,7 @@ static int	filter_evaluate_expression(const lld_filter_t *filter, const struct z
 	{
 		const lld_condition_t	*condition = (lld_condition_t *)filter->conditions.values[i];
 
-		ret = filter_condition_match(jp_row, lld_macro_paths, condition);
+		ret = filter_condition_match(jp_row, lld_macro_paths, condition, info);
 
 		zbx_snprintf(id, sizeof(id), "{" ZBX_FS_UI64 "}", condition->id);
 
@@ -438,87 +438,34 @@ static int	filter_evaluate_expression(const lld_filter_t *filter, const struct z
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: filter_evaluate                                                  *
  *                                                                            *
  * Purpose: check if the lld data passes filter evaluation                    *
  *                                                                            *
  * Parameters: filter          - [IN] the lld filter                          *
  *             jp_row          - [IN] the lld data row                        *
  *             lld_macro_paths - [IN] use json path to extract from jp_row    *
- *                                                                            *
- * Return value: SUCCEED - the lld data passed filter evaluation              *
- *               FAIL    - otherwise                                          *
- *                                                                            *
- ******************************************************************************/
-static int	filter_evaluate(const lld_filter_t *filter, const struct zbx_json_parse *jp_row,
-		const zbx_vector_ptr_t *lld_macro_paths)
-{
-	switch (filter->evaltype)
-	{
-		case CONDITION_EVAL_TYPE_AND_OR:
-			return filter_evaluate_and_or(filter, jp_row, lld_macro_paths);
-		case CONDITION_EVAL_TYPE_AND:
-			return filter_evaluate_and(filter, jp_row, lld_macro_paths);
-		case CONDITION_EVAL_TYPE_OR:
-			return filter_evaluate_or(filter, jp_row, lld_macro_paths);
-		case CONDITION_EVAL_TYPE_EXPRESSION:
-			return filter_evaluate_expression(filter, jp_row, lld_macro_paths);
-	}
-
-	return FAIL;
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: lld_check_received_data_for_filter                               *
- *                                                                            *
- * Purpose: Check if the LLD data contains a values for macros used in filter.*
- *          Create an informative warning for every macro that has not        *
- *          received any value.                                               *
- *                                                                            *
- * Parameters: filter          - [IN] the lld filter                          *
- *             jp_row          - [IN] the lld data row                        *
- *             lld_macro_paths - [IN] use json path to extract from jp_row    *
  *             info            - [OUT] the warning description                *
  *                                                                            *
+ * Return value: SUCCEED - the lld data passed filter evaluation              *
+ *               FAIL    - otherwise                                          *
+ *                                                                            *
  ******************************************************************************/
-static void	lld_check_received_data_for_filter(lld_filter_t *filter, const struct zbx_json_parse *jp_row,
+static int	filter_evaluate(const lld_filter_t *filter, const struct zbx_json_parse *jp_row,
 		const zbx_vector_ptr_t *lld_macro_paths, char **info)
 {
-	int			i, index;
-	zbx_lld_macro_path_t	lld_macro_path_local, *lld_macro_path;
-	char			*output = NULL;
-
-	for (i = 0; i < filter->conditions.values_num; i++)
+	switch (filter->evaltype)
 	{
-		const lld_condition_t	*condition = (lld_condition_t *)filter->conditions.values[i];
-
-		lld_macro_path_local.lld_macro = condition->macro;
-
-		if (FAIL != (index = zbx_vector_ptr_bsearch(lld_macro_paths, &lld_macro_path_local,
-				zbx_lld_macro_paths_compare)))
-		{
-			lld_macro_path = (zbx_lld_macro_path_t *)lld_macro_paths->values[index];
-
-			if (FAIL == zbx_jsonpath_query(jp_row, lld_macro_path->path, &output) || NULL == output)
-			{
-				*info = zbx_strdcatf(*info,
-						"Cannot accurately apply filter: no value received for macro \"%s\""
-						" json path '%s'.\n", lld_macro_path->lld_macro, lld_macro_path->path);
-			}
-			zbx_free(output);
-
-			continue;
-		}
-
-		if (NULL == zbx_json_pair_by_name(jp_row, condition->macro))
-		{
-			*info = zbx_strdcatf(*info,
-					"Cannot accurately apply filter: no value received for macro \"%s\".\n",
-					condition->macro);
-		}
+		case CONDITION_EVAL_TYPE_AND_OR:
+			return filter_evaluate_and_or(filter, jp_row, lld_macro_paths, info);
+		case CONDITION_EVAL_TYPE_AND:
+			return filter_evaluate_and(filter, jp_row, lld_macro_paths, info);
+		case CONDITION_EVAL_TYPE_OR:
+			return filter_evaluate_or(filter, jp_row, lld_macro_paths, info);
+		case CONDITION_EVAL_TYPE_EXPRESSION:
+			return filter_evaluate_expression(filter, jp_row, lld_macro_paths, info);
 	}
+
+	return FAIL;
 }
 
 static int	lld_override_conditions_load(zbx_vector_ptr_t *overrides, const zbx_vector_uint64_t *overrideids,
@@ -755,9 +702,10 @@ static int	regexp_strmatch_condition(const char *value, const char *pattern, uns
 }
 
 void	lld_override_item(const zbx_vector_ptr_t *overrides, const char *name, const char **delay,
-		const char **history, const char **trends, unsigned char *status, unsigned char *discover)
+		const char **history, const char **trends, zbx_vector_db_tag_ptr_t *override_tags,
+		unsigned char *status, unsigned char *discover)
 {
-	int	i, j;
+	int	i, j, k;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
@@ -798,6 +746,9 @@ void	lld_override_item(const zbx_vector_ptr_t *overrides, const char *name, cons
 			if (NULL != override_operation->trends)
 				*trends = override_operation->trends;
 
+			for (k = 0; k < override_operation->tags.values_num; k++)
+				zbx_vector_db_tag_ptr_append(override_tags, override_operation->tags.values[k]);
+
 			if (NULL != status)
 			{
 				switch (override_operation->status)
@@ -1001,7 +952,8 @@ void	lld_override_graph(const zbx_vector_ptr_t *overrides, const char *name,	uns
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
-int	lld_validate_item_override_no_discover(const zbx_vector_ptr_t *overrides, const char *name)
+int	lld_validate_item_override_no_discover(const zbx_vector_ptr_t *overrides, const char *name,
+		unsigned char override_default)
 {
 	int	i, j;
 
@@ -1018,16 +970,15 @@ int	lld_validate_item_override_no_discover(const zbx_vector_ptr_t *overrides, co
 			override_operation = (const zbx_lld_override_operation_t *)override->override_operations.values[j];
 
 			if (ZBX_LLD_OVERRIDE_OP_OBJECT_ITEM == override_operation->operationtype &&
-					ZBX_PROTOTYPE_NO_DISCOVER == override_operation->discover &&
 					SUCCEED == regexp_strmatch_condition(name, override_operation->value,
 					override_operation->operator))
 			{
-				return FAIL;
+				return ZBX_PROTOTYPE_NO_DISCOVER == override_operation->discover ? FAIL : SUCCEED;
 			}
 		}
 	}
 
-	return SUCCEED;
+	return ZBX_PROTOTYPE_NO_DISCOVER == override_default ? FAIL : SUCCEED;
 }
 
 static int	lld_rows_get(const char *value, lld_filter_t *filter, zbx_vector_ptr_t *lld_rows,
@@ -1063,9 +1014,7 @@ static int	lld_rows_get(const char *value, lld_filter_t *filter, zbx_vector_ptr_
 		if (FAIL == zbx_json_brackets_open(p, &jp_row))
 			continue;
 
-		lld_check_received_data_for_filter(filter, &jp_row, lld_macro_paths, info);
-
-		if (SUCCEED != filter_evaluate(filter, &jp_row, lld_macro_paths))
+		if (SUCCEED != filter_evaluate(filter, &jp_row, lld_macro_paths, info))
 			continue;
 
 		lld_row = (zbx_lld_row_t *)zbx_malloc(NULL, sizeof(zbx_lld_row_t));
@@ -1081,9 +1030,7 @@ static int	lld_rows_get(const char *value, lld_filter_t *filter, zbx_vector_ptr_
 
 			override = (lld_override_t *)overrides->values[i];
 
-			lld_check_received_data_for_filter(&override->filter, &jp_row, lld_macro_paths, info);
-
-			if (SUCCEED != filter_evaluate(&override->filter, &jp_row, lld_macro_paths))
+			if (SUCCEED != filter_evaluate(&override->filter, &jp_row, lld_macro_paths, info))
 				continue;
 
 			zbx_vector_ptr_append(&lld_row->overrides, override);
@@ -1134,8 +1081,6 @@ static void	lld_row_free(zbx_lld_row_t *lld_row)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: lld_process_discovery_rule                                       *
  *                                                                            *
  * Purpose: add or update items, triggers and graphs for discovery item       *
  *                                                                            *
@@ -1157,6 +1102,7 @@ int	lld_process_discovery_rule(zbx_uint64_t lld_ruleid, const char *value, char
 	lld_filter_t		filter;
 	time_t			now;
 	DC_ITEM			item;
+	zbx_config_t		cfg;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() itemid:" ZBX_FS_UI64, __func__, lld_ruleid);
 
@@ -1190,7 +1136,7 @@ int	lld_process_discovery_rule(zbx_uint64_t lld_ruleid, const char *value, char
 		filter.evaltype = atoi(row[2]);
 		filter.expression = zbx_strdup(NULL, row[3]);
 		lifetime_str = zbx_strdup(NULL, row[4]);
-		substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL, NULL, NULL, NULL,
+		substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 				&lifetime_str, MACRO_TYPE_COMMON, NULL, 0);
 
 		if (SUCCEED != is_time_suffix(lifetime_str, &lifetime, ZBX_LENGTH_UNLIMITED))
@@ -1236,6 +1182,9 @@ int	lld_process_discovery_rule(zbx_uint64_t lld_ruleid, const char *value, char
 
 	now = time(NULL);
 
+	zbx_config_get(&cfg, ZBX_CONFIG_FLAGS_AUDITLOG_ENABLED);
+	zbx_audit_init(cfg.auditlog_enabled);
+
 	if (SUCCEED != lld_update_items(hostid, lld_ruleid, &lld_rows, &lld_macro_paths, error, lifetime, now))
 	{
 		zabbix_log(LOG_LEVEL_DEBUG, "cannot update/add items because parent host was removed while"
@@ -1265,6 +1214,7 @@ int	lld_process_discovery_rule(zbx_uint64_t lld_ruleid, const char *value, char
 	if (NULL != info)
 		*error = zbx_strdcat(*error, info);
 out:
+	zbx_audit_flush();
 	DCconfig_clean_items(&item, &errcode, 1);
 	zbx_free(info);
 	zbx_free(discovery_key);
