@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,11 +21,11 @@
 
 #ifdef HAVE_UNIXODBC
 
+#include "odbc.h"
+
 #include <sql.h>
 #include <sqlext.h>
-#include <sqltypes.h>
 
-#include "odbc.h"
 #include "log.h"
 #include "zbxjson.h"
 #include "zbxalgo.h"
@@ -47,8 +47,6 @@ struct zbx_odbc_query_result
 #define ZBX_FLAG_ODBC_LLD	0x01
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_rc_str                                                  *
  *                                                                            *
  * Purpose: get human readable representation of ODBC return code             *
  *                                                                            *
@@ -82,8 +80,6 @@ static const char	*zbx_odbc_rc_str(SQLRETURN rc)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_diag                                                    *
  *                                                                            *
  * Purpose: diagnose result of ODBC function call                             *
  *                                                                            *
@@ -148,8 +144,6 @@ static int	zbx_odbc_diag(SQLSMALLINT h_type, SQLHANDLE h, SQLRETURN rc, char **d
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_log_odbc_connection_info                                     *
  *                                                                            *
  * Purpose: log details upon successful connection on behalf of caller        *
  *                                                                            *
@@ -204,8 +198,6 @@ static void	zbx_log_odbc_connection_info(const char *function, SQLHDBC hdbc)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_connection_string_append                                *
  *                                                                            *
  * Purpose: Appends a new argument to ODBC connection string.                 *
  *          Connection string is reallocated to fit new value.                *
@@ -231,8 +223,6 @@ static void zbx_odbc_connection_string_append(char **connection_str, const char
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_connect                                                 *
  *                                                                            *
  * Purpose: connect to ODBC data source                                       *
  *                                                                            *
@@ -343,8 +333,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_data_source_free                                        *
  *                                                                            *
  * Purpose: free resources allocated by successful zbx_odbc_connect() call    *
  *                                                                            *
@@ -363,8 +351,6 @@ void	zbx_odbc_data_source_free(zbx_odbc_data_source_t *data_source)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_select                                                  *
  *                                                                            *
  * Purpose: execute a query to ODBC data source                               *
  *                                                                            *
@@ -443,8 +429,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_query_result_free                                       *
  *                                                                            *
  * Purpose: free resources allocated by successful zbx_odbc_select() call     *
  *                                                                            *
@@ -468,8 +452,6 @@ void	zbx_odbc_query_result_free(zbx_odbc_query_result_t *query_result)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_fetch                                                   *
  *                                                                            *
  * Purpose: fetch single row of ODBC query result                             *
  *                                                                            *
@@ -548,8 +530,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_query_result_to_string                                  *
  *                                                                            *
  * Purpose: extract the first column of the first row of ODBC SQL query       *
  *                                                                            *
@@ -593,8 +573,6 @@ int	zbx_odbc_query_result_to_string(zbx_odbc_query_result_t *query_result, char
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: odbc_query_result_to_json                                        *
  *                                                                            *
  * Purpose: convert ODBC SQL query result into JSON                           *
  *                                                                            *
@@ -716,8 +694,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_query_result_to_lld_json                                *
  *                                                                            *
  * Purpose: public wrapper for odbc_query_result_to_json                      *
  *                                                                            *
@@ -728,8 +704,6 @@ int	zbx_odbc_query_result_to_lld_json(zbx_odbc_query_result_t *query_result, cha
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_odbc_query_result_to_json                                    *
  *                                                                            *
  * Purpose: public wrapper for odbc_query_result_to_json                      *
  *                                                                            *
