@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,29 +17,23 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
+#include "item_preproc.h"
+
+#include "zbxregexp.h"
 #include "log.h"
+#include "zbxembed.h"
+#include "zbxprometheus.h"
 
 /* LIBXML2 is used */
 #ifdef HAVE_LIBXML2
 #	include <libxml/parser.h>
-#	include <libxml/tree.h>
 #	include <libxml/xpath.h>
 #endif
 
-#include "zbxregexp.h"
-#include "zbxjson.h"
-#include "zbxembed.h"
-#include "zbxprometheus.h"
 #include "preproc_history.h"
 
-#include "item_preproc.h"
-
 extern zbx_es_t	es_engine;
-
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_numeric_type_hint                                   *
  *                                                                            *
  * Purpose: returns numeric type hint based on item value type                *
  *                                                                            *
@@ -62,8 +56,6 @@ static int	item_preproc_numeric_type_hint(unsigned char value_type)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_convert_value                                       *
  *                                                                            *
  * Purpose: convert variant value to the requested type                       *
  *                                                                            *
@@ -87,8 +79,6 @@ static int	item_preproc_convert_value(zbx_variant_t *value, unsigned char type,
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_item_preproc_convert_value_to_numeric                        *
  *                                                                            *
  * Purpose: converts variant value to numeric                                 *
  *                                                                            *
@@ -133,8 +123,6 @@ int	zbx_item_preproc_convert_value_to_numeric(zbx_variant_t *value_num, const zb
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_multiplier_variant                                  *
  *                                                                            *
  * Purpose: execute custom multiplier preprocessing operation on variant      *
  *          value type                                                        *
@@ -182,8 +170,6 @@ static int	item_preproc_multiplier_variant(unsigned char value_type, zbx_variant
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_validate_notsupport                                 *
  *                                                                            *
  * Purpose: executes during notsupported item preprocessing                   *
  *                                                                            *
@@ -197,8 +183,6 @@ static int	item_preproc_validate_notsupport(char **errmsg)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_multiplier                                          *
  *                                                                            *
  * Purpose: execute custom multiplier preprocessing operation                 *
  *                                                                            *
@@ -234,8 +218,6 @@ static int	item_preproc_multiplier(unsigned char value_type, zbx_variant_t *valu
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_delta_float                                         *
  *                                                                            *
  * Purpose: execute delta type preprocessing operation                        *
  *                                                                            *
@@ -274,8 +256,6 @@ static int	item_preproc_delta_float(zbx_variant_t *value, const zbx_timespec_t *
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_delta_uint64                                        *
  *                                                                            *
  * Purpose: execute delta type preprocessing operation                        *
  *                                                                            *
@@ -314,8 +294,6 @@ static int	item_preproc_delta_uint64(zbx_variant_t *value, const zbx_timespec_t
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_delta                                               *
  *                                                                            *
  * Purpose: execute delta type preprocessing operation                        *
  *                                                                            *
@@ -334,7 +312,6 @@ static int	item_preproc_delta_uint64(zbx_variant_t *value, const zbx_timespec_t
 static int	item_preproc_delta(unsigned char value_type, zbx_variant_t *value, const zbx_timespec_t *ts,
 		unsigned char op_type, zbx_variant_t *history_value, zbx_timespec_t *history_ts, char **errmsg)
 {
-	int				ret = FAIL;
 	zbx_variant_t			value_num;
 
 	if (FAIL == zbx_item_preproc_convert_value_to_numeric(&value_num, value, value_type, errmsg))
@@ -344,6 +321,8 @@ static int	item_preproc_delta(unsigned char value_type, zbx_variant_t *value, co
 
 	if (ZBX_VARIANT_NONE != history_value->type)
 	{
+		int				ret;
+
 		zbx_variant_copy(value, &value_num);
 
 		if (ZBX_VARIANT_DBL == value->type || ZBX_VARIANT_DBL == history_value->type)
@@ -372,8 +351,6 @@ static int	item_preproc_delta(unsigned char value_type, zbx_variant_t *value, co
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_delta_value                                         *
  *                                                                            *
  * Purpose: execute delta (simple change) preprocessing operation             *
  *                                                                            *
@@ -408,8 +385,6 @@ static int	item_preproc_delta_value(unsigned char value_type, zbx_variant_t *val
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_delta_speed                                         *
  *                                                                            *
  * Purpose: execute delta (speed per second) preprocessing operation          *
  *                                                                            *
@@ -444,8 +419,6 @@ static int	item_preproc_delta_speed(unsigned char value_type, zbx_variant_t *val
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: unescape_param                                                   *
  *                                                                            *
  * Purpose: copy first n chars from in to out, unescape escaped characters    *
  *          during copying                                                    *
@@ -497,8 +470,6 @@ static void	unescape_param(int op_type, const char *in, int len, char *out)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_trim                                                *
  *                                                                            *
  * Purpose: execute trim type preprocessing operation                         *
  *                                                                            *
@@ -530,8 +501,6 @@ static int item_preproc_trim(zbx_variant_t *value, unsigned char op_type, const
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_rtrim                                               *
  *                                                                            *
  * Purpose: execute right trim preprocessing operation                        *
  *                                                                            *
@@ -561,8 +530,6 @@ static int item_preproc_rtrim(zbx_variant_t *value, const char *params, char **e
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_ltrim                                               *
  *                                                                            *
  * Purpose: execute left trim preprocessing operation                         *
  *                                                                            *
@@ -592,8 +559,6 @@ static int item_preproc_ltrim(zbx_variant_t *value, const char *params, char **e
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_lrtrim                                              *
  *                                                                            *
  * Purpose: execute left and right trim preprocessing operation               *
  *                                                                            *
@@ -623,8 +588,6 @@ static int item_preproc_lrtrim(zbx_variant_t *value, const char *params, char **
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_2dec                                                *
  *                                                                            *
  * Purpose: execute decimal value conversion operation                        *
  *                                                                            *
@@ -688,8 +651,6 @@ static int	item_preproc_2dec(zbx_variant_t *value, unsigned char op_type, char *
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_bool2dec                                            *
  *                                                                            *
  * Purpose: execute boolean to decimal value conversion operation             *
  *                                                                            *
@@ -716,8 +677,6 @@ static int	item_preproc_bool2dec(zbx_variant_t *value, char **errmsg)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_oct2dec                                             *
  *                                                                            *
  * Purpose: execute octal to decimal value conversion operation               *
  *                                                                            *
@@ -744,8 +703,6 @@ static int	item_preproc_oct2dec(zbx_variant_t *value, char **errmsg)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_hex2dec                                             *
  *                                                                            *
  * Purpose: execute hexadecimal to decimal value conversion operation         *
  *                                                                            *
@@ -772,8 +729,6 @@ static int	item_preproc_hex2dec(zbx_variant_t *value, char **errmsg)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_regsub_op                                           *
  *                                                                            *
  * Purpose: execute regular expression substitution operation                 *
  *                                                                            *
@@ -808,6 +763,7 @@ static int	item_preproc_regsub_op(zbx_variant_t *value, const char *params, char
 	if (FAIL == zbx_regexp_compile_ext(pattern, &regex, 0, &regex_error))	/* PCRE_MULTILINE is not used here */
 	{
 		*errmsg = zbx_dsprintf(*errmsg, "invalid regular expression: %s", regex_error);
+		zbx_regexp_err_msg_free(regex_error);
 		return FAIL;
 	}
 
@@ -827,8 +783,6 @@ static int	item_preproc_regsub_op(zbx_variant_t *value, const char *params, char
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_regsub                                              *
  *                                                                            *
  * Purpose: execute regular expression substitution operation                 *
  *                                                                            *
@@ -862,8 +816,6 @@ static int	item_preproc_regsub(zbx_variant_t *value, const char *params, char **
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_jsonpath_op                                         *
  *                                                                            *
  * Purpose: execute jsonpath query                                            *
  *                                                                            *
@@ -902,8 +854,6 @@ static int	item_preproc_jsonpath_op(zbx_variant_t *value, const char *params, ch
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_jsonpath                                            *
  *                                                                            *
  * Purpose: execute jsonpath query                                            *
  *                                                                            *
@@ -930,119 +880,6 @@ static int	item_preproc_jsonpath(zbx_variant_t *value, const char *params, char
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_xpath_op                                            *
- *                                                                            *
- * Purpose: execute xpath query                                               *
- *                                                                            *
- * Parameters: value  - [IN/OUT] the value to process                         *
- *             params - [IN] the operation parameters                         *
- *             errmsg - [OUT] error message                                   *
- *                                                                            *
- * Return value: SUCCEED - the value was processed successfully               *
- *               FAIL - otherwise                                             *
- *                                                                            *
- ******************************************************************************/
-static int	item_preproc_xpath_op(zbx_variant_t *value, const char *params, char **errmsg)
-{
-#ifndef HAVE_LIBXML2
-	ZBX_UNUSED(value);
-	ZBX_UNUSED(params);
-	*errmsg = zbx_dsprintf(*errmsg, "Zabbix was compiled without libxml2 support");
-	return FAIL;
-#else
-	xmlDoc		*doc = NULL;
-	xmlXPathContext	*xpathCtx;
-	xmlXPathObject	*xpathObj;
-	xmlNodeSetPtr	nodeset;
-	xmlErrorPtr	pErr;
-	xmlBufferPtr	xmlBufferLocal;
-	int		ret = FAIL, i;
-	char		buffer[32], *ptr;
-
-	if (FAIL == item_preproc_convert_value(value, ZBX_VARIANT_STR, errmsg))
-		return FAIL;
-
-	if (NULL == (doc = xmlReadMemory(value->data.str, strlen(value->data.str), "noname.xml", NULL, 0)))
-	{
-		if (NULL != (pErr = xmlGetLastError()))
-			*errmsg = zbx_dsprintf(*errmsg, "cannot parse xml value: %s", pErr->message);
-		else
-			*errmsg = zbx_strdup(*errmsg, "cannot parse xml value");
-		return FAIL;
-	}
-
-	xpathCtx = xmlXPathNewContext(doc);
-
-	if (NULL == (xpathObj = xmlXPathEvalExpression((xmlChar *)params, xpathCtx)))
-	{
-		pErr = xmlGetLastError();
-		*errmsg = zbx_dsprintf(*errmsg, "cannot parse xpath: %s", pErr->message);
-		goto out;
-	}
-
-	switch (xpathObj->type)
-	{
-		case XPATH_NODESET:
-			xmlBufferLocal = xmlBufferCreate();
-
-			if (0 == xmlXPathNodeSetIsEmpty(xpathObj->nodesetval))
-			{
-				nodeset = xpathObj->nodesetval;
-				for (i = 0; i < nodeset->nodeNr; i++)
-					xmlNodeDump(xmlBufferLocal, doc, nodeset->nodeTab[i], 0, 0);
-			}
-			zbx_variant_clear(value);
-			zbx_variant_set_str(value, zbx_strdup(NULL, (const char *)xmlBufferLocal->content));
-
-			xmlBufferFree(xmlBufferLocal);
-			ret = SUCCEED;
-			break;
-		case XPATH_STRING:
-			zbx_variant_clear(value);
-			zbx_variant_set_str(value, zbx_strdup(NULL, (const char *)xpathObj->stringval));
-			ret = SUCCEED;
-			break;
-		case XPATH_BOOLEAN:
-			zbx_variant_clear(value);
-			zbx_variant_set_str(value, zbx_dsprintf(NULL, "%d", xpathObj->boolval));
-			ret = SUCCEED;
-			break;
-		case XPATH_NUMBER:
-			zbx_variant_clear(value);
-			zbx_snprintf(buffer, sizeof(buffer), ZBX_FS_DBL, xpathObj->floatval);
-
-			/* check for nan/inf values - isnan(), isinf() is not supported by c89/90    */
-			/* so simply check the if the result starts with digit (accounting for -inf) */
-			if (*(ptr = buffer) == '-')
-				ptr++;
-			if (0 != isdigit(*ptr))
-			{
-				del_zeros(buffer);
-				zbx_variant_set_str(value, zbx_strdup(NULL, buffer));
-				ret = SUCCEED;
-			}
-			else
-				*errmsg = zbx_strdup(*errmsg, "Invalid numeric value");
-			break;
-		default:
-			*errmsg = zbx_strdup(*errmsg, "Unknown result");
-			break;
-	}
-out:
-	if (NULL != xpathObj)
-		xmlXPathFreeObject(xpathObj);
-
-	xmlXPathFreeContext(xpathCtx);
-	xmlFreeDoc(doc);
-
-	return ret;
-#endif
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: item_preproc_xpath                                               *
  *                                                                            *
  * Purpose: execute xpath query                                               *
  *                                                                            *
@@ -1058,7 +895,10 @@ static int	item_preproc_xpath(zbx_variant_t *value, const char *params, char **e
 {
 	char	*err = NULL;
 
-	if (SUCCEED == item_preproc_xpath_op(value, params, &err))
+	if (FAIL == item_preproc_convert_value(value, ZBX_VARIANT_STR, errmsg))
+		return FAIL;
+
+	if (SUCCEED == zbx_query_xpath(value, params, &err))
 		return SUCCEED;
 
 	*errmsg = zbx_dsprintf(*errmsg, "cannot extract XML value with xpath \"%s\": %s", params, err);
@@ -1068,8 +908,6 @@ static int	item_preproc_xpath(zbx_variant_t *value, const char *params, char **e
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_validate_range                                      *
  *                                                                            *
  * Purpose: validates value to be within the specified range                  *
  * Parameters: value_type - [IN] the item type                                *
@@ -1146,8 +984,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_validate_regex                                      *
  *                                                                            *
  * Purpose: validates value to match regular expression                       *
  * Parameters: value_type - [IN] the item type                                *
@@ -1178,6 +1014,7 @@ static int	item_preproc_validate_regex(const zbx_variant_t *value, const char *p
 	if (FAIL == zbx_regexp_compile(params, &regex, &errptr))
 	{
 		errmsg = zbx_dsprintf(NULL, "invalid regular expression pattern: %s", errptr);
+		zbx_regexp_err_msg_free(errptr);
 		goto out;
 	}
 
@@ -1202,8 +1039,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_validate_not_regex                                  *
  *                                                                            *
  * Purpose: validates value to not match regular expression                   *
  * Parameters: value_type - [IN] the item type                                *
@@ -1234,6 +1069,7 @@ static int	item_preproc_validate_not_regex(const zbx_variant_t *value, const cha
 	if (FAIL == zbx_regexp_compile(params, &regex, &errptr))
 	{
 		errmsg = zbx_dsprintf(NULL, "invalid regular expression pattern: %s", errptr);
+		zbx_regexp_err_msg_free(errptr);
 		goto out;
 	}
 
@@ -1260,8 +1096,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_get_error_from_json                                 *
  *                                                                            *
  * Purpose: checks for presence of error field in json data                   *
  *                                                                            *
@@ -1316,8 +1150,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_get_error_from_xml                                  *
  *                                                                            *
  * Purpose: checks for presence of error field in XML data                    *
  *                                                                            *
@@ -1422,8 +1254,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_get_error_from_regex                                *
  *                                                                            *
  * Purpose: checks for presence of error pattern matching regular expression  *
  *                                                                            *
@@ -1486,8 +1316,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_throttle_value                                      *
  *                                                                            *
  * Purpose: throttles value by suppressing identical values                   *
  *                                                                            *
@@ -1520,8 +1348,6 @@ static int	item_preproc_throttle_value(zbx_variant_t *value, const zbx_timespec_
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_throttle_timed_value                                *
  *                                                                            *
  * Purpose: throttles value by suppressing identical values                   *
  *                                                                            *
@@ -1565,8 +1391,6 @@ static int	item_preproc_throttle_timed_value(zbx_variant_t *value, const zbx_tim
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_script                                              *
  *                                                                            *
  * Purpose: executes script passed with params                                *
  *                                                                            *
@@ -1629,8 +1453,6 @@ fail:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_prometheus_pattern                                  *
  *                                                                            *
  * Purpose: parse Prometheus format metrics                                   *
  *                                                                            *
@@ -1642,25 +1464,61 @@ fail:
  *               FAIL - otherwise                                             *
  *                                                                            *
  ******************************************************************************/
-static int	item_preproc_prometheus_pattern(zbx_variant_t *value, const char *params, char **errmsg)
+static int	item_preproc_prometheus_pattern(zbx_preproc_cache_t *cache, zbx_variant_t *value, const char *params,
+		char **errmsg)
 {
-	char	pattern[ITEM_PREPROC_PARAMS_LEN * ZBX_MAX_BYTES_IN_UTF8_CHAR + 1], *output, *value_out = NULL,
+	char	pattern[ITEM_PREPROC_PARAMS_LEN * ZBX_MAX_BYTES_IN_UTF8_CHAR + 1], *request, *output, *value_out = NULL,
 		*err = NULL;
-
-	if (FAIL == item_preproc_convert_value(value, ZBX_VARIANT_STR, errmsg))
-		return FAIL;
+	int	ret = FAIL;
 
 	zbx_strlcpy(pattern, params, sizeof(pattern));
 
-	if (NULL == (output = strchr(pattern, '\n')))
+	if (NULL == (request = strchr(pattern, '\n')))
 	{
 		*errmsg = zbx_strdup(*errmsg, "cannot find second parameter");
 		return FAIL;
 	}
+	*request++ = '\0';
 
+	if (NULL == (output = strchr(request, '\n')))
+	{
+		*errmsg = zbx_strdup(*errmsg, "cannot find third parameter");
+		return FAIL;
+	}
 	*output++ = '\0';
 
-	if (FAIL == zbx_prometheus_pattern(value->data.str, pattern, output, &value_out, &err))
+	if (NULL == cache)
+	{
+		if (FAIL == item_preproc_convert_value(value, ZBX_VARIANT_STR, errmsg))
+			return FAIL;
+
+		ret = zbx_prometheus_pattern(value->data.str, pattern, request, output, &value_out, &err);
+	}
+	else
+	{
+		zbx_prometheus_t	*prom_cache;
+
+		if (NULL == (prom_cache = (zbx_prometheus_t *)zbx_preproc_cache_get(cache,
+				ZBX_PREPROC_PROMETHEUS_PATTERN)))
+		{
+			prom_cache = (zbx_prometheus_t *)zbx_malloc(NULL, sizeof(zbx_prometheus_t));
+
+			if (FAIL == item_preproc_convert_value(value, ZBX_VARIANT_STR, errmsg))
+				return FAIL;
+
+			if (SUCCEED != zbx_prometheus_init(prom_cache, value->data.str, &err))
+			{
+				zbx_free(prom_cache);
+				goto out;
+			}
+
+			zbx_preproc_cache_put(cache, ZBX_PREPROC_PROMETHEUS_PATTERN, prom_cache);
+		}
+
+		ret = zbx_prometheus_pattern_ex(prom_cache, pattern, request, output, &value_out, &err);
+	}
+out:
+	if (FAIL == ret)
 	{
 		*errmsg = zbx_dsprintf(*errmsg, "cannot apply Prometheus pattern: %s", err);
 		zbx_free(err);
@@ -1674,8 +1532,6 @@ static int	item_preproc_prometheus_pattern(zbx_variant_t *value, const char *par
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_prometheus_to_json                                  *
  *                                                                            *
  * Purpose: convert Prometheus format metrics to JSON format                  *
  *                                                                            *
@@ -1708,8 +1564,6 @@ static int	item_preproc_prometheus_to_json(zbx_variant_t *value, const char *par
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_csv_to_json_add_field                               *
  *                                                                            *
  * Purpose: convert CSV format metrics to JSON format                         *
  *                                                                            *
@@ -1784,8 +1638,6 @@ static int	item_preproc_csv_to_json_add_field(struct zbx_json *json, char ***nam
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: item_preproc_csv_to_json                                         *
  *                                                                            *
  * Purpose: convert CSV format metrics to JSON format                         *
  *                                                                            *
@@ -2043,7 +1895,32 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Function: item_preproc_str_replace                                         *
+ * Purpose: convert XML format value to JSON format                           *
+ *                                                                            *
+ * Parameters: value  - [IN/OUT] the value to process                         *
+ *             errmsg - [OUT] error message                                   *
+ *                                                                            *
+ * Return value: SUCCEED - the value was processed successfully               *
+ *               FAIL - otherwise                                             *
+ *                                                                            *
+ ******************************************************************************/
+static int	item_preproc_xml_to_json(zbx_variant_t *value, char **errmsg)
+{
+	char	*json = NULL;
+
+	if (FAIL == item_preproc_convert_value(value, ZBX_VARIANT_STR, errmsg))
+		return FAIL;
+
+	if (FAIL == zbx_xml_to_json(value->data.str, &json, errmsg))
+		return FAIL;
+
+	zbx_variant_clear(value);
+	zbx_variant_set_str(value, json);
+
+	return SUCCEED;
+}
+
+/******************************************************************************
  *                                                                            *
  * Purpose: replace substrings in string                                      *
  *                                                                            *
@@ -2094,12 +1971,11 @@ static int	item_preproc_str_replace(zbx_variant_t *value, const char *params, ch
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_item_preproc                                                 *
  *                                                                            *
  * Purpose: execute preprocessing operation                                   *
  *                                                                            *
- * Parameters: value_type    - [IN] the item value type                       *
+ * Parameters: cache         - [IN/OUT] the preprocessing cache               *
+ *             value_type    - [IN] the item value type                       *
  *             value         - [IN/OUT] the value to process                  *
  *             ts            - [IN] the value timestamp                       *
  *             op            - [IN] the preprocessing operation to execute    *
@@ -2115,8 +1991,9 @@ static int	item_preproc_str_replace(zbx_variant_t *value, const char *params, ch
  *           returned with error set.                                         *
  *                                                                            *
  ******************************************************************************/
-int	zbx_item_preproc(unsigned char value_type, zbx_variant_t *value, const zbx_timespec_t *ts,
-		const zbx_preproc_op_t *op, zbx_variant_t *history_value, zbx_timespec_t *history_ts, char **error)
+int	zbx_item_preproc(zbx_preproc_cache_t *cache, unsigned char value_type, zbx_variant_t *value,
+		const zbx_timespec_t *ts, const zbx_preproc_op_t *op, zbx_variant_t *history_value,
+		zbx_timespec_t *history_ts, char **error)
 {
 	int	ret;
 
@@ -2187,7 +2064,7 @@ int	zbx_item_preproc(unsigned char value_type, zbx_variant_t *value, const zbx_t
 			ret = item_preproc_script(value, op->params, history_value, error);
 			break;
 		case ZBX_PREPROC_PROMETHEUS_PATTERN:
-			ret = item_preproc_prometheus_pattern(value, op->params, error);
+			ret = item_preproc_prometheus_pattern(cache, value, op->params, error);
 			break;
 		case ZBX_PREPROC_PROMETHEUS_TO_JSON:
 			ret = item_preproc_prometheus_to_json(value, op->params, error);
@@ -2201,6 +2078,9 @@ int	zbx_item_preproc(unsigned char value_type, zbx_variant_t *value, const zbx_t
 		case ZBX_PREPROC_VALIDATE_NOT_SUPPORTED:
 			ret = item_preproc_validate_notsupport(error);
 			break;
+		case ZBX_PREPROC_XML_TO_JSON:
+			ret = item_preproc_xml_to_json(value, error);
+			break;
 		default:
 			*error = zbx_dsprintf(*error, "unknown preprocessing operation");
 			ret = FAIL;
@@ -2210,8 +2090,6 @@ int	zbx_item_preproc(unsigned char value_type, zbx_variant_t *value, const zbx_t
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_item_preproc_handle_error                                    *
  *                                                                            *
  * Purpose: apply 'on fail' preprocessing error handler                       *
  *                                                                            *
@@ -2248,8 +2126,6 @@ int	zbx_item_preproc_handle_error(zbx_variant_t *value, const zbx_preproc_op_t *
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_item_preproc_test                                            *
  *                                                                            *
  * Purpose: test preprocessing steps                                          *
  *                                                                            *
@@ -2282,7 +2158,8 @@ int	zbx_item_preproc_test(unsigned char value_type, zbx_variant_t *value, const
 
 		zbx_preproc_history_pop_value(history_in, i, &history_value, &history_ts);
 
-		if (FAIL == (ret = zbx_item_preproc(value_type, value, ts, op, &history_value, &history_ts, error)))
+		if (FAIL == (ret = zbx_item_preproc(NULL, value_type, value, ts, op, &history_value, &history_ts,
+				error)))
 		{
 			results[i].action = op->error_handler;
 			results[i].error = zbx_strdup(NULL, *error);
