@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "threads.h"
 
 extern char		*CONFIG_SNMPTRAP_FILE;
-extern unsigned char	process_type;
+extern ZBX_THREAD_LOCAL unsigned char	process_type;
 
 ZBX_THREAD_ENTRY(snmptrapper_thread, args);
 
