@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,30 +17,28 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
+#include "proxydata.h"
 #include "db.h"
 #include "log.h"
 #include "proxy.h"
 
-#include "proxydata.h"
-#include "../../libs/zbxcrypto/tls_tcp_active.h"
 #include "zbxtasks.h"
 #include "mutexs.h"
 #include "daemon.h"
 #include "zbxcompress.h"
 
+
 extern unsigned char	program_type;
 static zbx_mutex_t	proxy_lock = ZBX_MUTEX_NULL;
 
 #define	LOCK_PROXY_HISTORY	if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE)) zbx_mutex_lock(proxy_lock)
 #define	UNLOCK_PROXY_HISTORY	if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE)) zbx_mutex_unlock(proxy_lock)
 
-int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, const char *info, int status,
-		int upload_status)
+int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, const char *info, int upload_status)
 {
 	struct zbx_json		json;
 	zbx_vector_ptr_t	tasks;
-	int			ret, flags = ZBX_TCP_PROTOCOL;
+	int			ret, flags = ZBX_TCP_PROTOCOL, status;
 
 	zbx_vector_ptr_create(&tasks);
 
@@ -51,11 +49,14 @@ int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, cons
 		case ZBX_PROXY_UPLOAD_DISABLED:
 			zbx_json_addstring(&json, ZBX_PROTO_TAG_PROXY_UPLOAD, ZBX_PROTO_VALUE_PROXY_UPLOAD_DISABLED,
 					ZBX_JSON_TYPE_STRING);
+			status = FAIL;
 			break;
 		case ZBX_PROXY_UPLOAD_ENABLED:
 			zbx_json_addstring(&json, ZBX_PROTO_TAG_PROXY_UPLOAD, ZBX_PROTO_VALUE_PROXY_UPLOAD_ENABLED,
 					ZBX_JSON_TYPE_STRING);
-			break;
+			ZBX_FALLTHROUGH;
+		default:
+			status = SUCCEED;
 	}
 
 	if (SUCCEED == status)
@@ -90,35 +91,6 @@ int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, cons
 }
 
 /******************************************************************************
- *                                                                            *
- * Purpose: check if the 'proxy data' packet has historical data              *
- *                                                                            *
- * Return value: SUCCEED - the 'proxy data' contains no historical records    *
- *               FAIL    - otherwise                                          *
- *                                                                            *
- ******************************************************************************/
-static int	proxy_data_no_history(const struct zbx_json_parse *jp)
-{
-	struct zbx_json_parse	jp_data;
-
-	if (SUCCEED == zbx_json_brackets_by_name(jp, ZBX_PROTO_TAG_HISTORY_DATA, &jp_data))
-		return FAIL;
-
-	if (SUCCEED == zbx_json_brackets_by_name(jp, ZBX_PROTO_TAG_DISCOVERY_DATA, &jp_data))
-		return FAIL;
-
-	if (SUCCEED == zbx_json_brackets_by_name(jp, ZBX_PROTO_TAG_AUTOREGISTRATION, &jp_data))
-		return FAIL;
-
-	if (SUCCEED == zbx_json_brackets_by_name(jp, ZBX_PROTO_TAG_HOST_AVAILABILITY, &jp_data))
-		return FAIL;
-
-	return SUCCEED;
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: zbx_recv_proxy_data                                              *
  *                                                                            *
  * Purpose: receive 'proxy data' request from proxy                           *
  *                                                                            *
@@ -159,13 +131,11 @@ void	zbx_recv_proxy_data(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_time
 	if (FAIL == (ret = zbx_hc_check_proxy(proxy.hostid)))
 	{
 		upload_status = ZBX_PROXY_UPLOAD_DISABLED;
-		ret = proxy_data_no_history(jp);
 	}
 	else
+	{
 		upload_status = ZBX_PROXY_UPLOAD_ENABLED;
 
-	if (SUCCEED == ret)
-	{
 		if (SUCCEED != (ret = process_proxy_data(&proxy, jp, ts, HOST_STATUS_PROXY_ACTIVE, NULL, &error)))
 		{
 			zabbix_log(LOG_LEVEL_WARNING, "received invalid proxy data from proxy \"%s\" at \"%s\": %s",
@@ -183,7 +153,7 @@ void	zbx_recv_proxy_data(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_time
 		goto out;
 	}
 
-	zbx_send_proxy_data_response(&proxy, sock, error, ret, upload_status);
+	zbx_send_proxy_data_response(&proxy, sock, error, upload_status);
 	responded = 1;
 
 out:
@@ -217,8 +187,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: send_data_to_server                                              *
  *                                                                            *
  * Purpose: sends data from proxy to server                                   *
  *                                                                            *
@@ -246,8 +214,6 @@ static int	send_data_to_server(zbx_socket_t *sock, char **buffer, size_t buffer_
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_send_proxy_data                                              *
  *                                                                            *
  * Purpose: sends 'proxy data' request to server                              *
  *                                                                            *
@@ -277,7 +243,7 @@ void	zbx_send_proxy_data(zbx_socket_t *sock, zbx_timespec_t *ts)
 	zbx_json_init(&j, ZBX_JSON_STAT_BUF_LEN);
 
 	zbx_json_addstring(&j, ZBX_PROTO_TAG_SESSION, zbx_dc_get_session_token(), ZBX_JSON_TYPE_STRING);
-	get_host_availability_data(&j, &availability_ts);
+	get_interface_availability_data(&j, &availability_ts);
 	proxy_get_hist_data(&j, &history_lastid, &more_history);
 	proxy_get_dhis_data(&j, &discovery_lastid, &more_discovery);
 	proxy_get_areg_data(&j, &areg_lastid, &more_areg);
@@ -358,10 +324,7 @@ out:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
-
 /******************************************************************************
- *                                                                            *
- * Function: zbx_send_task_data                                               *
  *                                                                            *
  * Purpose: sends 'proxy data' request to server                              *
  *                                                                            *
@@ -455,4 +418,3 @@ void	free_proxy_history_lock(void)
 	if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
 		zbx_mutex_destroy(&proxy_lock);
 }
-
