@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 #ifndef ZABBIX_PROXYDATA_H
 #define ZABBIX_PROXYDATA_H
 
-#include "comms.h"
 #include "zbxjson.h"
+#include "dbcache.h"
 
 extern int	CONFIG_TIMEOUT;
 extern int	CONFIG_TRAPPER_TIMEOUT;
@@ -30,8 +30,7 @@ void	zbx_recv_proxy_data(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_time
 void	zbx_send_proxy_data(zbx_socket_t *sock, zbx_timespec_t *ts);
 void	zbx_send_task_data(zbx_socket_t *sock, zbx_timespec_t *ts);
 
-int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, const char *info, int status,
-		int upload_status);
+int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, const char *info, int upload_status);
 
 int	init_proxy_history_lock(char **error);
 void	free_proxy_history_lock(void);
