@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,45 +17,39 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
+#include "trapper.h"
 
-#include "comms.h"
 #include "log.h"
-#include "zbxjson.h"
-#include "zbxserver.h"
-#include "dbcache.h"
 #include "proxy.h"
 #include "zbxself.h"
-
-#include "trapper.h"
 #include "active.h"
 #include "nodecommand.h"
 #include "proxyconfig.h"
 #include "proxydata.h"
-#include "../alerter/alerter_protocol.h"
-#include "trapper_preproc.h"
-#include "trapper_expressions_evaluate.h"
-
 #include "daemon.h"
 #include "zbxcrypto.h"
 #include "../../libs/zbxserver/zabbix_stats.h"
-#include "zbxipcservice.h"
-#include "trapper_item_test.h"
 #include "../poller/checks_snmp.h"
+#include "trapper_auth.h"
+#include "trapper_preproc.h"
+#include "trapper_expressions_evaluate.h"
+#include "trapper_item_test.h"
+#include "trapper_request.h"
+
+#ifdef HAVE_NETSNMP
+#	include "zbxrtc.h"
+#endif
 
 #define ZBX_MAX_SECTION_ENTRIES		4
 #define ZBX_MAX_ENTRY_ATTRIBUTES	3
 
-extern unsigned char	process_type, program_type;
-extern int		server_num, process_num;
-extern size_t		(*find_psk_in_cache)(const unsigned char *, unsigned char *, unsigned int *);
+extern ZBX_THREAD_LOCAL unsigned char	process_type;
+extern unsigned char			program_type;
+extern ZBX_THREAD_LOCAL int		server_num, process_num;
+extern size_t				(*find_psk_in_cache)(const unsigned char *, unsigned char *, unsigned int *);
 
 extern int	CONFIG_CONFSYNCER_FORKS;
 
-#ifdef HAVE_NETSNMP
-static volatile sig_atomic_t	snmp_cache_reload_requested;
-#endif
-
 typedef struct
 {
 	zbx_counter_value_t	online;
@@ -103,8 +97,6 @@ typedef struct
 zbx_status_section_t;
 
 /******************************************************************************
- *                                                                            *
- * Function: recv_agenthistory                                                *
  *                                                                            *
  * Purpose: processes the received values from active agents                  *
  *                                                                            *
@@ -135,8 +127,6 @@ static void	recv_agenthistory(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: recv_senderhistory                                               *
  *                                                                            *
  * Purpose: processes the received values from senders                        *
  *                                                                            *
@@ -167,16 +157,12 @@ static void	recv_senderhistory(zbx_socket_t *sock, struct zbx_json_parse *jp, zb
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: recv_proxy_heartbeat                                             *
  *                                                                            *
  * Purpose: process heartbeat sent by proxy servers                           *
  *                                                                            *
  * Return value:  SUCCEED - processed successfully                            *
  *                FAIL - an error occurred                                    *
  *                                                                            *
- * Author: Alexander Vladishev                                                *
- *                                                                            *
  ******************************************************************************/
 static void	recv_proxy_heartbeat(zbx_socket_t *sock, struct zbx_json_parse *jp)
 {
@@ -235,8 +221,6 @@ typedef struct
 zbx_queue_stats_t;
 
 /******************************************************************************
- *                                                                            *
- * Function: queue_stats_update                                               *
  *                                                                            *
  * Purpose: update queue stats with a new item delay                          *
  *                                                                            *
@@ -261,8 +245,6 @@ static void	queue_stats_update(zbx_queue_stats_t *stats, int delay)
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: queue_stats_export                                               *
  *                                                                            *
  * Purpose: export queue stats to JSON format                                 *
  *                                                                            *
@@ -305,8 +287,6 @@ static int	queue_compare_by_nextcheck_asc(zbx_queue_item_t **d1, zbx_queue_item_
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: recv_getqueue                                                    *
  *                                                                            *
  * Purpose: process queue request                                             *
  *                                                                            *
@@ -320,7 +300,7 @@ static int	queue_compare_by_nextcheck_asc(zbx_queue_item_t **d1, zbx_queue_item_
 static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 {
 	int			ret = FAIL, request_type = ZBX_GET_QUEUE_UNKNOWN, now, i, limit;
-	char			type[MAX_STRING_LEN], sessionid[MAX_STRING_LEN], limit_str[MAX_STRING_LEN];
+	char			type[MAX_STRING_LEN], limit_str[MAX_STRING_LEN];
 	zbx_user_t		user;
 	zbx_vector_ptr_t	queue;
 	struct zbx_json		json;
@@ -329,8 +309,9 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
-	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_SID, sessionid, sizeof(sessionid), NULL) ||
-			SUCCEED != DBget_user_by_active_session(sessionid, &user) || USER_TYPE_SUPER_ADMIN > user.type)
+	zbx_user_init(&user);
+
+	if (FAIL == zbx_get_user_from_json(jp, &user, NULL) || USER_TYPE_SUPER_ADMIN > user.type)
 	{
 		zbx_send_response(sock, ret, "Permission denied.", CONFIG_TIMEOUT);
 		goto out;
@@ -350,8 +331,8 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 		{
 			request_type = ZBX_GET_QUEUE_DETAILS;
 
-			if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_LIMIT, limit_str, sizeof(limit_str), NULL) ||
-					FAIL == is_uint31(limit_str, &limit))
+			if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_LIMIT, limit_str, sizeof(limit_str),
+					NULL) || FAIL == is_uint31(limit_str, &limit))
 			{
 				zbx_send_response(sock, ret, "Unsupported limit value.", CONFIG_TIMEOUT);
 				goto out;
@@ -387,7 +368,8 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 				{
 					zbx_queue_stats_t	data = {.id = id};
 
-					stats = (zbx_queue_stats_t *)zbx_hashset_insert(&queue_stats, &data, sizeof(data));
+					stats = (zbx_queue_stats_t *)zbx_hashset_insert(&queue_stats, &data,
+							sizeof(data));
 				}
 				queue_stats_update(stats, now - item->nextcheck);
 			}
@@ -412,7 +394,8 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 				{
 					zbx_queue_stats_t	data = {.id = id};
 
-					stats = (zbx_queue_stats_t *)zbx_hashset_insert(&queue_stats, &data, sizeof(data));
+					stats = (zbx_queue_stats_t *)zbx_hashset_insert(&queue_stats, &data,
+							sizeof(data));
 				}
 				queue_stats_update(stats, now - item->nextcheck);
 			}
@@ -456,158 +439,13 @@ static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	ret = SUCCEED;
 out:
+	zbx_user_free(&user);
+
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
 
-/******************************************************************************
- *                                                                            *
- * Function: recv_alert_send                                                  *
- *                                                                            *
- * Purpose: process alert send request that is used to test media types       *
- *                                                                            *
- * Parameters:  sock  - [IN] the request socket                               *
- *              jp    - [IN] the request data                                 *
- *                                                                            *
- * Return value:  SUCCEED - processed successfully                            *
- *                FAIL - an error occurred                                    *
- *                                                                            *
- ******************************************************************************/
-static void	recv_alert_send(zbx_socket_t *sock, const struct zbx_json_parse *jp)
-{
-	DB_RESULT		result;
-	DB_ROW			row;
-	int			ret = FAIL, errcode;
-	char			tmp[ZBX_MAX_UINT64_LEN + 1], sessionid[MAX_STRING_LEN], *sendto = NULL, *subject = NULL,
-				*message = NULL, *error = NULL, *params = NULL, *value = NULL, *debug = NULL;
-	zbx_uint64_t		mediatypeid;
-	size_t			string_alloc;
-	struct zbx_json		json;
-	struct zbx_json_parse	jp_data, jp_params;
-	unsigned char		*data = NULL,smtp_security, smtp_verify_peer, smtp_verify_host,
-				smtp_authentication, content_type, *response = NULL;
-	zbx_uint32_t		size;
-	zbx_user_t		user;
-	unsigned short		smtp_port;
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
-
-	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
-
-	if (SUCCEED != zbx_json_value_by_name(jp, ZBX_PROTO_TAG_SID, sessionid, sizeof(sessionid), NULL) ||
-			SUCCEED != DBget_user_by_active_session(sessionid, &user) || USER_TYPE_SUPER_ADMIN > user.type)
-	{
-		error = zbx_strdup(NULL, "Permission denied.");
-		goto fail;
-	}
-
-	if (SUCCEED != zbx_json_brackets_by_name(jp, ZBX_PROTO_TAG_DATA, &jp_data))
-	{
-		error = zbx_dsprintf(NULL, "Cannot parse request tag: %s.", ZBX_PROTO_TAG_DATA);
-		goto fail;
-	}
-
-	if (SUCCEED != zbx_json_value_by_name(&jp_data, ZBX_PROTO_TAG_MEDIATYPEID, tmp, sizeof(tmp), NULL) ||
-			SUCCEED != is_uint64(tmp, &mediatypeid))
-	{
-		error = zbx_dsprintf(NULL, "Cannot parse request tag: %s.", ZBX_PROTO_TAG_MEDIATYPEID);
-		goto fail;
-	}
-
-	string_alloc = 0;
-	if (SUCCEED == zbx_json_value_by_name_dyn(&jp_data, ZBX_PROTO_TAG_SENDTO, &sendto, &string_alloc, NULL))
-		string_alloc = 0;
-	if (SUCCEED == zbx_json_value_by_name_dyn(&jp_data, ZBX_PROTO_TAG_SUBJECT, &subject, &string_alloc, NULL))
-		string_alloc = 0;
-	if (SUCCEED == zbx_json_value_by_name_dyn(&jp_data, ZBX_PROTO_TAG_MESSAGE, &message, &string_alloc, NULL))
-		string_alloc = 0;
-
-	if (SUCCEED == zbx_json_brackets_by_name(&jp_data, ZBX_PROTO_TAG_PARAMETERS, &jp_params))
-	{
-		size_t	string_offset = 0;
-
-		zbx_strncpy_alloc(&params, &string_alloc, &string_offset, jp_params.start,
-				jp_params.end - jp_params.start + 1);
-	}
-
-	result = DBselect("select type,smtp_server,smtp_helo,smtp_email,exec_path,gsm_modem,username,"
-				"passwd,smtp_port,smtp_security,smtp_verify_peer,smtp_verify_host,smtp_authentication,"
-				"exec_params,maxsessions,maxattempts,attempt_interval,content_type,script,timeout"
-			" from media_type"
-			" where mediatypeid=" ZBX_FS_UI64, mediatypeid);
-
-	if (NULL == (row = DBfetch(result)))
-	{
-		DBfree_result(result);
-		error = zbx_dsprintf(NULL, "Cannot find the specified media type.");
-		goto fail;
-	}
-
-	if (FAIL == is_ushort(row[8], &smtp_port))
-	{
-		DBfree_result(result);
-		error = zbx_dsprintf(NULL, "Invalid port value.");
-		goto fail;
-	}
-
-	ZBX_STR2UCHAR(smtp_security, row[9]);
-	ZBX_STR2UCHAR(smtp_verify_peer, row[10]);
-	ZBX_STR2UCHAR(smtp_verify_host, row[11]);
-	ZBX_STR2UCHAR(smtp_authentication, row[12]);
-	ZBX_STR2UCHAR(content_type, row[17]);
-
-	size = zbx_alerter_serialize_alert_send(&data, mediatypeid, atoi(row[0]), row[1], row[2], row[3], row[4],
-			row[5], row[6], row[7], smtp_port, smtp_security, smtp_verify_peer, smtp_verify_host,
-			smtp_authentication, row[13], atoi(row[14]), atoi(row[15]), row[16], content_type, row[18],
-			row[19], sendto, subject, message, params);
-
-	DBfree_result(result);
-
-	if (SUCCEED != zbx_ipc_async_exchange(ZBX_IPC_SERVICE_ALERTER, ZBX_IPC_ALERTER_ALERT, SEC_PER_MIN, data, size,
-			&response, &error))
-	{
-		goto fail;
-	}
-
-	zbx_alerter_deserialize_result(response, &value, &errcode, &error, &debug);
-	zbx_free(response);
-
-	if (SUCCEED == errcode)
-		ret = SUCCEED;
-fail:
-	zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, SUCCEED == ret ? ZBX_PROTO_VALUE_SUCCESS :
-				ZBX_PROTO_VALUE_FAILED, ZBX_JSON_TYPE_STRING);
-
-	if (SUCCEED == ret)
-	{
-		if (NULL != value)
-			zbx_json_addstring(&json, ZBX_PROTO_TAG_DATA, value, ZBX_JSON_TYPE_STRING);
-	}
-	else
-	{
-		if (NULL != error && '\0' != *error)
-			zbx_json_addstring(&json, ZBX_PROTO_TAG_INFO, error, ZBX_JSON_TYPE_STRING);
-	}
-
-	if (NULL != debug)
-		zbx_json_addraw(&json, "debug", debug);
-
-	(void)zbx_tcp_send(sock, json.buffer);
-
-	zbx_free(params);
-	zbx_free(message);
-	zbx_free(subject);
-	zbx_free(sendto);
-	zbx_free(data);
-	zbx_free(value);
-	zbx_free(error);
-	zbx_free(debug);
-	zbx_json_free(&json);
-
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
-}
-
 static int	DBget_template_count(zbx_uint64_t *count)
 {
 	DB_RESULT	result;
@@ -706,7 +544,7 @@ static void	zbx_status_counters_free(void)
 }
 
 const zbx_status_section_t	status_sections[] = {
-/*	{SECTION NAME,			NUMBER OF SECTION ENTRIES	SECTION ACCESS LEVEL	SECTION READYNESS, */
+/*	{SECTION NAME,			NUMBER OF SECTION ENTRIES	SECTION ACCESS LEVEL	SECTION READINESS, */
 /*		{                                                                                                  */
 /*			{ENTRY INFORMATION,		COUNTER TYPE,                                              */
 /*				{                                                                                  */
@@ -921,8 +759,6 @@ static void	status_stats_export(struct zbx_json *json, zbx_user_type_t access_le
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: recv_getstatus                                                   *
  *                                                                            *
  * Purpose: process status request                                            *
  *                                                                            *
@@ -941,13 +777,14 @@ static int	recv_getstatus(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	zbx_user_t	user;
 	int		ret = FAIL, request_type = ZBX_GET_STATUS_UNKNOWN;
-	char		type[MAX_STRING_LEN], sessionid[MAX_STRING_LEN];
+	char		type[MAX_STRING_LEN];
 	struct zbx_json	json;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
-	if (SUCCEED != zbx_json_value_by_name(jp, ZBX_PROTO_TAG_SID, sessionid, sizeof(sessionid), NULL) ||
-			SUCCEED != DBget_user_by_active_session(sessionid, &user))
+	zbx_user_init(&user);
+
+	if (FAIL == zbx_get_user_from_json(jp, &user, NULL))
 	{
 		zbx_send_response(sock, ret, "Permission denied.", CONFIG_TIMEOUT);
 		goto out;
@@ -976,12 +813,14 @@ static int	recv_getstatus(zbx_socket_t *sock, struct zbx_json_parse *jp)
 	switch (request_type)
 	{
 		case ZBX_GET_STATUS_PING:
-			zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS, ZBX_JSON_TYPE_STRING);
+			zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS,
+					ZBX_JSON_TYPE_STRING);
 			zbx_json_addobject(&json, ZBX_PROTO_TAG_DATA);
 			zbx_json_close(&json);
 			break;
 		case ZBX_GET_STATUS_FULL:
-			zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS, ZBX_JSON_TYPE_STRING);
+			zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS,
+					ZBX_JSON_TYPE_STRING);
 			zbx_json_addobject(&json, ZBX_PROTO_TAG_DATA);
 			status_stats_export(&json, user.type);
 			zbx_json_close(&json);
@@ -998,6 +837,8 @@ static int	recv_getstatus(zbx_socket_t *sock, struct zbx_json_parse *jp)
 
 	ret = SUCCEED;
 out:
+	zbx_user_free(&user);
+
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
@@ -1008,8 +849,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: send_internal_stats_json                                         *
  *                                                                            *
  * Purpose: process Zabbix stats request                                      *
  *                                                                            *
@@ -1110,7 +949,7 @@ static void	active_passive_misconfig(zbx_socket_t *sock)
 	zbx_free(msg);
 }
 
-static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
+static int	process_trap(zbx_socket_t *sock, char *s, ssize_t bytes_received, zbx_timespec_t *ts)
 {
 	int	ret = SUCCEED;
 
@@ -1131,32 +970,43 @@ static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
 			return FAIL;
 		}
 
-		if (SUCCEED == zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_REQUEST, value, sizeof(value), NULL))
+		if (SUCCEED != zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_REQUEST, value, sizeof(value), NULL))
+			return FAIL;
+
+		if (0 == strcmp(value, ZBX_PROTO_VALUE_PROXY_CONFIG))
 		{
-			if (0 == strcmp(value, ZBX_PROTO_VALUE_PROXY_CONFIG))
+			if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 			{
-				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
-				{
-					send_proxyconfig(sock, &jp);
-				}
-				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
-				{
-					zabbix_log(LOG_LEVEL_WARNING, "received configuration data from server"
-							" at \"%s\", datalen " ZBX_FS_SIZE_T,
-							sock->peer, (zbx_fs_size_t)(jp.end - jp.start + 1));
-					recv_proxyconfig(sock, &jp);
-				}
-				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_ACTIVE))
-				{
-					/* This is a misconfiguration: the proxy is configured in active mode */
-					/* but server sends requests to it as to a proxy in passive mode. To  */
-					/* prevent logging of this problem for every request we report it     */
-					/* only when the server sends configuration to the proxy and ignore   */
-					/* it for other requests.                                             */
-					active_passive_misconfig(sock);
-				}
+				send_proxyconfig(sock, &jp);
 			}
-			else if (0 == strcmp(value, ZBX_PROTO_VALUE_AGENT_DATA))
+			else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "received configuration data from server"
+						" at \"%s\", datalen " ZBX_FS_SIZE_T,
+						sock->peer, (zbx_fs_size_t)(jp.end - jp.start + 1));
+				recv_proxyconfig(sock, &jp);
+			}
+			else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_ACTIVE))
+			{
+				/* This is a misconfiguration: the proxy is configured in active mode */
+				/* but server sends requests to it as to a proxy in passive mode. To  */
+				/* prevent logging of this problem for every request we report it     */
+				/* only when the server sends configuration to the proxy and ignore   */
+				/* it for other requests.                                             */
+				active_passive_misconfig(sock);
+			}
+		}
+		else
+		{
+			if (ZBX_GIBIBYTE < bytes_received)
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "message size " ZBX_FS_I64 " exceeds the maximum size "
+						ZBX_FS_UI64 " for request \"%s\" received from \"%s\"", bytes_received,
+						(zbx_uint64_t)ZBX_GIBIBYTE, value, sock->peer);
+				return FAIL;
+			}
+
+			if (0 == strcmp(value, ZBX_PROTO_VALUE_AGENT_DATA))
 			{
 				recv_agenthistory(sock, &jp, ts);
 			}
@@ -1204,11 +1054,6 @@ static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
 			{
 				ret = send_internal_stats_json(sock, &jp);
 			}
-			else if (0 == strcmp(value, ZBX_PROTO_VALUE_ZABBIX_ALERT_SEND))
-			{
-				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
-					recv_alert_send(sock, &jp);
-			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_PREPROCESSING_TEST))
 			{
 				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
@@ -1224,8 +1069,11 @@ static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
 				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					zbx_trapper_item_test(sock, &jp);
 			}
-			else
-				zabbix_log(LOG_LEVEL_WARNING, "unknown request received from \"%s\": [%s]", sock->peer, value);
+			else if (SUCCEED != trapper_process_request(value, sock, &jp))
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "unknown request received from \"%s\": [%s]", sock->peer,
+					value);
+			}
 		}
 	}
 	else if (0 == strncmp(s, "ZBX_GET_ACTIVE_CHECKS", 21))	/* request for list of active checks */
@@ -1243,6 +1091,14 @@ static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
 		DC_ITEM			item;
 		int			errcode;
 
+		if (ZBX_GIBIBYTE < bytes_received)
+		{
+			zabbix_log(LOG_LEVEL_WARNING, "message size " ZBX_FS_I64 " exceeds the maximum size "
+					ZBX_FS_UI64 " for XML protocol received from \"%s\"", bytes_received,
+					(zbx_uint64_t)ZBX_GIBIBYTE, sock->peer);
+			return FAIL;
+		}
+
 		memset(&av, 0, sizeof(zbx_agent_value_t));
 
 		if ('<' == *s)	/* XML protocol */
@@ -1302,27 +1158,23 @@ static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
 
 static void	process_trapper_child(zbx_socket_t *sock, zbx_timespec_t *ts)
 {
-	if (SUCCEED != zbx_tcp_recv_to(sock, CONFIG_TRAPPER_TIMEOUT))
+	ssize_t	bytes_received;
+
+	if (FAIL == (bytes_received = zbx_tcp_recv_ext(sock, CONFIG_TRAPPER_TIMEOUT, ZBX_TCP_LARGE)))
 		return;
 
-	process_trap(sock, sock->buffer, ts);
-}
-
-static void	zbx_trapper_sigusr_handler(int flags)
-{
-#ifdef HAVE_NETSNMP
-	if (ZBX_RTC_SNMP_CACHE_RELOAD == ZBX_RTC_GET_MSG(flags))
-		snmp_cache_reload_requested = 1;
-#else
-	ZBX_UNUSED(flags);
-#endif
+	process_trap(sock, sock->buffer, bytes_received, ts);
 }
 
 ZBX_THREAD_ENTRY(trapper_thread, args)
 {
-	double		sec = 0.0;
-	zbx_socket_t	s;
-	int		ret;
+	double			sec = 0.0;
+	zbx_socket_t		s;
+	int			ret;
+
+#ifdef HAVE_NETSNMP
+	zbx_ipc_async_socket_t	rtc;
+#endif
 
 	process_type = ((zbx_thread_args_t *)args)->process_type;
 	server_num = ((zbx_thread_args_t *)args)->server_num;
@@ -1334,9 +1186,6 @@ ZBX_THREAD_ENTRY(trapper_thread, args)
 	update_selfmon_counter(ZBX_PROCESS_STATE_BUSY);
 
 	memcpy(&s, (zbx_socket_t *)((zbx_thread_args_t *)args)->args, sizeof(zbx_socket_t));
-#ifdef HAVE_NETSNMP
-	zbx_init_snmp();
-#endif
 
 #if defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
 	zbx_tls_init_child();
@@ -1346,24 +1195,18 @@ ZBX_THREAD_ENTRY(trapper_thread, args)
 
 	DBconnect(ZBX_DB_CONNECT_NORMAL);
 
-	/* configuration sync is performed by trappers on passive Zabbix proxy */
-	if (1 == process_num && 0 == CONFIG_CONFSYNCER_FORKS)
-	{
-		zbx_setproctitle("%s [syncing configuration]", get_process_type_string(process_type));
-		DCsync_configuration(ZBX_DBSYNC_INIT, NULL);
-	}
-
-	zbx_set_sigusr_handler(zbx_trapper_sigusr_handler);
+#ifdef HAVE_NETSNMP
+	zbx_rtc_subscribe(&rtc, process_type, process_num);
+#endif
 
 	while (ZBX_IS_RUNNING())
 	{
 #ifdef HAVE_NETSNMP
-		if (1 == snmp_cache_reload_requested)
-		{
-			zbx_clear_cache_snmp(process_type, process_num);
-			snmp_cache_reload_requested = 0;
-		}
+		zbx_uint32_t	rtc_cmd;
+		unsigned char	*rtc_data;
+		int		snmp_reload = 0;
 #endif
+
 		zbx_setproctitle("%s #%d [processed data in " ZBX_FS_DBL " sec, waiting for connection]",
 				get_process_type_string(process_type), process_num, sec);
 
@@ -1387,6 +1230,22 @@ ZBX_THREAD_ENTRY(trapper_thread, args)
 			zbx_setproctitle("%s #%d [processing data]", get_process_type_string(process_type),
 					process_num);
 
+#ifdef HAVE_NETSNMP
+			while (SUCCEED == zbx_rtc_wait(&rtc, &rtc_cmd, &rtc_data, 0) && 0 != rtc_cmd)
+			{
+				if (ZBX_RTC_SNMP_CACHE_RELOAD == rtc_cmd && 0 == snmp_reload)
+				{
+					zbx_clear_cache_snmp(process_type, process_num);
+					snmp_reload = 1;
+				}
+				else if (ZBX_RTC_SHUTDOWN == rtc_cmd)
+				{
+					zbx_tcp_unaccept(&s);
+					goto out;
+				}
+
+			}
+#endif
 			sec = zbx_time();
 			process_trapper_child(&s, &ts);
 			sec = zbx_time() - sec;
@@ -1399,7 +1258,9 @@ ZBX_THREAD_ENTRY(trapper_thread, args)
 					zbx_socket_strerror());
 		}
 	}
-
+#ifdef HAVE_NETSNMP
+out:
+#endif
 	zbx_setproctitle("%s #%d [terminated]", get_process_type_string(process_type), process_num);
 
 	while (1)
