@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -17,21 +17,17 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include "common.h"
-#include "log.h"
-#include "zbxjson.h"
-#include "zbxalgo.h"
-#include "preproc.h"
 #include "trapper_preproc.h"
+
+#include "preproc.h"
 #include "../preprocessor/preproc_history.h"
+#include "trapper_auth.h"
 
 #define ZBX_STATE_NOT_SUPPORTED	1
 
 extern int	CONFIG_DOUBLE_PRECISION;
 
 /******************************************************************************
- *                                                                            *
- * Function: trapper_parse_preproc_test                                       *
  *                                                                            *
  * Purpose: parses preprocessing test request                                 *
  *                                                                            *
@@ -42,7 +38,7 @@ extern int	CONFIG_DOUBLE_PRECISION;
  *             values_num   - [OUT] the number of values                      *
  *             value_type   - [OUT] the value type                            *
  *             steps        - [OUT] the preprocessing steps                   *
- *             single       - [OUT] the single tag                            *
+ *             single     - [OUT] is preproc step single                      *
  *             state        - [OUT] the item state                            *
  *             bypass_first - [OUT] the flag to bypass first step             *
  *             error      - [OUT] the error message                           *
@@ -63,8 +59,9 @@ static int	trapper_parse_preproc_test(const struct zbx_json_parse *jp, char **va
 	size_t			size;
 	zbx_timespec_t		ts_now;
 
-	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_SID, buffer, sizeof(buffer), NULL) ||
-			SUCCEED != DBget_user_by_active_session(buffer, &user) || USER_TYPE_ZABBIX_ADMIN > user.type)
+	zbx_user_init(&user);
+
+	if (FAIL == zbx_get_user_from_json(jp, &user, NULL) || USER_TYPE_ZABBIX_ADMIN > user.type)
 	{
 		*error = zbx_strdup(NULL, "Permission denied.");
 		goto out;
@@ -220,12 +217,12 @@ out:
 	zbx_free(step_params);
 	zbx_free(error_handler_params);
 
+	zbx_user_free(&user);
+
 	return ret;
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: trapper_preproc_test_run                                         *
  *                                                                            *
  * Purpose: executes preprocessing test request                               *
  *                                                                            *
@@ -395,8 +392,6 @@ out:
 }
 
 /******************************************************************************
- *                                                                            *
- * Function: zbx_trapper_preproc_test                                         *
  *                                                                            *
  * Purpose: processes preprocessing test request                              *
  *                                                                            *
